/* Generated by Cython 0.27.1 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_1"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #ifdef __cplusplus
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough) || (defined(__GNUC__) && defined(__attribute__))
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pyregion___region_filter
#define __PYX_HAVE_API__pyregion___region_filter
#include "stdio.h"
#include "stdlib.h"
#include "numpy/arrayobject.h"
#include "math.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "stringsource",
  "pyregion/_region_filter.pyx",
  "pyregion/c_numpy.pxd",
};

/* "pyregion/_region_filter.pyx":15
 * c_numpy.import_array()
 * 
 * ctypedef int Py_ssize_t             # <<<<<<<<<<<<<<
 * 
 * 
 */
typedef int __pyx_t_8pyregion_14_region_filter_Py_ssize_t;

/*--- Type declarations ---*/
struct __pyx_obj_8pyregion_14_region_filter_RegionContext;
struct __pyx_obj_8pyregion_14_region_filter_RegionBase;
struct __pyx_obj_8pyregion_14_region_filter_RegionNot;
struct __pyx_obj_8pyregion_14_region_filter_RegionList;
struct __pyx_obj_8pyregion_14_region_filter_RegionOrList;
struct __pyx_obj_8pyregion_14_region_filter_RegionAndList;
struct __pyx_obj_8pyregion_14_region_filter_Transform;
struct __pyx_obj_8pyregion_14_region_filter_Rotated;
struct __pyx_obj_8pyregion_14_region_filter_Translated;
struct __pyx_obj_8pyregion_14_region_filter_Circle;
struct __pyx_obj_8pyregion_14_region_filter_Ellipse;
struct __pyx_obj_8pyregion_14_region_filter_Box;
struct __pyx_obj_8pyregion_14_region_filter_Polygon;
struct __pyx_obj_8pyregion_14_region_filter_AngleRange;
struct __pyx_t_8pyregion_14_region_filter_Metric;
struct __pyx_t_8pyregion_14_region_filter__RegionContext;

/* "pyregion/_region_filter.pyx":26
 * 
 * 
 * cdef struct Metric:             # <<<<<<<<<<<<<<
 *     double x0
 *     double y0
 */
struct __pyx_t_8pyregion_14_region_filter_Metric {
  double x0;
  double y0;
  double g_x;
  double g_y;
};

/* "pyregion/_region_filter.pyx":36
 *     m.y0 = y0
 * 
 * cdef struct _RegionContext:             # <<<<<<<<<<<<<<
 *     int (*update_metric)(Metric *m)
 * 
 */
struct __pyx_t_8pyregion_14_region_filter__RegionContext {
  int (*update_metric)(struct __pyx_t_8pyregion_14_region_filter_Metric *);
};

/* "pyregion/_region_filter.pyx":39
 *     int (*update_metric)(Metric *m)
 * 
 * cdef class RegionContext:             # <<<<<<<<<<<<<<
 *     cdef _RegionContext c
 * 
 */
struct __pyx_obj_8pyregion_14_region_filter_RegionContext {
  PyObject_HEAD
  struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionContext *__pyx_vtab;
  struct __pyx_t_8pyregion_14_region_filter__RegionContext c;
};


/* "pyregion/_region_filter.pyx":68
 * 
 * 
 * cdef class RegionBase:             # <<<<<<<<<<<<<<
 *     cdef Metric m
 *     cdef RegionContext c
 */
struct __pyx_obj_8pyregion_14_region_filter_RegionBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *__pyx_vtab;
  struct __pyx_t_8pyregion_14_region_filter_Metric m;
  struct __pyx_obj_8pyregion_14_region_filter_RegionContext *c;
};


/* "pyregion/_region_filter.pyx":237
 *         return ra
 * 
 * cdef class RegionNot(RegionBase):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionNot(r2)
 */
struct __pyx_obj_8pyregion_14_region_filter_RegionNot {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase __pyx_base;
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *child_region;
};


/* "pyregion/_region_filter.pyx":249
 *         return not (self.child_region._inside(x, y))
 * 
 * cdef class RegionList(RegionBase):             # <<<<<<<<<<<<<<
 *     cdef object child_regions
 * 
 */
struct __pyx_obj_8pyregion_14_region_filter_RegionList {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase __pyx_base;
  PyObject *child_regions;
};


/* "pyregion/_region_filter.pyx":282
 *         return self.child_regions
 * 
 * cdef class RegionOrList(RegionList):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionOrList(r1, r2, r3, r4, ...)
 */
struct __pyx_obj_8pyregion_14_region_filter_RegionOrList {
  struct __pyx_obj_8pyregion_14_region_filter_RegionList __pyx_base;
};


/* "pyregion/_region_filter.pyx":300
 *         return "Or" + repr(self.child_regions)
 * 
 * cdef class RegionAndList(RegionList):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionAndList(r1, r2, r3, r4, ...)
 */
struct __pyx_obj_8pyregion_14_region_filter_RegionAndList {
  struct __pyx_obj_8pyregion_14_region_filter_RegionList __pyx_base;
};


/* "pyregion/_region_filter.pyx":351
 *     return RegionOrList(*(region1_list + region2_list))
 * 
 * cdef class Transform(RegionBase):             # <<<<<<<<<<<<<<
 *     cdef RegionBase child_region
 * 
 */
struct __pyx_obj_8pyregion_14_region_filter_Transform {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase __pyx_base;
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *child_region;
};


/* "pyregion/_region_filter.pyx":383
 * 
 * 
 * cdef class Rotated(Transform):             # <<<<<<<<<<<<<<
 *     """
 *     Rotate the region by degree in anti-colockwise direction.
 */
struct __pyx_obj_8pyregion_14_region_filter_Rotated {
  struct __pyx_obj_8pyregion_14_region_filter_Transform __pyx_base;
  double sin_theta;
  double cos_theta;
  double origin_x;
  double origin_y;
};


/* "pyregion/_region_filter.pyx":426
 *         yp[0] = y2 + oy
 * 
 * cdef class Translated(Transform):             # <<<<<<<<<<<<<<
 *     """
 *     Translated region.
 */
struct __pyx_obj_8pyregion_14_region_filter_Translated {
  struct __pyx_obj_8pyregion_14_region_filter_Transform __pyx_base;
  double dx;
  double dy;
};


/* "pyregion/_region_filter.pyx":449
 * # Basic Shapes
 * 
 * cdef class Circle(RegionBase):             # <<<<<<<<<<<<<<
 *     """
 *     Circle.
 */
struct __pyx_obj_8pyregion_14_region_filter_Circle {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase __pyx_base;
  double xc;
  double yc;
  double radius;
  double radius2;
};


/* "pyregion/_region_filter.pyx":484
 *         return "Circle(%f, %f, %f)" % (self.xc, self.yc, self.radius)
 * 
 * cdef class Ellipse(RegionBase):             # <<<<<<<<<<<<<<
 *     """
 *     Ellipse.
 */
struct __pyx_obj_8pyregion_14_region_filter_Ellipse {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase __pyx_base;
  double xc;
  double yc;
  double radius_major;
  double radius_major_2;
  double radius_minor;
  double radius_minor_2;
  double radius_major_2_radius_minor_2;
};


/* "pyregion/_region_filter.pyx":525
 *         return "Ellipse(%f, %f, %f, %f)" % (self.xc, self.yc, self.radius_major, self.radius_minor)
 * 
 * cdef class Box(RegionBase):             # <<<<<<<<<<<<<<
 *     """
 *     Box.
 */
struct __pyx_obj_8pyregion_14_region_filter_Box {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase __pyx_base;
  double x1;
  double x2;
  double y1;
  double y2;
};


/* "pyregion/_region_filter.pyx":555
 *         return (self.x1 <= x) & (x <= self.x2) & (self.y1 <= y) & (y <= self.y2)
 * 
 * cdef class Polygon(RegionBase):             # <<<<<<<<<<<<<<
 *     """
 *     Polygon.
 */
struct __pyx_obj_8pyregion_14_region_filter_Polygon {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase __pyx_base;
  PyArrayObject *xa;
  PyArrayObject *ya;
  double *x;
  double *y;
  int n;
};


/* "pyregion/_region_filter.pyx":616
 *         return r
 * 
 * cdef class AngleRange(RegionBase):             # <<<<<<<<<<<<<<
 *     """
 *     AngleRange.
 */
struct __pyx_obj_8pyregion_14_region_filter_AngleRange {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase __pyx_base;
  double xc;
  double yc;
  double degree1;
  double degree2;
  double radian1;
  double radian2;
};



/* "pyregion/_region_filter.pyx":39
 *     int (*update_metric)(Metric *m)
 * 
 * cdef class RegionContext:             # <<<<<<<<<<<<<<
 *     cdef _RegionContext c
 * 
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionContext {
  PyObject *(*set_update_func)(struct __pyx_obj_8pyregion_14_region_filter_RegionContext *, int (*)(struct __pyx_t_8pyregion_14_region_filter_Metric *));
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionContext *__pyx_vtabptr_8pyregion_14_region_filter_RegionContext;


/* "pyregion/_region_filter.pyx":68
 * 
 * 
 * cdef class RegionBase:             # <<<<<<<<<<<<<<
 *     cdef Metric m
 *     cdef RegionContext c
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase {
  PyObject *(*update_metric)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *);
  PyObject *(*metric_set_origin)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *);
  npy_bool (*_inside)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double);
  PyArrayObject *(*_mask)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, npy_intp, npy_intp);
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *__pyx_vtabptr_8pyregion_14_region_filter_RegionBase;


/* "pyregion/_region_filter.pyx":237
 *         return ra
 * 
 * cdef class RegionNot(RegionBase):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionNot(r2)
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionNot {
  struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase __pyx_base;
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionNot *__pyx_vtabptr_8pyregion_14_region_filter_RegionNot;


/* "pyregion/_region_filter.pyx":249
 *         return not (self.child_region._inside(x, y))
 * 
 * cdef class RegionList(RegionBase):             # <<<<<<<<<<<<<<
 *     cdef object child_regions
 * 
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionList {
  struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase __pyx_base;
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionList *__pyx_vtabptr_8pyregion_14_region_filter_RegionList;


/* "pyregion/_region_filter.pyx":282
 *         return self.child_regions
 * 
 * cdef class RegionOrList(RegionList):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionOrList(r1, r2, r3, r4, ...)
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionOrList {
  struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionList __pyx_base;
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionOrList *__pyx_vtabptr_8pyregion_14_region_filter_RegionOrList;


/* "pyregion/_region_filter.pyx":300
 *         return "Or" + repr(self.child_regions)
 * 
 * cdef class RegionAndList(RegionList):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionAndList(r1, r2, r3, r4, ...)
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionAndList {
  struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionList __pyx_base;
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionAndList *__pyx_vtabptr_8pyregion_14_region_filter_RegionAndList;


/* "pyregion/_region_filter.pyx":351
 *     return RegionOrList(*(region1_list + region2_list))
 * 
 * cdef class Transform(RegionBase):             # <<<<<<<<<<<<<<
 *     cdef RegionBase child_region
 * 
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_Transform {
  struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase __pyx_base;
  int (*_transform)(struct __pyx_obj_8pyregion_14_region_filter_Transform *, double, double, double *, double *);
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Transform *__pyx_vtabptr_8pyregion_14_region_filter_Transform;


/* "pyregion/_region_filter.pyx":383
 * 
 * 
 * cdef class Rotated(Transform):             # <<<<<<<<<<<<<<
 *     """
 *     Rotate the region by degree in anti-colockwise direction.
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_Rotated {
  struct __pyx_vtabstruct_8pyregion_14_region_filter_Transform __pyx_base;
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Rotated *__pyx_vtabptr_8pyregion_14_region_filter_Rotated;


/* "pyregion/_region_filter.pyx":426
 *         yp[0] = y2 + oy
 * 
 * cdef class Translated(Transform):             # <<<<<<<<<<<<<<
 *     """
 *     Translated region.
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_Translated {
  struct __pyx_vtabstruct_8pyregion_14_region_filter_Transform __pyx_base;
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Translated *__pyx_vtabptr_8pyregion_14_region_filter_Translated;


/* "pyregion/_region_filter.pyx":449
 * # Basic Shapes
 * 
 * cdef class Circle(RegionBase):             # <<<<<<<<<<<<<<
 *     """
 *     Circle.
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_Circle {
  struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase __pyx_base;
  PyObject *(*_set_v)(struct __pyx_obj_8pyregion_14_region_filter_Circle *, double, double, double);
  PyObject *(*_get_v)(struct __pyx_obj_8pyregion_14_region_filter_Circle *);
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Circle *__pyx_vtabptr_8pyregion_14_region_filter_Circle;


/* "pyregion/_region_filter.pyx":484
 *         return "Circle(%f, %f, %f)" % (self.xc, self.yc, self.radius)
 * 
 * cdef class Ellipse(RegionBase):             # <<<<<<<<<<<<<<
 *     """
 *     Ellipse.
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_Ellipse {
  struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase __pyx_base;
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Ellipse *__pyx_vtabptr_8pyregion_14_region_filter_Ellipse;


/* "pyregion/_region_filter.pyx":525
 *         return "Ellipse(%f, %f, %f, %f)" % (self.xc, self.yc, self.radius_major, self.radius_minor)
 * 
 * cdef class Box(RegionBase):             # <<<<<<<<<<<<<<
 *     """
 *     Box.
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_Box {
  struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase __pyx_base;
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Box *__pyx_vtabptr_8pyregion_14_region_filter_Box;


/* "pyregion/_region_filter.pyx":555
 *         return (self.x1 <= x) & (x <= self.x2) & (self.y1 <= y) & (y <= self.y2)
 * 
 * cdef class Polygon(RegionBase):             # <<<<<<<<<<<<<<
 *     """
 *     Polygon.
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_Polygon {
  struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase __pyx_base;
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Polygon *__pyx_vtabptr_8pyregion_14_region_filter_Polygon;


/* "pyregion/_region_filter.pyx":616
 *         return r
 * 
 * cdef class AngleRange(RegionBase):             # <<<<<<<<<<<<<<
 *     """
 *     AngleRange.
 */

struct __pyx_vtabstruct_8pyregion_14_region_filter_AngleRange {
  struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase __pyx_base;
  double (*_fix_angle)(struct __pyx_obj_8pyregion_14_region_filter_AngleRange *, double);
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_AngleRange *__pyx_vtabptr_8pyregion_14_region_filter_AngleRange;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_EqObjC(op1, op2, intval, inplace)\
    PyObject_RichCompare(op1, op2, Py_EQ)
    #endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* DelItemInt.proto */
#define __Pyx_DelItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_DelItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_DelItem_Generic(o, to_py_func(i))))
static int __Pyx_DelItem_Generic(PyObject *o, PyObject *j);
static CYTHON_INLINE int __Pyx_DelItemInt_Fast(PyObject *o, Py_ssize_t i,
                                               int is_list, int wraparound);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
static int __Pyx_CLineForTraceback(int c_line);

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_bool(npy_bool value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_8pyregion_14_region_filter_13RegionContext_set_update_func(struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_self, int (*__pyx_v_update_metric)(struct __pyx_t_8pyregion_14_region_filter_Metric *)); /* proto*/
static PyObject *__pyx_f_8pyregion_14_region_filter_10RegionBase_update_metric(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_8pyregion_14_region_filter_10RegionBase_metric_set_origin(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, double __pyx_v_xc, double __pyx_v_yc, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_cnt); /* proto*/
static npy_bool __pyx_f_8pyregion_14_region_filter_10RegionBase__inside(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, CYTHON_UNUSED double __pyx_v_x, CYTHON_UNUSED double __pyx_v_y); /* proto*/
static PyArrayObject *__pyx_f_8pyregion_14_region_filter_10RegionBase__mask(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, npy_intp __pyx_v_nx, npy_intp __pyx_v_ny); /* proto*/
static npy_bool __pyx_f_8pyregion_14_region_filter_9RegionNot__inside(struct __pyx_obj_8pyregion_14_region_filter_RegionNot *__pyx_v_self, double __pyx_v_x, double __pyx_v_y); /* proto*/
static npy_bool __pyx_f_8pyregion_14_region_filter_12RegionOrList__inside(struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *__pyx_v_self, double __pyx_v_x, double __pyx_v_y); /* proto*/
static npy_bool __pyx_f_8pyregion_14_region_filter_13RegionAndList__inside(struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *__pyx_v_self, double __pyx_v_x, double __pyx_v_y); /* proto*/
static int __pyx_f_8pyregion_14_region_filter_9Transform__transform(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Transform *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double *__pyx_v_xp, double *__pyx_v_yp); /* proto*/
static npy_bool __pyx_f_8pyregion_14_region_filter_9Transform__inside(struct __pyx_obj_8pyregion_14_region_filter_Transform *__pyx_v_self, double __pyx_v_x, double __pyx_v_y); /* proto*/
static int __pyx_f_8pyregion_14_region_filter_7Rotated__transform(struct __pyx_obj_8pyregion_14_region_filter_Rotated *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double *__pyx_v_xp, double *__pyx_v_yp); /* proto*/
static int __pyx_f_8pyregion_14_region_filter_10Translated__transform(struct __pyx_obj_8pyregion_14_region_filter_Translated *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double *__pyx_v_xp, double *__pyx_v_yp); /* proto*/
static PyObject *__pyx_f_8pyregion_14_region_filter_6Circle__set_v(struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self, double __pyx_v_xc, double __pyx_v_yc, double __pyx_v_radius); /* proto*/
static PyObject *__pyx_f_8pyregion_14_region_filter_6Circle__get_v(struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self); /* proto*/
static npy_bool __pyx_f_8pyregion_14_region_filter_6Circle__inside(struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self, double __pyx_v_x, double __pyx_v_y); /* proto*/
static npy_bool __pyx_f_8pyregion_14_region_filter_7Ellipse__inside(struct __pyx_obj_8pyregion_14_region_filter_Ellipse *__pyx_v_self, double __pyx_v_x, double __pyx_v_y); /* proto*/
static npy_bool __pyx_f_8pyregion_14_region_filter_3Box__inside(struct __pyx_obj_8pyregion_14_region_filter_Box *__pyx_v_self, double __pyx_v_x, double __pyx_v_y); /* proto*/
static npy_bool __pyx_f_8pyregion_14_region_filter_7Polygon__inside(struct __pyx_obj_8pyregion_14_region_filter_Polygon *__pyx_v_self, double __pyx_v_x, double __pyx_v_y); /* proto*/
static double __pyx_f_8pyregion_14_region_filter_10AngleRange__fix_angle(struct __pyx_obj_8pyregion_14_region_filter_AngleRange *__pyx_v_self, double __pyx_v_a); /* proto*/
static npy_bool __pyx_f_8pyregion_14_region_filter_10AngleRange__inside(struct __pyx_obj_8pyregion_14_region_filter_AngleRange *__pyx_v_self, double __pyx_v_x, double __pyx_v_y); /* proto*/

/* Module declarations from 'numpy' */

/* Module declarations from 'pyregion.c_numpy' */
static PyTypeObject *__pyx_ptype_8pyregion_7c_numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_8pyregion_7c_numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_8pyregion_7c_numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_8pyregion_7c_numpy_broadcast = 0;

/* Module declarations from 'pyregion.c_python' */

/* Module declarations from 'pyregion._region_filter' */
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_RegionContext = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_RegionBase = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_RegionNot = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_RegionList = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_RegionOrList = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_RegionAndList = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_Transform = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_Rotated = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_Translated = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_Circle = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_Ellipse = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_Box = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_Polygon = 0;
static PyTypeObject *__pyx_ptype_8pyregion_14_region_filter_AngleRange = 0;
static struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_8pyregion_14_region_filter_metric_wcs = 0;
static int __pyx_f_8pyregion_14_region_filter_MetricSetOrigin(struct __pyx_t_8pyregion_14_region_filter_Metric *, double, double); /*proto*/
static int __pyx_f_8pyregion_14_region_filter__update_metric_default(struct __pyx_t_8pyregion_14_region_filter_Metric *); /*proto*/
static int __pyx_f_8pyregion_14_region_filter__update_metric_wcs(struct __pyx_t_8pyregion_14_region_filter_Metric *); /*proto*/
#define __Pyx_MODULE_NAME "pyregion._region_filter"
int __pyx_module_is_main_pyregion___region_filter = 0;

/* Implementation of 'pyregion._region_filter' */
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_ValueError;
static const char __pyx_k_c[] = "c";
static const char __pyx_k_x[] = "x";
static const char __pyx_k_y[] = "y";
static const char __pyx_k_Or[] = "Or";
static const char __pyx_k_dx[] = "dx";
static const char __pyx_k_dy[] = "dy";
static const char __pyx_k_xc[] = "xc";
static const char __pyx_k_yc[] = "yc";
static const char __pyx_k_And[] = "And";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_width[] = "width";
static const char __pyx_k_asList[] = "asList";
static const char __pyx_k_degree[] = "degree";
static const char __pyx_k_height[] = "height";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_radius[] = "radius";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_degree1[] = "degree1";
static const char __pyx_k_degree2[] = "degree2";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_region1[] = "region1";
static const char __pyx_k_region2[] = "region2";
static const char __pyx_k_RegionOr[] = "RegionOr";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_origin_x[] = "origin_x";
static const char __pyx_k_origin_y[] = "origin_y";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_RegionAnd[] = "RegionAnd";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_inside_xy[] = "inside_xy";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_inside_x_y[] = "inside_x_y";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_set_context[] = "set_context";
static const char __pyx_k_Circle_f_f_f[] = "Circle(%f, %f, %f)";
static const char __pyx_k_child_region[] = "child_region";
static const char __pyx_k_radius_major[] = "radius_major";
static const char __pyx_k_radius_minor[] = "radius_minor";
static const char __pyx_k_region1_list[] = "region1_list";
static const char __pyx_k_region2_list[] = "region2_list";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_Ellipse_f_f_f_f[] = "Ellipse(%f, %f, %f, %f)";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_NotYetImplemented[] = "NotYetImplemented";
static const char __pyx_k_RegionOr_line_335[] = "RegionOr (line 335)";
static const char __pyx_k_AngleRange_f_f_f_f[] = "AngleRange(%f, %f, %f, %f)";
static const char __pyx_k_RegionAnd_line_319[] = "RegionAnd (line 319)";
static const char __pyx_k_check_type_of_list[] = "_check_type_of_list";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_r_RegionOr_reg1_reg2[] = "\n    >>> r = RegionOr(reg1, reg2)\n    ";
static const char __pyx_k_RegionFilterException[] = "RegionFilterException";
static const char __pyx_k_r_RegionAnd_reg1_reg2[] = "\n    >>> r = RegionAnd(reg1, reg2)\n    ";
static const char __pyx_k_BaseClassInitException[] = "BaseClassInitException";
static const char __pyx_k_pyregion__region_filter[] = "pyregion._region_filter";
static const char __pyx_k_pyregion__region_filter_pyx[] = "pyregion/_region_filter.pyx";
static const char __pyx_k_input_array_has_a_wrong_shape[] = "input array has a wrong shape";
static const char __pyx_k_All_elements_should_be_subclass[] = "All elements should be subclass of RegionBase type: %s";
static const char __pyx_k_Pickling_of_struct_members_such[] = "Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)";
static const char __pyx_k_self_c_cannot_be_converted_to_a[] = "self.c cannot be converted to a Python object for pickling";
static const char __pyx_k_self_x_self_y_cannot_be_converte[] = "self.x,self.y cannot be converted to a Python object for pickling";
static const char __pyx_k_shape_of_the_input_image_must_be[] = "shape of the input image must be 2d: %s is given";
static const char __pyx_k_the_inut_needs_to_be_a_numpy_2_d[] = "the inut needs to be a numpy 2-d array or a tuple of two integers";
static PyObject *__pyx_kp_s_All_elements_should_be_subclass;
static PyObject *__pyx_n_s_And;
static PyObject *__pyx_kp_s_AngleRange_f_f_f_f;
static PyObject *__pyx_n_s_BaseClassInitException;
static PyObject *__pyx_kp_s_Circle_f_f_f;
static PyObject *__pyx_kp_s_Ellipse_f_f_f_f;
static PyObject *__pyx_n_s_NotYetImplemented;
static PyObject *__pyx_n_s_Or;
static PyObject *__pyx_kp_s_Pickling_of_struct_members_such;
static PyObject *__pyx_n_s_RegionAnd;
static PyObject *__pyx_kp_u_RegionAnd_line_319;
static PyObject *__pyx_n_s_RegionFilterException;
static PyObject *__pyx_n_s_RegionOr;
static PyObject *__pyx_kp_u_RegionOr_line_335;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_asList;
static PyObject *__pyx_n_s_c;
static PyObject *__pyx_n_s_check_type_of_list;
static PyObject *__pyx_n_s_child_region;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_degree;
static PyObject *__pyx_n_s_degree1;
static PyObject *__pyx_n_s_degree2;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_dx;
static PyObject *__pyx_n_s_dy;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_height;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_kp_s_input_array_has_a_wrong_shape;
static PyObject *__pyx_n_s_inside_x_y;
static PyObject *__pyx_n_s_inside_xy;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_origin_x;
static PyObject *__pyx_n_s_origin_y;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_pyregion__region_filter;
static PyObject *__pyx_kp_s_pyregion__region_filter_pyx;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_kp_u_r_RegionAnd_reg1_reg2;
static PyObject *__pyx_kp_u_r_RegionOr_reg1_reg2;
static PyObject *__pyx_n_s_radius;
static PyObject *__pyx_n_s_radius_major;
static PyObject *__pyx_n_s_radius_minor;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_region1;
static PyObject *__pyx_n_s_region1_list;
static PyObject *__pyx_n_s_region2;
static PyObject *__pyx_n_s_region2_list;
static PyObject *__pyx_kp_s_self_c_cannot_be_converted_to_a;
static PyObject *__pyx_kp_s_self_x_self_y_cannot_be_converte;
static PyObject *__pyx_n_s_set_context;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_kp_s_shape_of_the_input_image_must_be;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_kp_s_the_inut_needs_to_be_a_numpy_2_d;
static PyObject *__pyx_n_s_width;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_xc;
static PyObject *__pyx_n_s_y;
static PyObject *__pyx_n_s_yc;
static int __pyx_pf_8pyregion_14_region_filter_13RegionContext___init__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_13RegionContext_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_13RegionContext_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_10RegionBase___init__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_2set_context(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_cnt); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_4__invert__(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_6__and__(PyObject *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_o); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_8__or__(PyObject *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_o); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_10mask(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, PyObject *__pyx_v_img_or_shape); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_12inside1(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, double __pyx_v_x, double __pyx_v_y); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_14inside(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_16inside_xy(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, PyObject *__pyx_v_xy); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_18inside_x_y(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_20__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_22__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_9RegionNot___init__(struct __pyx_obj_8pyregion_14_region_filter_RegionNot *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_child_region); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_9RegionNot_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionNot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_9RegionNot_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionNot *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionList__check_type_of_list(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, PyObject *__pyx_v_kl); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_10RegionList_2__init__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, PyObject *__pyx_v_kl); /* proto */
static Py_ssize_t __pyx_pf_8pyregion_14_region_filter_10RegionList_4__len__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionList_6__getitem__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, Py_ssize_t __pyx_v_x); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_10RegionList_8__setitem__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, Py_ssize_t __pyx_v_x, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_y); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_10RegionList_10__delitem__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, Py_ssize_t __pyx_v_x); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_10RegionList_12__contains__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionList_14__repr__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionList_16asList(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionList_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionList_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_12RegionOrList___repr__(struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_12RegionOrList_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_12RegionOrList_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_13RegionAndList___repr__(struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_13RegionAndList_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_13RegionAndList_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_RegionAnd(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_region1, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_region2); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_2RegionOr(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_region1, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_region2); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_9Transform___init__(struct __pyx_obj_8pyregion_14_region_filter_Transform *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_child_region); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_9Transform_5child___get__(struct __pyx_obj_8pyregion_14_region_filter_Transform *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_9Transform_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Transform *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_9Transform_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Transform *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_7Rotated___init__(struct __pyx_obj_8pyregion_14_region_filter_Rotated *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_child_region, double __pyx_v_degree, double __pyx_v_origin_x, double __pyx_v_origin_y); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_7Rotated_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Rotated *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_7Rotated_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Rotated *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_10Translated___init__(struct __pyx_obj_8pyregion_14_region_filter_Translated *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_child_region, double __pyx_v_dx, double __pyx_v_dy); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10Translated_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Translated *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10Translated_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Translated *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_6Circle___init__(struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self, double __pyx_v_xc, double __pyx_v_yc, double __pyx_v_radius, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_6Circle_2__repr__(struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_6Circle_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_6Circle_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_7Ellipse___init__(struct __pyx_obj_8pyregion_14_region_filter_Ellipse *__pyx_v_self, double __pyx_v_xc, double __pyx_v_yc, double __pyx_v_radius_major, double __pyx_v_radius_minor, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_7Ellipse_2__repr__(struct __pyx_obj_8pyregion_14_region_filter_Ellipse *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_7Ellipse_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Ellipse *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_7Ellipse_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Ellipse *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_3Box___init__(struct __pyx_obj_8pyregion_14_region_filter_Box *__pyx_v_self, double __pyx_v_xc, double __pyx_v_yc, double __pyx_v_width, double __pyx_v_height, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_3Box_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Box *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_3Box_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Box *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_7Polygon___init__(struct __pyx_obj_8pyregion_14_region_filter_Polygon *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_7Polygon_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Polygon *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_7Polygon_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Polygon *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_8pyregion_14_region_filter_10AngleRange___init__(struct __pyx_obj_8pyregion_14_region_filter_AngleRange *__pyx_v_self, double __pyx_v_xc, double __pyx_v_yc, double __pyx_v_degree1, double __pyx_v_degree2, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10AngleRange_2__repr__(struct __pyx_obj_8pyregion_14_region_filter_AngleRange *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10AngleRange_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_AngleRange *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8pyregion_14_region_filter_10AngleRange_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_AngleRange *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_RegionContext(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_RegionBase(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_RegionNot(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_RegionList(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_RegionOrList(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_RegionAndList(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Transform(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Rotated(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Translated(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Circle(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Ellipse(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Box(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Polygon(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_8pyregion_14_region_filter_AngleRange(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_2;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_codeobj__32;
static PyObject *__pyx_codeobj__34;

/* "pyregion/_region_filter.pyx":32
 *     double g_y
 * 
 * cdef int MetricSetOrigin(Metric *m, double x0, double y0):             # <<<<<<<<<<<<<<
 *     m.x0 = x0
 *     m.y0 = y0
 */

static int __pyx_f_8pyregion_14_region_filter_MetricSetOrigin(struct __pyx_t_8pyregion_14_region_filter_Metric *__pyx_v_m, double __pyx_v_x0, double __pyx_v_y0) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("MetricSetOrigin", 0);

  /* "pyregion/_region_filter.pyx":33
 * 
 * cdef int MetricSetOrigin(Metric *m, double x0, double y0):
 *     m.x0 = x0             # <<<<<<<<<<<<<<
 *     m.y0 = y0
 * 
 */
  __pyx_v_m->x0 = __pyx_v_x0;

  /* "pyregion/_region_filter.pyx":34
 * cdef int MetricSetOrigin(Metric *m, double x0, double y0):
 *     m.x0 = x0
 *     m.y0 = y0             # <<<<<<<<<<<<<<
 * 
 * cdef struct _RegionContext:
 */
  __pyx_v_m->y0 = __pyx_v_y0;

  /* "pyregion/_region_filter.pyx":32
 *     double g_y
 * 
 * cdef int MetricSetOrigin(Metric *m, double x0, double y0):             # <<<<<<<<<<<<<<
 *     m.x0 = x0
 *     m.y0 = y0
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":42
 *     cdef _RegionContext c
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_13RegionContext_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_13RegionContext_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_13RegionContext___init__(((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_13RegionContext___init__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":45
 *         pass
 * 
 *     cdef set_update_func(self, int (*update_metric)(Metric *m)):             # <<<<<<<<<<<<<<
 *         self.c.update_metric = update_metric
 * 
 */

static PyObject *__pyx_f_8pyregion_14_region_filter_13RegionContext_set_update_func(struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_self, int (*__pyx_v_update_metric)(struct __pyx_t_8pyregion_14_region_filter_Metric *)) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_update_func", 0);

  /* "pyregion/_region_filter.pyx":46
 * 
 *     cdef set_update_func(self, int (*update_metric)(Metric *m)):
 *         self.c.update_metric = update_metric             # <<<<<<<<<<<<<<
 * 
 * cdef int _update_metric_default(Metric *m):
 */
  __pyx_v_self->c.update_metric = __pyx_v_update_metric;

  /* "pyregion/_region_filter.pyx":45
 *         pass
 * 
 *     cdef set_update_func(self, int (*update_metric)(Metric *m)):             # <<<<<<<<<<<<<<
 *         self.c.update_metric = update_metric
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_13RegionContext_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_13RegionContext_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_13RegionContext_2__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_13RegionContext_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionContext.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_13RegionContext_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_13RegionContext_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_13RegionContext_4__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_13RegionContext_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionContext.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":48
 *         self.c.update_metric = update_metric
 * 
 * cdef int _update_metric_default(Metric *m):             # <<<<<<<<<<<<<<
 *     m.g_x = 1.
 *     m.g_y = 1.
 */

static int __pyx_f_8pyregion_14_region_filter__update_metric_default(struct __pyx_t_8pyregion_14_region_filter_Metric *__pyx_v_m) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_update_metric_default", 0);

  /* "pyregion/_region_filter.pyx":49
 * 
 * cdef int _update_metric_default(Metric *m):
 *     m.g_x = 1.             # <<<<<<<<<<<<<<
 *     m.g_y = 1.
 * 
 */
  __pyx_v_m->g_x = 1.;

  /* "pyregion/_region_filter.pyx":50
 * cdef int _update_metric_default(Metric *m):
 *     m.g_x = 1.
 *     m.g_y = 1.             # <<<<<<<<<<<<<<
 * 
 * cdef int _update_metric_wcs(Metric *m):
 */
  __pyx_v_m->g_y = 1.;

  /* "pyregion/_region_filter.pyx":48
 *         self.c.update_metric = update_metric
 * 
 * cdef int _update_metric_default(Metric *m):             # <<<<<<<<<<<<<<
 *     m.g_x = 1.
 *     m.g_y = 1.
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":52
 *     m.g_y = 1.
 * 
 * cdef int _update_metric_wcs(Metric *m):             # <<<<<<<<<<<<<<
 *     cdef double theta
 *     theta = m.y0 / 180. * 3.1415926;
 */

static int __pyx_f_8pyregion_14_region_filter__update_metric_wcs(struct __pyx_t_8pyregion_14_region_filter_Metric *__pyx_v_m) {
  double __pyx_v_theta;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_update_metric_wcs", 0);

  /* "pyregion/_region_filter.pyx":54
 * cdef int _update_metric_wcs(Metric *m):
 *     cdef double theta
 *     theta = m.y0 / 180. * 3.1415926;             # <<<<<<<<<<<<<<
 * 
 *     m.g_x = cos(theta)
 */
  __pyx_v_theta = ((__pyx_v_m->y0 / 180.) * 3.1415926);

  /* "pyregion/_region_filter.pyx":56
 *     theta = m.y0 / 180. * 3.1415926;
 * 
 *     m.g_x = cos(theta)             # <<<<<<<<<<<<<<
 *     m.g_y = 1.
 * 
 */
  __pyx_v_m->g_x = cos(__pyx_v_theta);

  /* "pyregion/_region_filter.pyx":57
 * 
 *     m.g_x = cos(theta)
 *     m.g_y = 1.             # <<<<<<<<<<<<<<
 * 
 * cdef RegionContext metric_wcs
 */
  __pyx_v_m->g_y = 1.;

  /* "pyregion/_region_filter.pyx":52
 *     m.g_y = 1.
 * 
 * cdef int _update_metric_wcs(Metric *m):             # <<<<<<<<<<<<<<
 *     cdef double theta
 *     theta = m.y0 / 180. * 3.1415926;
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":72
 *     cdef RegionContext c
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         raise BaseClassInitException()
 * 
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_10RegionBase_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_10RegionBase_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionBase___init__(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_10RegionBase___init__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyregion/_region_filter.pyx":73
 * 
 *     def __init__(self):
 *         raise BaseClassInitException()             # <<<<<<<<<<<<<<
 * 
 *     cdef update_metric(self):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_BaseClassInitException); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 73, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 73, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 73, __pyx_L1_error)

  /* "pyregion/_region_filter.pyx":72
 *     cdef RegionContext c
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         raise BaseClassInitException()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":75
 *         raise BaseClassInitException()
 * 
 *     cdef update_metric(self):             # <<<<<<<<<<<<<<
 *         if self.c:
 *             self.c.c.update_metric(&(self.m))
 */

static PyObject *__pyx_f_8pyregion_14_region_filter_10RegionBase_update_metric(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("update_metric", 0);

  /* "pyregion/_region_filter.pyx":76
 * 
 *     cdef update_metric(self):
 *         if self.c:             # <<<<<<<<<<<<<<
 *             self.c.c.update_metric(&(self.m))
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self->c)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(1, 76, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "pyregion/_region_filter.pyx":77
 *     cdef update_metric(self):
 *         if self.c:
 *             self.c.c.update_metric(&(self.m))             # <<<<<<<<<<<<<<
 *         else:
 *             _update_metric_default(&(self.m))
 */
    __pyx_v_self->c->c.update_metric((&__pyx_v_self->m));

    /* "pyregion/_region_filter.pyx":76
 * 
 *     cdef update_metric(self):
 *         if self.c:             # <<<<<<<<<<<<<<
 *             self.c.c.update_metric(&(self.m))
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pyregion/_region_filter.pyx":79
 *             self.c.c.update_metric(&(self.m))
 *         else:
 *             _update_metric_default(&(self.m))             # <<<<<<<<<<<<<<
 * 
 *     def set_context(self, RegionContext cnt):
 */
  /*else*/ {
    __pyx_f_8pyregion_14_region_filter__update_metric_default((&__pyx_v_self->m));
  }
  __pyx_L3:;

  /* "pyregion/_region_filter.pyx":75
 *         raise BaseClassInitException()
 * 
 *     cdef update_metric(self):             # <<<<<<<<<<<<<<
 *         if self.c:
 *             self.c.c.update_metric(&(self.m))
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.update_metric", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":81
 *             _update_metric_default(&(self.m))
 * 
 *     def set_context(self, RegionContext cnt):             # <<<<<<<<<<<<<<
 *         self.c = cnt
 *         self.update_metric()
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_3set_context(PyObject *__pyx_v_self, PyObject *__pyx_v_cnt); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_3set_context(PyObject *__pyx_v_self, PyObject *__pyx_v_cnt) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_context (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_cnt), __pyx_ptype_8pyregion_14_region_filter_RegionContext, 1, "cnt", 0))) __PYX_ERR(1, 81, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionBase_2set_context(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self), ((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)__pyx_v_cnt));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_2set_context(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_cnt) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("set_context", 0);

  /* "pyregion/_region_filter.pyx":82
 * 
 *     def set_context(self, RegionContext cnt):
 *         self.c = cnt             # <<<<<<<<<<<<<<
 *         self.update_metric()
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_cnt));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_cnt));
  __Pyx_GOTREF(__pyx_v_self->c);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->c));
  __pyx_v_self->c = __pyx_v_cnt;

  /* "pyregion/_region_filter.pyx":83
 *     def set_context(self, RegionContext cnt):
 *         self.c = cnt
 *         self.update_metric()             # <<<<<<<<<<<<<<
 * 
 *     cdef metric_set_origin(self, double xc, double yc,
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *)__pyx_v_self->__pyx_vtab)->update_metric(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":81
 *             _update_metric_default(&(self.m))
 * 
 *     def set_context(self, RegionContext cnt):             # <<<<<<<<<<<<<<
 *         self.c = cnt
 *         self.update_metric()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.set_context", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":85
 *         self.update_metric()
 * 
 *     cdef metric_set_origin(self, double xc, double yc,             # <<<<<<<<<<<<<<
 *                            RegionContext cnt):
 * 
 */

static PyObject *__pyx_f_8pyregion_14_region_filter_10RegionBase_metric_set_origin(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, double __pyx_v_xc, double __pyx_v_yc, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_cnt) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("metric_set_origin", 0);

  /* "pyregion/_region_filter.pyx":88
 *                            RegionContext cnt):
 * 
 *         MetricSetOrigin(&(self.m), xc, yc)             # <<<<<<<<<<<<<<
 *         if cnt:
 *             self.set_context(cnt)
 */
  __pyx_f_8pyregion_14_region_filter_MetricSetOrigin((&__pyx_v_self->m), __pyx_v_xc, __pyx_v_yc);

  /* "pyregion/_region_filter.pyx":89
 * 
 *         MetricSetOrigin(&(self.m), xc, yc)
 *         if cnt:             # <<<<<<<<<<<<<<
 *             self.set_context(cnt)
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_cnt)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(1, 89, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "pyregion/_region_filter.pyx":90
 *         MetricSetOrigin(&(self.m), xc, yc)
 *         if cnt:
 *             self.set_context(cnt)             # <<<<<<<<<<<<<<
 *         else:
 *             self.update_metric()
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_set_context); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 90, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_cnt)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 90, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_cnt)};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 90, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_cnt)};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 90, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 90, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(((PyObject *)__pyx_v_cnt));
        __Pyx_GIVEREF(((PyObject *)__pyx_v_cnt));
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)__pyx_v_cnt));
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 90, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pyregion/_region_filter.pyx":89
 * 
 *         MetricSetOrigin(&(self.m), xc, yc)
 *         if cnt:             # <<<<<<<<<<<<<<
 *             self.set_context(cnt)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pyregion/_region_filter.pyx":92
 *             self.set_context(cnt)
 *         else:
 *             self.update_metric()             # <<<<<<<<<<<<<<
 * 
 *     def __invert__(self):
 */
  /*else*/ {
    __pyx_t_2 = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *)__pyx_v_self->__pyx_vtab)->update_metric(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 92, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "pyregion/_region_filter.pyx":85
 *         self.update_metric()
 * 
 *     cdef metric_set_origin(self, double xc, double yc,             # <<<<<<<<<<<<<<
 *                            RegionContext cnt):
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.metric_set_origin", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":94
 *             self.update_metric()
 * 
 *     def __invert__(self):             # <<<<<<<<<<<<<<
 *         return RegionNot(self)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_5__invert__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_5__invert__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__invert__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionBase_4__invert__(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_4__invert__(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__invert__", 0);

  /* "pyregion/_region_filter.pyx":95
 * 
 *     def __invert__(self):
 *         return RegionNot(self)             # <<<<<<<<<<<<<<
 * 
 *     def __and__(self, RegionBase o):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8pyregion_14_region_filter_RegionNot), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":94
 *             self.update_metric()
 * 
 *     def __invert__(self):             # <<<<<<<<<<<<<<
 *         return RegionNot(self)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.__invert__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":97
 *         return RegionNot(self)
 * 
 *     def __and__(self, RegionBase o):             # <<<<<<<<<<<<<<
 *         return RegionAnd(self, o)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_7__and__(PyObject *__pyx_v_self, PyObject *__pyx_v_o); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_7__and__(PyObject *__pyx_v_self, PyObject *__pyx_v_o) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__and__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_o), __pyx_ptype_8pyregion_14_region_filter_RegionBase, 1, "o", 0))) __PYX_ERR(1, 97, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionBase_6__and__(((PyObject *)__pyx_v_self), ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_o));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_6__and__(PyObject *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_o) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__and__", 0);

  /* "pyregion/_region_filter.pyx":98
 * 
 *     def __and__(self, RegionBase o):
 *         return RegionAnd(self, o)             # <<<<<<<<<<<<<<
 * 
 *     def __or__(self, RegionBase o):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_RegionAnd); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_self, ((PyObject *)__pyx_v_o)};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 98, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_self, ((PyObject *)__pyx_v_o)};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 98, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 98, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_self);
    __Pyx_GIVEREF(__pyx_v_self);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_self);
    __Pyx_INCREF(((PyObject *)__pyx_v_o));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_o));
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, ((PyObject *)__pyx_v_o));
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 98, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":97
 *         return RegionNot(self)
 * 
 *     def __and__(self, RegionBase o):             # <<<<<<<<<<<<<<
 *         return RegionAnd(self, o)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.__and__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":100
 *         return RegionAnd(self, o)
 * 
 *     def __or__(self, RegionBase o):             # <<<<<<<<<<<<<<
 *         return RegionOr(self, o)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_9__or__(PyObject *__pyx_v_self, PyObject *__pyx_v_o); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_9__or__(PyObject *__pyx_v_self, PyObject *__pyx_v_o) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__or__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_o), __pyx_ptype_8pyregion_14_region_filter_RegionBase, 1, "o", 0))) __PYX_ERR(1, 100, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionBase_8__or__(((PyObject *)__pyx_v_self), ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_o));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_8__or__(PyObject *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_o) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__or__", 0);

  /* "pyregion/_region_filter.pyx":101
 * 
 *     def __or__(self, RegionBase o):
 *         return RegionOr(self, o)             # <<<<<<<<<<<<<<
 * 
 *     cdef npy_bool _inside(self, double x, double y):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_RegionOr); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_self, ((PyObject *)__pyx_v_o)};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 101, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_self, ((PyObject *)__pyx_v_o)};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 101, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 101, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_self);
    __Pyx_GIVEREF(__pyx_v_self);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_self);
    __Pyx_INCREF(((PyObject *)__pyx_v_o));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_o));
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, ((PyObject *)__pyx_v_o));
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 101, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":100
 *         return RegionAnd(self, o)
 * 
 *     def __or__(self, RegionBase o):             # <<<<<<<<<<<<<<
 *         return RegionOr(self, o)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.__or__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":103
 *         return RegionOr(self, o)
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         return (0)
 * 
 */

static npy_bool __pyx_f_8pyregion_14_region_filter_10RegionBase__inside(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, CYTHON_UNUSED double __pyx_v_x, CYTHON_UNUSED double __pyx_v_y) {
  npy_bool __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_inside", 0);

  /* "pyregion/_region_filter.pyx":104
 * 
 *     cdef npy_bool _inside(self, double x, double y):
 *         return (0)             # <<<<<<<<<<<<<<
 * 
 *     def mask(self, img_or_shape):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":103
 *         return RegionOr(self, o)
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         return (0)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":106
 *         return (0)
 * 
 *     def mask(self, img_or_shape):             # <<<<<<<<<<<<<<
 *         """
 *         Create a mask ( a 2-d image whose pixel value is 1 if the
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_11mask(PyObject *__pyx_v_self, PyObject *__pyx_v_img_or_shape); /*proto*/
static char __pyx_doc_8pyregion_14_region_filter_10RegionBase_10mask[] = "\n        Create a mask ( a 2-d image whose pixel value is 1 if the\n        pixel is inside the filter, otherwise 0). It takes a single\n        argument which is numpy 2d array (or any python object with\n        *shape* attribute) or a tuple of two integer representing the\n        image shape.\n        ";
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_11mask(PyObject *__pyx_v_self, PyObject *__pyx_v_img_or_shape) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("mask (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionBase_10mask(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self), ((PyObject *)__pyx_v_img_or_shape));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_10mask(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, PyObject *__pyx_v_img_or_shape) {
  int __pyx_v_nx;
  int __pyx_v_ny;
  PyObject *__pyx_v_shape = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("mask", 0);

  /* "pyregion/_region_filter.pyx":117
 *         cdef int l, nx, ny
 * 
 *         if hasattr(img_or_shape, "shape"):             # <<<<<<<<<<<<<<
 *             shape = img_or_shape.shape
 *         elif c_python.PySequence_Check(img_or_shape):
 */
  __pyx_t_1 = __Pyx_HasAttr(__pyx_v_img_or_shape, __pyx_n_s_shape); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(1, 117, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "pyregion/_region_filter.pyx":118
 * 
 *         if hasattr(img_or_shape, "shape"):
 *             shape = img_or_shape.shape             # <<<<<<<<<<<<<<
 *         elif c_python.PySequence_Check(img_or_shape):
 *             shape = img_or_shape
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_img_or_shape, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 118, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_shape = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyregion/_region_filter.pyx":117
 *         cdef int l, nx, ny
 * 
 *         if hasattr(img_or_shape, "shape"):             # <<<<<<<<<<<<<<
 *             shape = img_or_shape.shape
 *         elif c_python.PySequence_Check(img_or_shape):
 */
    goto __pyx_L3;
  }

  /* "pyregion/_region_filter.pyx":119
 *         if hasattr(img_or_shape, "shape"):
 *             shape = img_or_shape.shape
 *         elif c_python.PySequence_Check(img_or_shape):             # <<<<<<<<<<<<<<
 *             shape = img_or_shape
 *         else:
 */
  __pyx_t_2 = (PySequence_Check(__pyx_v_img_or_shape) != 0);
  if (__pyx_t_2) {

    /* "pyregion/_region_filter.pyx":120
 *             shape = img_or_shape.shape
 *         elif c_python.PySequence_Check(img_or_shape):
 *             shape = img_or_shape             # <<<<<<<<<<<<<<
 *         else:
 *             raise RegionFilterException("the inut needs to be a numpy 2-d array"
 */
    __Pyx_INCREF(__pyx_v_img_or_shape);
    __pyx_v_shape = __pyx_v_img_or_shape;

    /* "pyregion/_region_filter.pyx":119
 *         if hasattr(img_or_shape, "shape"):
 *             shape = img_or_shape.shape
 *         elif c_python.PySequence_Check(img_or_shape):             # <<<<<<<<<<<<<<
 *             shape = img_or_shape
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pyregion/_region_filter.pyx":122
 *             shape = img_or_shape
 *         else:
 *             raise RegionFilterException("the inut needs to be a numpy 2-d array"             # <<<<<<<<<<<<<<
 *                                         " or a tuple of two integers")
 * 
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_RegionFilterException); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 122, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 122, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(1, 122, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "pyregion/_region_filter.pyx":125
 *                                         " or a tuple of two integers")
 * 
 *         if c_python.PySequence_Length(shape) != 2:             # <<<<<<<<<<<<<<
 *             raise RegionFilterException("shape of the input image must be 2d: "
 *                                         "%s is given" % (str(shape)))
 */
  __pyx_t_2 = ((PySequence_Length(__pyx_v_shape) != 2) != 0);
  if (__pyx_t_2) {

    /* "pyregion/_region_filter.pyx":126
 * 
 *         if c_python.PySequence_Length(shape) != 2:
 *             raise RegionFilterException("shape of the input image must be 2d: "             # <<<<<<<<<<<<<<
 *                                         "%s is given" % (str(shape)))
 * 
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_RegionFilterException); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "pyregion/_region_filter.pyx":127
 *         if c_python.PySequence_Length(shape) != 2:
 *             raise RegionFilterException("shape of the input image must be 2d: "
 *                                         "%s is given" % (str(shape)))             # <<<<<<<<<<<<<<
 * 
 *         ny = c_python.PySequence_GetItem(shape, 0)
 */
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 127, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_shape);
    __Pyx_GIVEREF(__pyx_v_shape);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_shape);
    __pyx_t_6 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_5, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 127, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_shape_of_the_input_image_must_be, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 127, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 126, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 126, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 126, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(1, 126, __pyx_L1_error)

    /* "pyregion/_region_filter.pyx":125
 *                                         " or a tuple of two integers")
 * 
 *         if c_python.PySequence_Length(shape) != 2:             # <<<<<<<<<<<<<<
 *             raise RegionFilterException("shape of the input image must be 2d: "
 *                                         "%s is given" % (str(shape)))
 */
  }

  /* "pyregion/_region_filter.pyx":129
 *                                         "%s is given" % (str(shape)))
 * 
 *         ny = c_python.PySequence_GetItem(shape, 0)             # <<<<<<<<<<<<<<
 *         nx = c_python.PySequence_GetItem(shape, 1)
 * 
 */
  __pyx_t_4 = PySequence_GetItem(__pyx_v_shape, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 129, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_ny = __pyx_t_8;

  /* "pyregion/_region_filter.pyx":130
 * 
 *         ny = c_python.PySequence_GetItem(shape, 0)
 *         nx = c_python.PySequence_GetItem(shape, 1)             # <<<<<<<<<<<<<<
 * 
 *         return self._mask(nx, ny)
 */
  __pyx_t_4 = PySequence_GetItem(__pyx_v_shape, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 130, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_nx = __pyx_t_8;

  /* "pyregion/_region_filter.pyx":132
 *         nx = c_python.PySequence_GetItem(shape, 1)
 * 
 *         return self._mask(nx, ny)             # <<<<<<<<<<<<<<
 * 
 *     cdef c_numpy.ndarray _mask(self, c_numpy.npy_intp nx, c_numpy.npy_intp ny):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *)__pyx_v_self->__pyx_vtab)->_mask(__pyx_v_self, __pyx_v_nx, __pyx_v_ny)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":106
 *         return (0)
 * 
 *     def mask(self, img_or_shape):             # <<<<<<<<<<<<<<
 *         """
 *         Create a mask ( a 2-d image whose pixel value is 1 if the
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.mask", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":134
 *         return self._mask(nx, ny)
 * 
 *     cdef c_numpy.ndarray _mask(self, c_numpy.npy_intp nx, c_numpy.npy_intp ny):             # <<<<<<<<<<<<<<
 * 
 *         cdef c_numpy.npy_intp ny_nx[2]
 */

static PyArrayObject *__pyx_f_8pyregion_14_region_filter_10RegionBase__mask(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, npy_intp __pyx_v_nx, npy_intp __pyx_v_ny) {
  npy_intp __pyx_v_ny_nx[2];
  PyArrayObject *__pyx_v_ra = 0;
  npy_bool *__pyx_v_rd;
  int __pyx_v_iy;
  int __pyx_v_ix;
  PyArrayObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  __Pyx_RefNannySetupContext("_mask", 0);

  /* "pyregion/_region_filter.pyx":141
 *         cdef int iy, ix
 * 
 *         ny_nx[0] = ny             # <<<<<<<<<<<<<<
 *         ny_nx[1] = nx
 * 
 */
  (__pyx_v_ny_nx[0]) = __pyx_v_ny;

  /* "pyregion/_region_filter.pyx":142
 * 
 *         ny_nx[0] = ny
 *         ny_nx[1] = nx             # <<<<<<<<<<<<<<
 * 
 *         ra = c_numpy.PyArray_EMPTY(2, ny_nx,
 */
  (__pyx_v_ny_nx[1]) = __pyx_v_nx;

  /* "pyregion/_region_filter.pyx":144
 *         ny_nx[1] = nx
 * 
 *         ra = c_numpy.PyArray_EMPTY(2, ny_nx,             # <<<<<<<<<<<<<<
 *                                    c_numpy.NPY_BOOL, 0)
 * 
 */
  __pyx_t_1 = PyArray_EMPTY(2, __pyx_v_ny_nx, NPY_BOOL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_8pyregion_7c_numpy_ndarray))))) __PYX_ERR(1, 144, __pyx_L1_error)
  __pyx_v_ra = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":147
 *                                    c_numpy.NPY_BOOL, 0)
 * 
 *         rd = <npy_bool *> c_numpy.PyArray_DATA(ra)             # <<<<<<<<<<<<<<
 * 
 *         for iy from 0 <= iy < ny:
 */
  __pyx_v_rd = ((npy_bool *)PyArray_DATA(__pyx_v_ra));

  /* "pyregion/_region_filter.pyx":149
 *         rd = <npy_bool *> c_numpy.PyArray_DATA(ra)
 * 
 *         for iy from 0 <= iy < ny:             # <<<<<<<<<<<<<<
 *             for ix from 0 <= ix < nx:
 *                 #rd[iy*nx + ix] = self._inside(i, j)
 */
  __pyx_t_2 = __pyx_v_ny;
  for (__pyx_v_iy = 0; __pyx_v_iy < __pyx_t_2; __pyx_v_iy++) {

    /* "pyregion/_region_filter.pyx":150
 * 
 *         for iy from 0 <= iy < ny:
 *             for ix from 0 <= ix < nx:             # <<<<<<<<<<<<<<
 *                 #rd[iy*nx + ix] = self._inside(i, j)
 *                 # altenatively more optimized
 */
    __pyx_t_3 = __pyx_v_nx;
    for (__pyx_v_ix = 0; __pyx_v_ix < __pyx_t_3; __pyx_v_ix++) {

      /* "pyregion/_region_filter.pyx":154
 *                 # altenatively more optimized
 *                 #rd[0] = self._inside(ix+1, iy+1) # +1 for (1,1) based..
 *                 rd[0] = self._inside(ix, iy)             # <<<<<<<<<<<<<<
 *                 rd = rd + 1
 * 
 */
      (__pyx_v_rd[0]) = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *)__pyx_v_self->__pyx_vtab)->_inside(__pyx_v_self, __pyx_v_ix, __pyx_v_iy);

      /* "pyregion/_region_filter.pyx":155
 *                 #rd[0] = self._inside(ix+1, iy+1) # +1 for (1,1) based..
 *                 rd[0] = self._inside(ix, iy)
 *                 rd = rd + 1             # <<<<<<<<<<<<<<
 * 
 *         return ra
 */
      __pyx_v_rd = (__pyx_v_rd + 1);
    }
  }

  /* "pyregion/_region_filter.pyx":157
 *                 rd = rd + 1
 * 
 *         return ra             # <<<<<<<<<<<<<<
 * 
 *     def inside1(self, double x, double y):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_ra));
  __pyx_r = __pyx_v_ra;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":134
 *         return self._mask(nx, ny)
 * 
 *     cdef c_numpy.ndarray _mask(self, c_numpy.npy_intp nx, c_numpy.npy_intp ny):             # <<<<<<<<<<<<<<
 * 
 *         cdef c_numpy.npy_intp ny_nx[2]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase._mask", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_ra);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":159
 *         return ra
 * 
 *     def inside1(self, double x, double y):             # <<<<<<<<<<<<<<
 *         """
 *         inside1(float, float) : returns True if the point (x,y) is inside the filter.
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_13inside1(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8pyregion_14_region_filter_10RegionBase_12inside1[] = "\n        inside1(float, float) : returns True if the point (x,y) is inside the filter.\n        ";
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_13inside1(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_x;
  double __pyx_v_y;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("inside1 (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("inside1", 1, 2, 2, 1); __PYX_ERR(1, 159, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "inside1") < 0)) __PYX_ERR(1, 159, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_x = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 159, __pyx_L3_error)
    __pyx_v_y = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_y == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 159, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("inside1", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 159, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.inside1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionBase_12inside1(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self), __pyx_v_x, __pyx_v_y);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_12inside1(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, double __pyx_v_x, double __pyx_v_y) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("inside1", 0);

  /* "pyregion/_region_filter.pyx":163
 *         inside1(float, float) : returns True if the point (x,y) is inside the filter.
 *         """
 *         return self._inside(x, y)             # <<<<<<<<<<<<<<
 * 
 *     def inside(self, x, y=None):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_npy_bool(((struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *)__pyx_v_self->__pyx_vtab)->_inside(__pyx_v_self, __pyx_v_x, __pyx_v_y)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":159
 *         return ra
 * 
 *     def inside1(self, double x, double y):             # <<<<<<<<<<<<<<
 *         """
 *         inside1(float, float) : returns True if the point (x,y) is inside the filter.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.inside1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":165
 *         return self._inside(x, y)
 * 
 *     def inside(self, x, y=None):             # <<<<<<<<<<<<<<
 *         if y is None:
 *             if len(x.shape) == 2 and x.shape[-1] == 2:
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_15inside(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_15inside(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("inside (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "inside") < 0)) __PYX_ERR(1, 165, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_x = values[0];
    __pyx_v_y = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("inside", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 165, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.inside", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionBase_14inside(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self), __pyx_v_x, __pyx_v_y);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_14inside(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("inside", 0);

  /* "pyregion/_region_filter.pyx":166
 * 
 *     def inside(self, x, y=None):
 *         if y is None:             # <<<<<<<<<<<<<<
 *             if len(x.shape) == 2 and x.shape[-1] == 2:
 *                 return self.inside_xy(x)
 */
  __pyx_t_1 = (__pyx_v_y == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "pyregion/_region_filter.pyx":167
 *     def inside(self, x, y=None):
 *         if y is None:
 *             if len(x.shape) == 2 and x.shape[-1] == 2:             # <<<<<<<<<<<<<<
 *                 return self.inside_xy(x)
 *             else:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyObject_Length(__pyx_t_3); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 167, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = ((__pyx_t_4 == 2) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_2 = __pyx_t_1;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_x, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_3, -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_t_5, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(1, 167, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_2 = __pyx_t_1;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_2) {

      /* "pyregion/_region_filter.pyx":168
 *         if y is None:
 *             if len(x.shape) == 2 and x.shape[-1] == 2:
 *                 return self.inside_xy(x)             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError("input array has a wrong shape")
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_inside_xy); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 168, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_6) {
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_x); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 168, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_x};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 168, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_3);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_x};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 168, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_3);
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 168, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
          __Pyx_INCREF(__pyx_v_x);
          __Pyx_GIVEREF(__pyx_v_x);
          PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_x);
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 168, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_r = __pyx_t_3;
      __pyx_t_3 = 0;
      goto __pyx_L0;

      /* "pyregion/_region_filter.pyx":167
 *     def inside(self, x, y=None):
 *         if y is None:
 *             if len(x.shape) == 2 and x.shape[-1] == 2:             # <<<<<<<<<<<<<<
 *                 return self.inside_xy(x)
 *             else:
 */
    }

    /* "pyregion/_region_filter.pyx":170
 *                 return self.inside_xy(x)
 *             else:
 *                 raise ValueError("input array has a wrong shape")             # <<<<<<<<<<<<<<
 *         else:
 *             return self.inside_x_y(x, y)
 */
    /*else*/ {
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 170, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(1, 170, __pyx_L1_error)
    }

    /* "pyregion/_region_filter.pyx":166
 * 
 *     def inside(self, x, y=None):
 *         if y is None:             # <<<<<<<<<<<<<<
 *             if len(x.shape) == 2 and x.shape[-1] == 2:
 *                 return self.inside_xy(x)
 */
  }

  /* "pyregion/_region_filter.pyx":172
 *                 raise ValueError("input array has a wrong shape")
 *         else:
 *             return self.inside_x_y(x, y)             # <<<<<<<<<<<<<<
 * 
 *     def inside_xy(self, xy):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_inside_x_y); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 172, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_x, __pyx_v_y};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 172, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_x, __pyx_v_y};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 172, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_6 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 172, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_INCREF(__pyx_v_x);
      __Pyx_GIVEREF(__pyx_v_x);
      PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_8, __pyx_v_x);
      __Pyx_INCREF(__pyx_v_y);
      __Pyx_GIVEREF(__pyx_v_y);
      PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_8, __pyx_v_y);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 172, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "pyregion/_region_filter.pyx":165
 *         return self._inside(x, y)
 * 
 *     def inside(self, x, y=None):             # <<<<<<<<<<<<<<
 *         if y is None:
 *             if len(x.shape) == 2 and x.shape[-1] == 2:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.inside", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":174
 *             return self.inside_x_y(x, y)
 * 
 *     def inside_xy(self, xy):             # <<<<<<<<<<<<<<
 *         """
 *         inside(x, y) : given the numpy array of x and y, returns an
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_17inside_xy(PyObject *__pyx_v_self, PyObject *__pyx_v_xy); /*proto*/
static char __pyx_doc_8pyregion_14_region_filter_10RegionBase_16inside_xy[] = "\n        inside(x, y) : given the numpy array of x and y, returns an\n        array b of same shape, where b[i] = inside1(x[i], y[i])\n        ";
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_17inside_xy(PyObject *__pyx_v_self, PyObject *__pyx_v_xy) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("inside_xy (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionBase_16inside_xy(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self), ((PyObject *)__pyx_v_xy));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_16inside_xy(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, PyObject *__pyx_v_xy) {
  PyArrayObject *__pyx_v_xya = 0;
  PyArrayObject *__pyx_v_ra = 0;
  double *__pyx_v_xyd;
  npy_bool *__pyx_v_rd;
  int __pyx_v_i;
  int __pyx_v_n;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("inside_xy", 0);

  /* "pyregion/_region_filter.pyx":186
 *         cdef int n
 * 
 *         xya = c_numpy.PyArray_ContiguousFromAny(xy, c_numpy.NPY_DOUBLE, 1, 0)             # <<<<<<<<<<<<<<
 * 
 *         ra = c_numpy.PyArray_EMPTY(1, xya.dimensions,
 */
  __pyx_t_1 = PyArray_ContiguousFromAny(__pyx_v_xy, NPY_DOUBLE, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_8pyregion_7c_numpy_ndarray))))) __PYX_ERR(1, 186, __pyx_L1_error)
  __pyx_v_xya = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":188
 *         xya = c_numpy.PyArray_ContiguousFromAny(xy, c_numpy.NPY_DOUBLE, 1, 0)
 * 
 *         ra = c_numpy.PyArray_EMPTY(1, xya.dimensions,             # <<<<<<<<<<<<<<
 *                                    c_numpy.NPY_BOOL, 0)
 * 
 */
  __pyx_t_1 = PyArray_EMPTY(1, __pyx_v_xya->dimensions, NPY_BOOL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_8pyregion_7c_numpy_ndarray))))) __PYX_ERR(1, 188, __pyx_L1_error)
  __pyx_v_ra = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":191
 *                                    c_numpy.NPY_BOOL, 0)
 * 
 *         xyd = <double *> c_numpy.PyArray_DATA(xya)             # <<<<<<<<<<<<<<
 *         rd = <npy_bool *> c_numpy.PyArray_DATA(ra)
 * 
 */
  __pyx_v_xyd = ((double *)PyArray_DATA(__pyx_v_xya));

  /* "pyregion/_region_filter.pyx":192
 * 
 *         xyd = <double *> c_numpy.PyArray_DATA(xya)
 *         rd = <npy_bool *> c_numpy.PyArray_DATA(ra)             # <<<<<<<<<<<<<<
 * 
 *         n = xya.dimensions[0]  # c_numpy.PyArray_SIZE(xya) / 2
 */
  __pyx_v_rd = ((npy_bool *)PyArray_DATA(__pyx_v_ra));

  /* "pyregion/_region_filter.pyx":194
 *         rd = <npy_bool *> c_numpy.PyArray_DATA(ra)
 * 
 *         n = xya.dimensions[0]  # c_numpy.PyArray_SIZE(xya) / 2             # <<<<<<<<<<<<<<
 *         #_inside_ptr = self._inside
 *         for i from 0 <= i < n:
 */
  __pyx_v_n = (__pyx_v_xya->dimensions[0]);

  /* "pyregion/_region_filter.pyx":196
 *         n = xya.dimensions[0]  # c_numpy.PyArray_SIZE(xya) / 2
 *         #_inside_ptr = self._inside
 *         for i from 0 <= i < n:             # <<<<<<<<<<<<<<
 *             rd[i] = self._inside(xyd[2 * i], xyd[2 * i + 1])
 *         return ra
 */
  __pyx_t_2 = __pyx_v_n;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_2; __pyx_v_i++) {

    /* "pyregion/_region_filter.pyx":197
 *         #_inside_ptr = self._inside
 *         for i from 0 <= i < n:
 *             rd[i] = self._inside(xyd[2 * i], xyd[2 * i + 1])             # <<<<<<<<<<<<<<
 *         return ra
 * 
 */
    (__pyx_v_rd[__pyx_v_i]) = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *)__pyx_v_self->__pyx_vtab)->_inside(__pyx_v_self, (__pyx_v_xyd[(2 * __pyx_v_i)]), (__pyx_v_xyd[((2 * __pyx_v_i) + 1)]));
  }

  /* "pyregion/_region_filter.pyx":198
 *         for i from 0 <= i < n:
 *             rd[i] = self._inside(xyd[2 * i], xyd[2 * i + 1])
 *         return ra             # <<<<<<<<<<<<<<
 * 
 *     def inside_x_y(self, x, y):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_ra));
  __pyx_r = ((PyObject *)__pyx_v_ra);
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":174
 *             return self.inside_x_y(x, y)
 * 
 *     def inside_xy(self, xy):             # <<<<<<<<<<<<<<
 *         """
 *         inside(x, y) : given the numpy array of x and y, returns an
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.inside_xy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_xya);
  __Pyx_XDECREF((PyObject *)__pyx_v_ra);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":200
 *         return ra
 * 
 *     def inside_x_y(self, x, y):             # <<<<<<<<<<<<<<
 *         """
 *         inside(x, y) : given the numpy array of x and y, returns an
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_19inside_x_y(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8pyregion_14_region_filter_10RegionBase_18inside_x_y[] = "\n        inside(x, y) : given the numpy array of x and y, returns an\n        array b of same shape, where b[i] = inside1(x[i], y[i])\n        ";
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_19inside_x_y(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("inside_x_y (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("inside_x_y", 1, 2, 2, 1); __PYX_ERR(1, 200, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "inside_x_y") < 0)) __PYX_ERR(1, 200, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_x = values[0];
    __pyx_v_y = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("inside_x_y", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 200, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.inside_x_y", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionBase_18inside_x_y(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self), __pyx_v_x, __pyx_v_y);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_18inside_x_y(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y) {
  PyArrayObject *__pyx_v_xa = 0;
  PyArrayObject *__pyx_v_ya = 0;
  PyArrayObject *__pyx_v_ra = 0;
  double *__pyx_v_xd;
  double *__pyx_v_yd;
  npy_bool *__pyx_v_rd;
  int __pyx_v_i;
  int __pyx_v_n;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("inside_x_y", 0);

  /* "pyregion/_region_filter.pyx":217
 *         # FIX : check if two input has identical shape
 * 
 *         xa = c_numpy.PyArray_ContiguousFromAny(x, c_numpy.NPY_DOUBLE, 1, 0)             # <<<<<<<<<<<<<<
 *         ya = c_numpy.PyArray_ContiguousFromAny(y, c_numpy.NPY_DOUBLE, 1, 0)
 * 
 */
  __pyx_t_1 = PyArray_ContiguousFromAny(__pyx_v_x, NPY_DOUBLE, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_8pyregion_7c_numpy_ndarray))))) __PYX_ERR(1, 217, __pyx_L1_error)
  __pyx_v_xa = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":218
 * 
 *         xa = c_numpy.PyArray_ContiguousFromAny(x, c_numpy.NPY_DOUBLE, 1, 0)
 *         ya = c_numpy.PyArray_ContiguousFromAny(y, c_numpy.NPY_DOUBLE, 1, 0)             # <<<<<<<<<<<<<<
 * 
 *         ra = c_numpy.PyArray_EMPTY(xa.nd, xa.dimensions,
 */
  __pyx_t_1 = PyArray_ContiguousFromAny(__pyx_v_y, NPY_DOUBLE, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_8pyregion_7c_numpy_ndarray))))) __PYX_ERR(1, 218, __pyx_L1_error)
  __pyx_v_ya = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":220
 *         ya = c_numpy.PyArray_ContiguousFromAny(y, c_numpy.NPY_DOUBLE, 1, 0)
 * 
 *         ra = c_numpy.PyArray_EMPTY(xa.nd, xa.dimensions,             # <<<<<<<<<<<<<<
 *                                    c_numpy.NPY_BOOL, 0)
 * 
 */
  __pyx_t_1 = PyArray_EMPTY(__pyx_v_xa->nd, __pyx_v_xa->dimensions, NPY_BOOL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_8pyregion_7c_numpy_ndarray))))) __PYX_ERR(1, 220, __pyx_L1_error)
  __pyx_v_ra = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":223
 *                                    c_numpy.NPY_BOOL, 0)
 * 
 *         xd = <double *> c_numpy.PyArray_DATA(xa)             # <<<<<<<<<<<<<<
 *         yd = <double *> c_numpy.PyArray_DATA(ya)
 *         rd = <npy_bool *> c_numpy.PyArray_DATA(ra)
 */
  __pyx_v_xd = ((double *)PyArray_DATA(__pyx_v_xa));

  /* "pyregion/_region_filter.pyx":224
 * 
 *         xd = <double *> c_numpy.PyArray_DATA(xa)
 *         yd = <double *> c_numpy.PyArray_DATA(ya)             # <<<<<<<<<<<<<<
 *         rd = <npy_bool *> c_numpy.PyArray_DATA(ra)
 * 
 */
  __pyx_v_yd = ((double *)PyArray_DATA(__pyx_v_ya));

  /* "pyregion/_region_filter.pyx":225
 *         xd = <double *> c_numpy.PyArray_DATA(xa)
 *         yd = <double *> c_numpy.PyArray_DATA(ya)
 *         rd = <npy_bool *> c_numpy.PyArray_DATA(ra)             # <<<<<<<<<<<<<<
 * 
 *         n = c_numpy.PyArray_SIZE(xa)
 */
  __pyx_v_rd = ((npy_bool *)PyArray_DATA(__pyx_v_ra));

  /* "pyregion/_region_filter.pyx":227
 *         rd = <npy_bool *> c_numpy.PyArray_DATA(ra)
 * 
 *         n = c_numpy.PyArray_SIZE(xa)             # <<<<<<<<<<<<<<
 *         #_inside_ptr = self._inside
 *         for i from 0 <= i < n:
 */
  __pyx_v_n = PyArray_SIZE(__pyx_v_xa);

  /* "pyregion/_region_filter.pyx":229
 *         n = c_numpy.PyArray_SIZE(xa)
 *         #_inside_ptr = self._inside
 *         for i from 0 <= i < n:             # <<<<<<<<<<<<<<
 *             #self._inside(xd[0], yd[0])
 *             #rd[i] = _inside_ptr(self, xd[i], yd[i])
 */
  __pyx_t_2 = __pyx_v_n;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_2; __pyx_v_i++) {

    /* "pyregion/_region_filter.pyx":232
 *             #self._inside(xd[0], yd[0])
 *             #rd[i] = _inside_ptr(self, xd[i], yd[i])
 *             rd[i] = self._inside(xd[i], yd[i])             # <<<<<<<<<<<<<<
 *             #rd[i] = _inside_ptr(self, xd[i], yd[i])
 *             #print xd[i], yd[i], rd[i]
 */
    (__pyx_v_rd[__pyx_v_i]) = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *)__pyx_v_self->__pyx_vtab)->_inside(__pyx_v_self, (__pyx_v_xd[__pyx_v_i]), (__pyx_v_yd[__pyx_v_i]));
  }

  /* "pyregion/_region_filter.pyx":235
 *             #rd[i] = _inside_ptr(self, xd[i], yd[i])
 *             #print xd[i], yd[i], rd[i]
 *         return ra             # <<<<<<<<<<<<<<
 * 
 * cdef class RegionNot(RegionBase):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_ra));
  __pyx_r = ((PyObject *)__pyx_v_ra);
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":200
 *         return ra
 * 
 *     def inside_x_y(self, x, y):             # <<<<<<<<<<<<<<
 *         """
 *         inside(x, y) : given the numpy array of x and y, returns an
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.inside_x_y", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_xa);
  __Pyx_XDECREF((PyObject *)__pyx_v_ya);
  __Pyx_XDECREF((PyObject *)__pyx_v_ra);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_21__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_21__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionBase_20__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_20__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_23__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionBase_23__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionBase_22__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionBase_22__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionBase.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":243
 *     cdef RegionBase child_region
 * 
 *     def __init__(self, RegionBase child_region):             # <<<<<<<<<<<<<<
 *         self.child_region = child_region
 * 
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_9RegionNot_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_9RegionNot_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_child_region = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_child_region,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_child_region)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 243, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_child_region = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 243, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionNot.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_child_region), __pyx_ptype_8pyregion_14_region_filter_RegionBase, 1, "child_region", 0))) __PYX_ERR(1, 243, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_9RegionNot___init__(((struct __pyx_obj_8pyregion_14_region_filter_RegionNot *)__pyx_v_self), __pyx_v_child_region);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_9RegionNot___init__(struct __pyx_obj_8pyregion_14_region_filter_RegionNot *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_child_region) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyregion/_region_filter.pyx":244
 * 
 *     def __init__(self, RegionBase child_region):
 *         self.child_region = child_region             # <<<<<<<<<<<<<<
 * 
 *     cdef npy_bool _inside(self, double x, double y):
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_child_region));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_child_region));
  __Pyx_GOTREF(__pyx_v_self->child_region);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->child_region));
  __pyx_v_self->child_region = __pyx_v_child_region;

  /* "pyregion/_region_filter.pyx":243
 *     cdef RegionBase child_region
 * 
 *     def __init__(self, RegionBase child_region):             # <<<<<<<<<<<<<<
 *         self.child_region = child_region
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":246
 *         self.child_region = child_region
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         return not (self.child_region._inside(x, y))
 * 
 */

static npy_bool __pyx_f_8pyregion_14_region_filter_9RegionNot__inside(struct __pyx_obj_8pyregion_14_region_filter_RegionNot *__pyx_v_self, double __pyx_v_x, double __pyx_v_y) {
  npy_bool __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_inside", 0);

  /* "pyregion/_region_filter.pyx":247
 * 
 *     cdef npy_bool _inside(self, double x, double y):
 *         return not (self.child_region._inside(x, y))             # <<<<<<<<<<<<<<
 * 
 * cdef class RegionList(RegionBase):
 */
  __pyx_r = (!(((struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *)__pyx_v_self->child_region->__pyx_vtab)->_inside(__pyx_v_self->child_region, __pyx_v_x, __pyx_v_y) != 0));
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":246
 *         self.child_region = child_region
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         return not (self.child_region._inside(x, y))
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_9RegionNot_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_9RegionNot_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_9RegionNot_2__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_RegionNot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_9RegionNot_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionNot *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionNot.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_9RegionNot_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_9RegionNot_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_9RegionNot_4__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_RegionNot *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_9RegionNot_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionNot *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionNot.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":252
 *     cdef object child_regions
 * 
 *     def _check_type_of_list(self, kl):             # <<<<<<<<<<<<<<
 *         for k in kl:
 *             if not isinstance(k, RegionBase):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionList_1_check_type_of_list(PyObject *__pyx_v_self, PyObject *__pyx_v_kl); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionList_1_check_type_of_list(PyObject *__pyx_v_self, PyObject *__pyx_v_kl) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_check_type_of_list (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionList__check_type_of_list(((struct __pyx_obj_8pyregion_14_region_filter_RegionList *)__pyx_v_self), ((PyObject *)__pyx_v_kl));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionList__check_type_of_list(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, PyObject *__pyx_v_kl) {
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("_check_type_of_list", 0);

  /* "pyregion/_region_filter.pyx":253
 * 
 *     def _check_type_of_list(self, kl):
 *         for k in kl:             # <<<<<<<<<<<<<<
 *             if not isinstance(k, RegionBase):
 *                 raise TypeError("All elements should be subclass of RegionBase type: %s" % k)
 */
  if (likely(PyList_CheckExact(__pyx_v_kl)) || PyTuple_CheckExact(__pyx_v_kl)) {
    __pyx_t_1 = __pyx_v_kl; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_kl); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 253, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 253, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(1, 253, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 253, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(1, 253, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 253, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(1, 253, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "pyregion/_region_filter.pyx":254
 *     def _check_type_of_list(self, kl):
 *         for k in kl:
 *             if not isinstance(k, RegionBase):             # <<<<<<<<<<<<<<
 *                 raise TypeError("All elements should be subclass of RegionBase type: %s" % k)
 * 
 */
    __pyx_t_5 = __Pyx_TypeCheck(__pyx_v_k, __pyx_ptype_8pyregion_14_region_filter_RegionBase); 
    __pyx_t_6 = ((!(__pyx_t_5 != 0)) != 0);
    if (__pyx_t_6) {

      /* "pyregion/_region_filter.pyx":255
 *         for k in kl:
 *             if not isinstance(k, RegionBase):
 *                 raise TypeError("All elements should be subclass of RegionBase type: %s" % k)             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, *kl):
 */
      __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_All_elements_should_be_subclass, __pyx_v_k); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 255, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 255, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 255, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __PYX_ERR(1, 255, __pyx_L1_error)

      /* "pyregion/_region_filter.pyx":254
 *     def _check_type_of_list(self, kl):
 *         for k in kl:
 *             if not isinstance(k, RegionBase):             # <<<<<<<<<<<<<<
 *                 raise TypeError("All elements should be subclass of RegionBase type: %s" % k)
 * 
 */
    }

    /* "pyregion/_region_filter.pyx":253
 * 
 *     def _check_type_of_list(self, kl):
 *         for k in kl:             # <<<<<<<<<<<<<<
 *             if not isinstance(k, RegionBase):
 *                 raise TypeError("All elements should be subclass of RegionBase type: %s" % k)
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":252
 *     cdef object child_regions
 * 
 *     def _check_type_of_list(self, kl):             # <<<<<<<<<<<<<<
 *         for k in kl:
 *             if not isinstance(k, RegionBase):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("pyregion._region_filter.RegionList._check_type_of_list", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":257
 *                 raise TypeError("All elements should be subclass of RegionBase type: %s" % k)
 * 
 *     def __init__(self, *kl):             # <<<<<<<<<<<<<<
 *         self._check_type_of_list(kl)
 *         self.child_regions = list(kl)
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_10RegionList_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_10RegionList_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kl = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_kl = __pyx_args;
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionList_2__init__(((struct __pyx_obj_8pyregion_14_region_filter_RegionList *)__pyx_v_self), __pyx_v_kl);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kl);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_10RegionList_2__init__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, PyObject *__pyx_v_kl) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyregion/_region_filter.pyx":258
 * 
 *     def __init__(self, *kl):
 *         self._check_type_of_list(kl)             # <<<<<<<<<<<<<<
 *         self.child_regions = list(kl)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_check_type_of_list); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_kl); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 258, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_kl};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 258, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_kl};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 258, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 258, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_kl);
      __Pyx_GIVEREF(__pyx_v_kl);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_kl);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 258, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":259
 *     def __init__(self, *kl):
 *         self._check_type_of_list(kl)
 *         self.child_regions = list(kl)             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __pyx_t_1 = PySequence_List(__pyx_v_kl); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->child_regions);
  __Pyx_DECREF(__pyx_v_self->child_regions);
  __pyx_v_self->child_regions = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":257
 *                 raise TypeError("All elements should be subclass of RegionBase type: %s" % k)
 * 
 *     def __init__(self, *kl):             # <<<<<<<<<<<<<<
 *         self._check_type_of_list(kl)
 *         self.child_regions = list(kl)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyregion._region_filter.RegionList.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":261
 *         self.child_regions = list(kl)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return len(self.child_regions)
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_8pyregion_14_region_filter_10RegionList_5__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_8pyregion_14_region_filter_10RegionList_5__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionList_4__len__(((struct __pyx_obj_8pyregion_14_region_filter_RegionList *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_8pyregion_14_region_filter_10RegionList_4__len__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "pyregion/_region_filter.pyx":262
 * 
 *     def __len__(self):
 *         return len(self.child_regions)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, Py_ssize_t x):
 */
  __pyx_t_1 = __pyx_v_self->child_regions;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(1, 262, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":261
 *         self.child_regions = list(kl)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return len(self.child_regions)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionList.__len__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":264
 *         return len(self.child_regions)
 * 
 *     def __getitem__(self, Py_ssize_t x):             # <<<<<<<<<<<<<<
 *         return self.child_regions[x]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionList_7__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_x); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionList_7__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_x) {
  Py_ssize_t __pyx_v_x;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  assert(__pyx_arg_x); {
    __pyx_v_x = __Pyx_PyIndex_AsSsize_t(__pyx_arg_x); if (unlikely((__pyx_v_x == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 264, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionList.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionList_6__getitem__(((struct __pyx_obj_8pyregion_14_region_filter_RegionList *)__pyx_v_self), ((Py_ssize_t)__pyx_v_x));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionList_6__getitem__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, Py_ssize_t __pyx_v_x) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "pyregion/_region_filter.pyx":265
 * 
 *     def __getitem__(self, Py_ssize_t x):
 *         return self.child_regions[x]             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, Py_ssize_t x, RegionBase y):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_self->child_regions, __pyx_v_x, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":264
 *         return len(self.child_regions)
 * 
 *     def __getitem__(self, Py_ssize_t x):             # <<<<<<<<<<<<<<
 *         return self.child_regions[x]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionList.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":267
 *         return self.child_regions[x]
 * 
 *     def __setitem__(self, Py_ssize_t x, RegionBase y):             # <<<<<<<<<<<<<<
 *         self.child_regions[x] = y
 * 
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_10RegionList_9__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_x, PyObject *__pyx_v_y); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_10RegionList_9__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_x, PyObject *__pyx_v_y) {
  Py_ssize_t __pyx_v_x;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  assert(__pyx_arg_x); {
    __pyx_v_x = __Pyx_PyIndex_AsSsize_t(__pyx_arg_x); if (unlikely((__pyx_v_x == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 267, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionList.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_y), __pyx_ptype_8pyregion_14_region_filter_RegionBase, 1, "y", 0))) __PYX_ERR(1, 267, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionList_8__setitem__(((struct __pyx_obj_8pyregion_14_region_filter_RegionList *)__pyx_v_self), ((Py_ssize_t)__pyx_v_x), ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_y));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_10RegionList_8__setitem__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, Py_ssize_t __pyx_v_x, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_y) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "pyregion/_region_filter.pyx":268
 * 
 *     def __setitem__(self, Py_ssize_t x, RegionBase y):
 *         self.child_regions[x] = y             # <<<<<<<<<<<<<<
 * 
 *     def __delitem__(self, Py_ssize_t x):
 */
  if (unlikely(__Pyx_SetItemInt(__pyx_v_self->child_regions, __pyx_v_x, ((PyObject *)__pyx_v_y), Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1) < 0)) __PYX_ERR(1, 268, __pyx_L1_error)

  /* "pyregion/_region_filter.pyx":267
 *         return self.child_regions[x]
 * 
 *     def __setitem__(self, Py_ssize_t x, RegionBase y):             # <<<<<<<<<<<<<<
 *         self.child_regions[x] = y
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionList.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":270
 *         self.child_regions[x] = y
 * 
 *     def __delitem__(self, Py_ssize_t x):             # <<<<<<<<<<<<<<
 *         del self.child_regions[x]
 * 
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_10RegionList_11__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_x); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_10RegionList_11__delitem__(PyObject *__pyx_v_self, PyObject *__pyx_arg_x) {
  Py_ssize_t __pyx_v_x;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__delitem__ (wrapper)", 0);
  assert(__pyx_arg_x); {
    __pyx_v_x = __Pyx_PyIndex_AsSsize_t(__pyx_arg_x); if (unlikely((__pyx_v_x == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 270, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionList.__delitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionList_10__delitem__(((struct __pyx_obj_8pyregion_14_region_filter_RegionList *)__pyx_v_self), ((Py_ssize_t)__pyx_v_x));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_10RegionList_10__delitem__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, Py_ssize_t __pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__delitem__", 0);

  /* "pyregion/_region_filter.pyx":271
 * 
 *     def __delitem__(self, Py_ssize_t x):
 *         del self.child_regions[x]             # <<<<<<<<<<<<<<
 * 
 *     def __contains__(self, RegionBase x):
 */
  if (unlikely(__Pyx_DelItemInt(__pyx_v_self->child_regions, __pyx_v_x, Py_ssize_t, 1, PyInt_FromSsize_t, 0, 1, 1) < 0)) __PYX_ERR(1, 271, __pyx_L1_error)

  /* "pyregion/_region_filter.pyx":270
 *         self.child_regions[x] = y
 * 
 *     def __delitem__(self, Py_ssize_t x):             # <<<<<<<<<<<<<<
 *         del self.child_regions[x]
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionList.__delitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":273
 *         del self.child_regions[x]
 * 
 *     def __contains__(self, RegionBase x):             # <<<<<<<<<<<<<<
 *         return x in self.child_regions
 * 
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_10RegionList_13__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_x); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_10RegionList_13__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__contains__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_8pyregion_14_region_filter_RegionBase, 1, "x", 0))) __PYX_ERR(1, 273, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionList_12__contains__(((struct __pyx_obj_8pyregion_14_region_filter_RegionList *)__pyx_v_self), ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_x));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_10RegionList_12__contains__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_x) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__contains__", 0);

  /* "pyregion/_region_filter.pyx":274
 * 
 *     def __contains__(self, RegionBase x):
 *         return x in self.child_regions             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_t_1 = (__Pyx_PySequence_ContainsTF(((PyObject *)__pyx_v_x), __pyx_v_self->child_regions, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(1, 274, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":273
 *         del self.child_regions[x]
 * 
 *     def __contains__(self, RegionBase x):             # <<<<<<<<<<<<<<
 *         return x in self.child_regions
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionList.__contains__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":276
 *         return x in self.child_regions
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return repr(self.child_regions)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionList_15__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionList_15__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionList_14__repr__(((struct __pyx_obj_8pyregion_14_region_filter_RegionList *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionList_14__repr__(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "pyregion/_region_filter.pyx":277
 * 
 *     def __repr__(self):
 *         return repr(self.child_regions)             # <<<<<<<<<<<<<<
 * 
 *     def asList(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_self->child_regions;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Repr(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":276
 *         return x in self.child_regions
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return repr(self.child_regions)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyregion._region_filter.RegionList.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":279
 *         return repr(self.child_regions)
 * 
 *     def asList(self):             # <<<<<<<<<<<<<<
 *         return self.child_regions
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionList_17asList(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionList_17asList(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("asList (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionList_16asList(((struct __pyx_obj_8pyregion_14_region_filter_RegionList *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionList_16asList(struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("asList", 0);

  /* "pyregion/_region_filter.pyx":280
 * 
 *     def asList(self):
 *         return self.child_regions             # <<<<<<<<<<<<<<
 * 
 * cdef class RegionOrList(RegionList):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->child_regions);
  __pyx_r = __pyx_v_self->child_regions;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":279
 *         return repr(self.child_regions)
 * 
 *     def asList(self):             # <<<<<<<<<<<<<<
 *         return self.child_regions
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionList_19__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionList_19__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionList_18__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_RegionList *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionList_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionList.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionList_21__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10RegionList_21__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10RegionList_20__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_RegionList *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10RegionList_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionList *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionList.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":286
 *     >>> r = RegionOrList(r1, r2, r3, r4, ...)
 *     """
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef c_python.PyListObject *child_regions
 *         cdef int i, n
 */

static npy_bool __pyx_f_8pyregion_14_region_filter_12RegionOrList__inside(struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *__pyx_v_self, double __pyx_v_x, double __pyx_v_y) {
  PyListObject *__pyx_v_child_regions;
  int __pyx_v_i;
  int __pyx_v_n;
  npy_bool __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  void *__pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("_inside", 0);

  /* "pyregion/_region_filter.pyx":290
 *         cdef int i, n
 * 
 *         child_regions = <c_python.PyListObject *> self.child_regions             # <<<<<<<<<<<<<<
 *         n = c_python.PyList_GET_SIZE(child_regions)
 *         for i from 0 <= i < n:
 */
  __pyx_v_child_regions = ((PyListObject *)__pyx_v_self->__pyx_base.child_regions);

  /* "pyregion/_region_filter.pyx":291
 * 
 *         child_regions = <c_python.PyListObject *> self.child_regions
 *         n = c_python.PyList_GET_SIZE(child_regions)             # <<<<<<<<<<<<<<
 *         for i from 0 <= i < n:
 *             if (<RegionBase> c_python.PyList_GET_ITEM(child_regions, i))._inside(x, y):
 */
  __pyx_v_n = PyList_GET_SIZE(__pyx_v_child_regions);

  /* "pyregion/_region_filter.pyx":292
 *         child_regions = <c_python.PyListObject *> self.child_regions
 *         n = c_python.PyList_GET_SIZE(child_regions)
 *         for i from 0 <= i < n:             # <<<<<<<<<<<<<<
 *             if (<RegionBase> c_python.PyList_GET_ITEM(child_regions, i))._inside(x, y):
 *                 return 1
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_1; __pyx_v_i++) {

    /* "pyregion/_region_filter.pyx":293
 *         n = c_python.PyList_GET_SIZE(child_regions)
 *         for i from 0 <= i < n:
 *             if (<RegionBase> c_python.PyList_GET_ITEM(child_regions, i))._inside(x, y):             # <<<<<<<<<<<<<<
 *                 return 1
 *         return 0
 */
    __pyx_t_2 = PyList_GET_ITEM(__pyx_v_child_regions, __pyx_v_i);
    __pyx_t_3 = (((struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *)((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_t_2)->__pyx_vtab)->_inside(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_t_2), __pyx_v_x, __pyx_v_y) != 0);
    if (__pyx_t_3) {

      /* "pyregion/_region_filter.pyx":294
 *         for i from 0 <= i < n:
 *             if (<RegionBase> c_python.PyList_GET_ITEM(child_regions, i))._inside(x, y):
 *                 return 1             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
      __pyx_r = 1;
      goto __pyx_L0;

      /* "pyregion/_region_filter.pyx":293
 *         n = c_python.PyList_GET_SIZE(child_regions)
 *         for i from 0 <= i < n:
 *             if (<RegionBase> c_python.PyList_GET_ITEM(child_regions, i))._inside(x, y):             # <<<<<<<<<<<<<<
 *                 return 1
 *         return 0
 */
    }
  }

  /* "pyregion/_region_filter.pyx":295
 *             if (<RegionBase> c_python.PyList_GET_ITEM(child_regions, i))._inside(x, y):
 *                 return 1
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":286
 *     >>> r = RegionOrList(r1, r2, r3, r4, ...)
 *     """
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef c_python.PyListObject *child_regions
 *         cdef int i, n
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":297
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "Or" + repr(self.child_regions)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_12RegionOrList_1__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_12RegionOrList_1__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_12RegionOrList___repr__(((struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_12RegionOrList___repr__(struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "pyregion/_region_filter.pyx":298
 * 
 *     def __repr__(self):
 *         return "Or" + repr(self.child_regions)             # <<<<<<<<<<<<<<
 * 
 * cdef class RegionAndList(RegionList):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_self->__pyx_base.child_regions;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Repr(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_n_s_Or, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":297
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "Or" + repr(self.child_regions)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyregion._region_filter.RegionOrList.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_12RegionOrList_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_12RegionOrList_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_12RegionOrList_2__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_12RegionOrList_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionOrList.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_12RegionOrList_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_12RegionOrList_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_12RegionOrList_4__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_12RegionOrList_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionOrList.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":305
 *     """
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef c_python.PyListObject *child_regions
 *         cdef int i, n
 */

static npy_bool __pyx_f_8pyregion_14_region_filter_13RegionAndList__inside(struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *__pyx_v_self, double __pyx_v_x, double __pyx_v_y) {
  PyListObject *__pyx_v_child_regions;
  int __pyx_v_i;
  int __pyx_v_n;
  npy_bool __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  void *__pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("_inside", 0);

  /* "pyregion/_region_filter.pyx":309
 *         cdef int i, n
 * 
 *         child_regions = <c_python.PyListObject *> self.child_regions             # <<<<<<<<<<<<<<
 *         n = c_python.PyList_GET_SIZE(child_regions)
 *         for i from 0 <= i < n:
 */
  __pyx_v_child_regions = ((PyListObject *)__pyx_v_self->__pyx_base.child_regions);

  /* "pyregion/_region_filter.pyx":310
 * 
 *         child_regions = <c_python.PyListObject *> self.child_regions
 *         n = c_python.PyList_GET_SIZE(child_regions)             # <<<<<<<<<<<<<<
 *         for i from 0 <= i < n:
 *             if not (<RegionBase> c_python.PyList_GET_ITEM(child_regions, i))._inside(x, y):
 */
  __pyx_v_n = PyList_GET_SIZE(__pyx_v_child_regions);

  /* "pyregion/_region_filter.pyx":311
 *         child_regions = <c_python.PyListObject *> self.child_regions
 *         n = c_python.PyList_GET_SIZE(child_regions)
 *         for i from 0 <= i < n:             # <<<<<<<<<<<<<<
 *             if not (<RegionBase> c_python.PyList_GET_ITEM(child_regions, i))._inside(x, y):
 *                 return 0
 */
  __pyx_t_1 = __pyx_v_n;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_1; __pyx_v_i++) {

    /* "pyregion/_region_filter.pyx":312
 *         n = c_python.PyList_GET_SIZE(child_regions)
 *         for i from 0 <= i < n:
 *             if not (<RegionBase> c_python.PyList_GET_ITEM(child_regions, i))._inside(x, y):             # <<<<<<<<<<<<<<
 *                 return 0
 *         return 1
 */
    __pyx_t_2 = PyList_GET_ITEM(__pyx_v_child_regions, __pyx_v_i);
    __pyx_t_3 = ((!(((struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *)((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_t_2)->__pyx_vtab)->_inside(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_t_2), __pyx_v_x, __pyx_v_y) != 0)) != 0);
    if (__pyx_t_3) {

      /* "pyregion/_region_filter.pyx":313
 *         for i from 0 <= i < n:
 *             if not (<RegionBase> c_python.PyList_GET_ITEM(child_regions, i))._inside(x, y):
 *                 return 0             # <<<<<<<<<<<<<<
 *         return 1
 * 
 */
      __pyx_r = 0;
      goto __pyx_L0;

      /* "pyregion/_region_filter.pyx":312
 *         n = c_python.PyList_GET_SIZE(child_regions)
 *         for i from 0 <= i < n:
 *             if not (<RegionBase> c_python.PyList_GET_ITEM(child_regions, i))._inside(x, y):             # <<<<<<<<<<<<<<
 *                 return 0
 *         return 1
 */
    }
  }

  /* "pyregion/_region_filter.pyx":314
 *             if not (<RegionBase> c_python.PyList_GET_ITEM(child_regions, i))._inside(x, y):
 *                 return 0
 *         return 1             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":305
 *     """
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef c_python.PyListObject *child_regions
 *         cdef int i, n
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":316
 *         return 1
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "And" + repr(self.child_regions)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_13RegionAndList_1__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_13RegionAndList_1__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_13RegionAndList___repr__(((struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_13RegionAndList___repr__(struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "pyregion/_region_filter.pyx":317
 * 
 *     def __repr__(self):
 *         return "And" + repr(self.child_regions)             # <<<<<<<<<<<<<<
 * 
 * def RegionAnd(RegionBase region1, RegionBase region2):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_self->__pyx_base.child_regions;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Repr(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_n_s_And, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":316
 *         return 1
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "And" + repr(self.child_regions)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pyregion._region_filter.RegionAndList.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_13RegionAndList_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_13RegionAndList_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_13RegionAndList_2__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_13RegionAndList_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionAndList.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_13RegionAndList_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_13RegionAndList_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_13RegionAndList_4__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_13RegionAndList_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.RegionAndList.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":319
 *         return "And" + repr(self.child_regions)
 * 
 * def RegionAnd(RegionBase region1, RegionBase region2):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionAnd(reg1, reg2)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_1RegionAnd(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8pyregion_14_region_filter_RegionAnd[] = "\n    >>> r = RegionAnd(reg1, reg2)\n    ";
static PyMethodDef __pyx_mdef_8pyregion_14_region_filter_1RegionAnd = {"RegionAnd", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_1RegionAnd, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8pyregion_14_region_filter_RegionAnd};
static PyObject *__pyx_pw_8pyregion_14_region_filter_1RegionAnd(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_region1 = 0;
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_region2 = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("RegionAnd (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_region1,&__pyx_n_s_region2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_region1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_region2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("RegionAnd", 1, 2, 2, 1); __PYX_ERR(1, 319, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "RegionAnd") < 0)) __PYX_ERR(1, 319, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_region1 = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)values[0]);
    __pyx_v_region2 = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("RegionAnd", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 319, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionAnd", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_region1), __pyx_ptype_8pyregion_14_region_filter_RegionBase, 1, "region1", 0))) __PYX_ERR(1, 319, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_region2), __pyx_ptype_8pyregion_14_region_filter_RegionBase, 1, "region2", 0))) __PYX_ERR(1, 319, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_RegionAnd(__pyx_self, __pyx_v_region1, __pyx_v_region2);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_RegionAnd(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_region1, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_region2) {
  PyObject *__pyx_v_region1_list = NULL;
  PyObject *__pyx_v_region2_list = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("RegionAnd", 0);

  /* "pyregion/_region_filter.pyx":323
 *     >>> r = RegionAnd(reg1, reg2)
 *     """
 *     if isinstance(region1, RegionAndList):             # <<<<<<<<<<<<<<
 *         region1_list = region1.asList()
 *     else:
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_region1), __pyx_ptype_8pyregion_14_region_filter_RegionAndList); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "pyregion/_region_filter.pyx":324
 *     """
 *     if isinstance(region1, RegionAndList):
 *         region1_list = region1.asList()             # <<<<<<<<<<<<<<
 *     else:
 *         region1_list = [region1]
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_region1), __pyx_n_s_asList); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 324, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 324, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 324, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_region1_list = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyregion/_region_filter.pyx":323
 *     >>> r = RegionAnd(reg1, reg2)
 *     """
 *     if isinstance(region1, RegionAndList):             # <<<<<<<<<<<<<<
 *         region1_list = region1.asList()
 *     else:
 */
    goto __pyx_L3;
  }

  /* "pyregion/_region_filter.pyx":326
 *         region1_list = region1.asList()
 *     else:
 *         region1_list = [region1]             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(region2, RegionAndList):
 */
  /*else*/ {
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 326, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)__pyx_v_region1));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_region1));
    PyList_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_region1));
    __pyx_v_region1_list = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "pyregion/_region_filter.pyx":328
 *         region1_list = [region1]
 * 
 *     if isinstance(region2, RegionAndList):             # <<<<<<<<<<<<<<
 *         region2_list = region2.asList()
 *     else:
 */
  __pyx_t_2 = __Pyx_TypeCheck(((PyObject *)__pyx_v_region2), __pyx_ptype_8pyregion_14_region_filter_RegionAndList); 
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "pyregion/_region_filter.pyx":329
 * 
 *     if isinstance(region2, RegionAndList):
 *         region2_list = region2.asList()             # <<<<<<<<<<<<<<
 *     else:
 *         region2_list = [region2]
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_region2), __pyx_n_s_asList); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 329, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 329, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_region2_list = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyregion/_region_filter.pyx":328
 *         region1_list = [region1]
 * 
 *     if isinstance(region2, RegionAndList):             # <<<<<<<<<<<<<<
 *         region2_list = region2.asList()
 *     else:
 */
    goto __pyx_L4;
  }

  /* "pyregion/_region_filter.pyx":331
 *         region2_list = region2.asList()
 *     else:
 *         region2_list = [region2]             # <<<<<<<<<<<<<<
 * 
 *     return RegionAndList(*(region1_list + region2_list))
 */
  /*else*/ {
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)__pyx_v_region2));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_region2));
    PyList_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_region2));
    __pyx_v_region2_list = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_L4:;

  /* "pyregion/_region_filter.pyx":333
 *         region2_list = [region2]
 * 
 *     return RegionAndList(*(region1_list + region2_list))             # <<<<<<<<<<<<<<
 * 
 * def RegionOr(RegionBase region1, RegionBase region2):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyNumber_Add(__pyx_v_region1_list, __pyx_v_region2_list); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PySequence_Tuple(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8pyregion_14_region_filter_RegionAndList), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":319
 *         return "And" + repr(self.child_regions)
 * 
 * def RegionAnd(RegionBase region1, RegionBase region2):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionAnd(reg1, reg2)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyregion._region_filter.RegionAnd", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_region1_list);
  __Pyx_XDECREF(__pyx_v_region2_list);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":335
 *     return RegionAndList(*(region1_list + region2_list))
 * 
 * def RegionOr(RegionBase region1, RegionBase region2):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionOr(reg1, reg2)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_3RegionOr(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_8pyregion_14_region_filter_2RegionOr[] = "\n    >>> r = RegionOr(reg1, reg2)\n    ";
static PyMethodDef __pyx_mdef_8pyregion_14_region_filter_3RegionOr = {"RegionOr", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_3RegionOr, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8pyregion_14_region_filter_2RegionOr};
static PyObject *__pyx_pw_8pyregion_14_region_filter_3RegionOr(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_region1 = 0;
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_region2 = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("RegionOr (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_region1,&__pyx_n_s_region2,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_region1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_region2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("RegionOr", 1, 2, 2, 1); __PYX_ERR(1, 335, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "RegionOr") < 0)) __PYX_ERR(1, 335, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_region1 = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)values[0]);
    __pyx_v_region2 = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("RegionOr", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 335, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.RegionOr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_region1), __pyx_ptype_8pyregion_14_region_filter_RegionBase, 1, "region1", 0))) __PYX_ERR(1, 335, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_region2), __pyx_ptype_8pyregion_14_region_filter_RegionBase, 1, "region2", 0))) __PYX_ERR(1, 335, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_2RegionOr(__pyx_self, __pyx_v_region1, __pyx_v_region2);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_2RegionOr(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_region1, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_region2) {
  PyObject *__pyx_v_region1_list = NULL;
  PyObject *__pyx_v_region2_list = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("RegionOr", 0);

  /* "pyregion/_region_filter.pyx":339
 *     >>> r = RegionOr(reg1, reg2)
 *     """
 *     if isinstance(region1, RegionOrList):             # <<<<<<<<<<<<<<
 *         region1_list = region1.asList()
 *     else:
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_region1), __pyx_ptype_8pyregion_14_region_filter_RegionOrList); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "pyregion/_region_filter.pyx":340
 *     """
 *     if isinstance(region1, RegionOrList):
 *         region1_list = region1.asList()             # <<<<<<<<<<<<<<
 *     else:
 *         region1_list = [region1]
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_region1), __pyx_n_s_asList); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 340, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 340, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 340, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_region1_list = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyregion/_region_filter.pyx":339
 *     >>> r = RegionOr(reg1, reg2)
 *     """
 *     if isinstance(region1, RegionOrList):             # <<<<<<<<<<<<<<
 *         region1_list = region1.asList()
 *     else:
 */
    goto __pyx_L3;
  }

  /* "pyregion/_region_filter.pyx":342
 *         region1_list = region1.asList()
 *     else:
 *         region1_list = [region1]             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(region2, RegionOrList):
 */
  /*else*/ {
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 342, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)__pyx_v_region1));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_region1));
    PyList_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_region1));
    __pyx_v_region1_list = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "pyregion/_region_filter.pyx":344
 *         region1_list = [region1]
 * 
 *     if isinstance(region2, RegionOrList):             # <<<<<<<<<<<<<<
 *         region2_list = region2.asList()
 *     else:
 */
  __pyx_t_2 = __Pyx_TypeCheck(((PyObject *)__pyx_v_region2), __pyx_ptype_8pyregion_14_region_filter_RegionOrList); 
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "pyregion/_region_filter.pyx":345
 * 
 *     if isinstance(region2, RegionOrList):
 *         region2_list = region2.asList()             # <<<<<<<<<<<<<<
 *     else:
 *         region2_list = [region2]
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_region2), __pyx_n_s_asList); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 345, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 345, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 345, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_region2_list = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "pyregion/_region_filter.pyx":344
 *         region1_list = [region1]
 * 
 *     if isinstance(region2, RegionOrList):             # <<<<<<<<<<<<<<
 *         region2_list = region2.asList()
 *     else:
 */
    goto __pyx_L4;
  }

  /* "pyregion/_region_filter.pyx":347
 *         region2_list = region2.asList()
 *     else:
 *         region2_list = [region2]             # <<<<<<<<<<<<<<
 * 
 *     return RegionOrList(*(region1_list + region2_list))
 */
  /*else*/ {
    __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 347, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)__pyx_v_region2));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_region2));
    PyList_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_region2));
    __pyx_v_region2_list = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __pyx_L4:;

  /* "pyregion/_region_filter.pyx":349
 *         region2_list = [region2]
 * 
 *     return RegionOrList(*(region1_list + region2_list))             # <<<<<<<<<<<<<<
 * 
 * cdef class Transform(RegionBase):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyNumber_Add(__pyx_v_region1_list, __pyx_v_region2_list); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PySequence_Tuple(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8pyregion_14_region_filter_RegionOrList), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":335
 *     return RegionAndList(*(region1_list + region2_list))
 * 
 * def RegionOr(RegionBase region1, RegionBase region2):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionOr(reg1, reg2)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyregion._region_filter.RegionOr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_region1_list);
  __Pyx_XDECREF(__pyx_v_region2_list);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":354
 *     cdef RegionBase child_region
 * 
 *     def __init__(self, RegionBase child_region):             # <<<<<<<<<<<<<<
 *         self.child_region = child_region
 * 
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_9Transform_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_9Transform_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_child_region = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_child_region,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_child_region)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 354, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_child_region = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 354, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.Transform.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_child_region), __pyx_ptype_8pyregion_14_region_filter_RegionBase, 1, "child_region", 0))) __PYX_ERR(1, 354, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_9Transform___init__(((struct __pyx_obj_8pyregion_14_region_filter_Transform *)__pyx_v_self), __pyx_v_child_region);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_9Transform___init__(struct __pyx_obj_8pyregion_14_region_filter_Transform *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_child_region) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyregion/_region_filter.pyx":355
 * 
 *     def __init__(self, RegionBase child_region):
 *         self.child_region = child_region             # <<<<<<<<<<<<<<
 * 
 *     property child:
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_child_region));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_child_region));
  __Pyx_GOTREF(__pyx_v_self->child_region);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->child_region));
  __pyx_v_self->child_region = __pyx_v_child_region;

  /* "pyregion/_region_filter.pyx":354
 *     cdef RegionBase child_region
 * 
 *     def __init__(self, RegionBase child_region):             # <<<<<<<<<<<<<<
 *         self.child_region = child_region
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":358
 * 
 *     property child:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.child_region
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_9Transform_5child_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_9Transform_5child_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_9Transform_5child___get__(((struct __pyx_obj_8pyregion_14_region_filter_Transform *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_9Transform_5child___get__(struct __pyx_obj_8pyregion_14_region_filter_Transform *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pyregion/_region_filter.pyx":359
 *     property child:
 *         def __get__(self):
 *             return self.child_region             # <<<<<<<<<<<<<<
 * 
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->child_region));
  __pyx_r = ((PyObject *)__pyx_v_self->child_region);
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":358
 * 
 *     property child:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.child_region
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":361
 *             return self.child_region
 * 
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):             # <<<<<<<<<<<<<<
 *         xp[0] = x
 *         yp[0] = y
 */

static int __pyx_f_8pyregion_14_region_filter_9Transform__transform(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Transform *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double *__pyx_v_xp, double *__pyx_v_yp) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_transform", 0);

  /* "pyregion/_region_filter.pyx":362
 * 
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):
 *         xp[0] = x             # <<<<<<<<<<<<<<
 *         yp[0] = y
 * 
 */
  (__pyx_v_xp[0]) = __pyx_v_x;

  /* "pyregion/_region_filter.pyx":363
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):
 *         xp[0] = x
 *         yp[0] = y             # <<<<<<<<<<<<<<
 * 
 *     cdef npy_bool _inside(self, double x, double y):
 */
  (__pyx_v_yp[0]) = __pyx_v_y;

  /* "pyregion/_region_filter.pyx":361
 *             return self.child_region
 * 
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):             # <<<<<<<<<<<<<<
 *         xp[0] = x
 *         yp[0] = y
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":365
 *         yp[0] = y
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef double xp, yp
 *         cdef npy_bool r
 */

static npy_bool __pyx_f_8pyregion_14_region_filter_9Transform__inside(struct __pyx_obj_8pyregion_14_region_filter_Transform *__pyx_v_self, double __pyx_v_x, double __pyx_v_y) {
  double __pyx_v_xp;
  double __pyx_v_yp;
  npy_bool __pyx_v_r;
  npy_bool __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_inside", 0);

  /* "pyregion/_region_filter.pyx":369
 *         cdef npy_bool r
 * 
 *         self._transform(x, y, &xp, &yp)             # <<<<<<<<<<<<<<
 *         r = self.child_region._inside(xp, yp)
 * 
 */
  ((struct __pyx_vtabstruct_8pyregion_14_region_filter_Transform *)__pyx_v_self->__pyx_base.__pyx_vtab)->_transform(__pyx_v_self, __pyx_v_x, __pyx_v_y, (&__pyx_v_xp), (&__pyx_v_yp));

  /* "pyregion/_region_filter.pyx":370
 * 
 *         self._transform(x, y, &xp, &yp)
 *         r = self.child_region._inside(xp, yp)             # <<<<<<<<<<<<<<
 * 
 *         return r
 */
  __pyx_v_r = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase *)__pyx_v_self->child_region->__pyx_vtab)->_inside(__pyx_v_self->child_region, __pyx_v_xp, __pyx_v_yp);

  /* "pyregion/_region_filter.pyx":372
 *         r = self.child_region._inside(xp, yp)
 * 
 *         return r             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_r;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":365
 *         yp[0] = y
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef double xp, yp
 *         cdef npy_bool r
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_9Transform_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_9Transform_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_9Transform_2__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Transform *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_9Transform_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Transform *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Transform.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_9Transform_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_9Transform_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_9Transform_4__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Transform *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_9Transform_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Transform *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Transform.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":395
 *     cdef double origin_x, origin_y
 * 
 *     def __init__(self, RegionBase child_region,             # <<<<<<<<<<<<<<
 *                  double degree, double origin_x, double origin_y):
 *         cdef double theta
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_7Rotated_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_7Rotated_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_child_region = 0;
  double __pyx_v_degree;
  double __pyx_v_origin_x;
  double __pyx_v_origin_y;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_child_region,&__pyx_n_s_degree,&__pyx_n_s_origin_x,&__pyx_n_s_origin_y,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_child_region)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_degree)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, 1); __PYX_ERR(1, 395, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_origin_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, 2); __PYX_ERR(1, 395, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_origin_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, 3); __PYX_ERR(1, 395, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 395, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_child_region = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)values[0]);
    __pyx_v_degree = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_degree == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 396, __pyx_L3_error)
    __pyx_v_origin_x = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_origin_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 396, __pyx_L3_error)
    __pyx_v_origin_y = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_origin_y == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 396, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 395, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.Rotated.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_child_region), __pyx_ptype_8pyregion_14_region_filter_RegionBase, 1, "child_region", 0))) __PYX_ERR(1, 395, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_7Rotated___init__(((struct __pyx_obj_8pyregion_14_region_filter_Rotated *)__pyx_v_self), __pyx_v_child_region, __pyx_v_degree, __pyx_v_origin_x, __pyx_v_origin_y);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_7Rotated___init__(struct __pyx_obj_8pyregion_14_region_filter_Rotated *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_child_region, double __pyx_v_degree, double __pyx_v_origin_x, double __pyx_v_origin_y) {
  double __pyx_v_theta;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyregion/_region_filter.pyx":399
 *         cdef double theta
 * 
 *         Transform.__init__(self, child_region)             # <<<<<<<<<<<<<<
 * 
 *         theta = degree / 180. * M_PI  #3.1415926
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_8pyregion_14_region_filter_Transform), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 399, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_child_region)};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 399, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_child_region)};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 399, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 399, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, ((PyObject *)__pyx_v_self));
    __Pyx_INCREF(((PyObject *)__pyx_v_child_region));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_child_region));
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, ((PyObject *)__pyx_v_child_region));
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 399, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":401
 *         Transform.__init__(self, child_region)
 * 
 *         theta = degree / 180. * M_PI  #3.1415926             # <<<<<<<<<<<<<<
 *         self.sin_theta = sin(theta)
 *         self.cos_theta = cos(theta)
 */
  __pyx_v_theta = ((__pyx_v_degree / 180.) * M_PI);

  /* "pyregion/_region_filter.pyx":402
 * 
 *         theta = degree / 180. * M_PI  #3.1415926
 *         self.sin_theta = sin(theta)             # <<<<<<<<<<<<<<
 *         self.cos_theta = cos(theta)
 * 
 */
  __pyx_v_self->sin_theta = sin(__pyx_v_theta);

  /* "pyregion/_region_filter.pyx":403
 *         theta = degree / 180. * M_PI  #3.1415926
 *         self.sin_theta = sin(theta)
 *         self.cos_theta = cos(theta)             # <<<<<<<<<<<<<<
 * 
 *         self.origin_x = origin_x
 */
  __pyx_v_self->cos_theta = cos(__pyx_v_theta);

  /* "pyregion/_region_filter.pyx":405
 *         self.cos_theta = cos(theta)
 * 
 *         self.origin_x = origin_x             # <<<<<<<<<<<<<<
 *         self.origin_y = origin_y
 * 
 */
  __pyx_v_self->origin_x = __pyx_v_origin_x;

  /* "pyregion/_region_filter.pyx":406
 * 
 *         self.origin_x = origin_x
 *         self.origin_y = origin_y             # <<<<<<<<<<<<<<
 * 
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):
 */
  __pyx_v_self->origin_y = __pyx_v_origin_y;

  /* "pyregion/_region_filter.pyx":395
 *     cdef double origin_x, origin_y
 * 
 *     def __init__(self, RegionBase child_region,             # <<<<<<<<<<<<<<
 *                  double degree, double origin_x, double origin_y):
 *         cdef double theta
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyregion._region_filter.Rotated.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":408
 *         self.origin_y = origin_y
 * 
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):             # <<<<<<<<<<<<<<
 *         cdef double x1, x2, y1, y2
 *         cdef double st, ct, ox, oy
 */

static int __pyx_f_8pyregion_14_region_filter_7Rotated__transform(struct __pyx_obj_8pyregion_14_region_filter_Rotated *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double *__pyx_v_xp, double *__pyx_v_yp) {
  double __pyx_v_x1;
  double __pyx_v_x2;
  double __pyx_v_y1;
  double __pyx_v_y2;
  double __pyx_v_st;
  double __pyx_v_ct;
  double __pyx_v_ox;
  double __pyx_v_oy;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __Pyx_RefNannySetupContext("_transform", 0);

  /* "pyregion/_region_filter.pyx":412
 *         cdef double st, ct, ox, oy
 * 
 *         st = self.sin_theta             # <<<<<<<<<<<<<<
 *         ct = self.cos_theta
 *         ox = self.origin_x
 */
  __pyx_t_1 = __pyx_v_self->sin_theta;
  __pyx_v_st = __pyx_t_1;

  /* "pyregion/_region_filter.pyx":413
 * 
 *         st = self.sin_theta
 *         ct = self.cos_theta             # <<<<<<<<<<<<<<
 *         ox = self.origin_x
 *         oy = self.origin_y
 */
  __pyx_t_1 = __pyx_v_self->cos_theta;
  __pyx_v_ct = __pyx_t_1;

  /* "pyregion/_region_filter.pyx":414
 *         st = self.sin_theta
 *         ct = self.cos_theta
 *         ox = self.origin_x             # <<<<<<<<<<<<<<
 *         oy = self.origin_y
 * 
 */
  __pyx_t_1 = __pyx_v_self->origin_x;
  __pyx_v_ox = __pyx_t_1;

  /* "pyregion/_region_filter.pyx":415
 *         ct = self.cos_theta
 *         ox = self.origin_x
 *         oy = self.origin_y             # <<<<<<<<<<<<<<
 * 
 *         x1 = x - ox
 */
  __pyx_t_1 = __pyx_v_self->origin_y;
  __pyx_v_oy = __pyx_t_1;

  /* "pyregion/_region_filter.pyx":417
 *         oy = self.origin_y
 * 
 *         x1 = x - ox             # <<<<<<<<<<<<<<
 *         y1 = y - oy
 * 
 */
  __pyx_v_x1 = (__pyx_v_x - __pyx_v_ox);

  /* "pyregion/_region_filter.pyx":418
 * 
 *         x1 = x - ox
 *         y1 = y - oy             # <<<<<<<<<<<<<<
 * 
 *         x2 = ct * x1 + st * y1
 */
  __pyx_v_y1 = (__pyx_v_y - __pyx_v_oy);

  /* "pyregion/_region_filter.pyx":420
 *         y1 = y - oy
 * 
 *         x2 = ct * x1 + st * y1             # <<<<<<<<<<<<<<
 *         y2 = -st * x1 + ct * y1
 * 
 */
  __pyx_v_x2 = ((__pyx_v_ct * __pyx_v_x1) + (__pyx_v_st * __pyx_v_y1));

  /* "pyregion/_region_filter.pyx":421
 * 
 *         x2 = ct * x1 + st * y1
 *         y2 = -st * x1 + ct * y1             # <<<<<<<<<<<<<<
 * 
 *         xp[0] = x2 + ox
 */
  __pyx_v_y2 = (((-__pyx_v_st) * __pyx_v_x1) + (__pyx_v_ct * __pyx_v_y1));

  /* "pyregion/_region_filter.pyx":423
 *         y2 = -st * x1 + ct * y1
 * 
 *         xp[0] = x2 + ox             # <<<<<<<<<<<<<<
 *         yp[0] = y2 + oy
 * 
 */
  (__pyx_v_xp[0]) = (__pyx_v_x2 + __pyx_v_ox);

  /* "pyregion/_region_filter.pyx":424
 * 
 *         xp[0] = x2 + ox
 *         yp[0] = y2 + oy             # <<<<<<<<<<<<<<
 * 
 * cdef class Translated(Transform):
 */
  (__pyx_v_yp[0]) = (__pyx_v_y2 + __pyx_v_oy);

  /* "pyregion/_region_filter.pyx":408
 *         self.origin_y = origin_y
 * 
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):             # <<<<<<<<<<<<<<
 *         cdef double x1, x2, y1, y2
 *         cdef double st, ct, ox, oy
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Rotated_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Rotated_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_7Rotated_2__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Rotated *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_7Rotated_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Rotated *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Rotated.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Rotated_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Rotated_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_7Rotated_4__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Rotated *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_7Rotated_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Rotated *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Rotated.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":436
 *     cdef double dy
 * 
 *     def __init__(self, RegionBase child_region,             # <<<<<<<<<<<<<<
 *                  double dx, double dy):
 *         Transform.__init__(self, child_region)
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_10Translated_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_10Translated_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_child_region = 0;
  double __pyx_v_dx;
  double __pyx_v_dy;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_child_region,&__pyx_n_s_dx,&__pyx_n_s_dy,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_child_region)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dx)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); __PYX_ERR(1, 436, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dy)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); __PYX_ERR(1, 436, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 436, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_child_region = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)values[0]);
    __pyx_v_dx = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_dx == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 437, __pyx_L3_error)
    __pyx_v_dy = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_dy == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 437, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 436, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.Translated.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_child_region), __pyx_ptype_8pyregion_14_region_filter_RegionBase, 1, "child_region", 0))) __PYX_ERR(1, 436, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10Translated___init__(((struct __pyx_obj_8pyregion_14_region_filter_Translated *)__pyx_v_self), __pyx_v_child_region, __pyx_v_dx, __pyx_v_dy);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_10Translated___init__(struct __pyx_obj_8pyregion_14_region_filter_Translated *__pyx_v_self, struct __pyx_obj_8pyregion_14_region_filter_RegionBase *__pyx_v_child_region, double __pyx_v_dx, double __pyx_v_dy) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyregion/_region_filter.pyx":438
 *     def __init__(self, RegionBase child_region,
 *                  double dx, double dy):
 *         Transform.__init__(self, child_region)             # <<<<<<<<<<<<<<
 * 
 *         self.dx = dx
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_8pyregion_14_region_filter_Transform), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_child_region)};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 438, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_child_region)};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 438, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, ((PyObject *)__pyx_v_self));
    __Pyx_INCREF(((PyObject *)__pyx_v_child_region));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_child_region));
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, ((PyObject *)__pyx_v_child_region));
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 438, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":440
 *         Transform.__init__(self, child_region)
 * 
 *         self.dx = dx             # <<<<<<<<<<<<<<
 *         self.dy = dy
 * 
 */
  __pyx_v_self->dx = __pyx_v_dx;

  /* "pyregion/_region_filter.pyx":441
 * 
 *         self.dx = dx
 *         self.dy = dy             # <<<<<<<<<<<<<<
 * 
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):
 */
  __pyx_v_self->dy = __pyx_v_dy;

  /* "pyregion/_region_filter.pyx":436
 *     cdef double dy
 * 
 *     def __init__(self, RegionBase child_region,             # <<<<<<<<<<<<<<
 *                  double dx, double dy):
 *         Transform.__init__(self, child_region)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyregion._region_filter.Translated.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":443
 *         self.dy = dy
 * 
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):             # <<<<<<<<<<<<<<
 *         xp[0] = x - self.dx
 *         yp[0] = y - self.dy
 */

static int __pyx_f_8pyregion_14_region_filter_10Translated__transform(struct __pyx_obj_8pyregion_14_region_filter_Translated *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double *__pyx_v_xp, double *__pyx_v_yp) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_transform", 0);

  /* "pyregion/_region_filter.pyx":444
 * 
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):
 *         xp[0] = x - self.dx             # <<<<<<<<<<<<<<
 *         yp[0] = y - self.dy
 * 
 */
  (__pyx_v_xp[0]) = (__pyx_v_x - __pyx_v_self->dx);

  /* "pyregion/_region_filter.pyx":445
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):
 *         xp[0] = x - self.dx
 *         yp[0] = y - self.dy             # <<<<<<<<<<<<<<
 * 
 * # Basic Shapes
 */
  (__pyx_v_yp[0]) = (__pyx_v_y - __pyx_v_self->dy);

  /* "pyregion/_region_filter.pyx":443
 *         self.dy = dy
 * 
 *     cdef int _transform(self, double x, double y, double *xp, double *yp):             # <<<<<<<<<<<<<<
 *         xp[0] = x - self.dx
 *         yp[0] = y - self.dy
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10Translated_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10Translated_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10Translated_2__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Translated *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10Translated_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Translated *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Translated.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10Translated_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10Translated_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10Translated_4__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Translated *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10Translated_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Translated *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Translated.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":461
 *     cdef double radius2
 * 
 *     cdef _set_v(self, double xc, double yc, double radius):             # <<<<<<<<<<<<<<
 *         self.xc = xc
 *         self.yc = yc
 */

static PyObject *__pyx_f_8pyregion_14_region_filter_6Circle__set_v(struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self, double __pyx_v_xc, double __pyx_v_yc, double __pyx_v_radius) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_set_v", 0);

  /* "pyregion/_region_filter.pyx":462
 * 
 *     cdef _set_v(self, double xc, double yc, double radius):
 *         self.xc = xc             # <<<<<<<<<<<<<<
 *         self.yc = yc
 *         self.radius = radius
 */
  __pyx_v_self->xc = __pyx_v_xc;

  /* "pyregion/_region_filter.pyx":463
 *     cdef _set_v(self, double xc, double yc, double radius):
 *         self.xc = xc
 *         self.yc = yc             # <<<<<<<<<<<<<<
 *         self.radius = radius
 *         self.radius2 = radius * radius
 */
  __pyx_v_self->yc = __pyx_v_yc;

  /* "pyregion/_region_filter.pyx":464
 *         self.xc = xc
 *         self.yc = yc
 *         self.radius = radius             # <<<<<<<<<<<<<<
 *         self.radius2 = radius * radius
 * 
 */
  __pyx_v_self->radius = __pyx_v_radius;

  /* "pyregion/_region_filter.pyx":465
 *         self.yc = yc
 *         self.radius = radius
 *         self.radius2 = radius * radius             # <<<<<<<<<<<<<<
 * 
 *     cdef _get_v(self):
 */
  __pyx_v_self->radius2 = (__pyx_v_radius * __pyx_v_radius);

  /* "pyregion/_region_filter.pyx":461
 *     cdef double radius2
 * 
 *     cdef _set_v(self, double xc, double yc, double radius):             # <<<<<<<<<<<<<<
 *         self.xc = xc
 *         self.yc = yc
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":467
 *         self.radius2 = radius * radius
 * 
 *     cdef _get_v(self):             # <<<<<<<<<<<<<<
 *         return (self.xc, self.yc, self.radius)
 * 
 */

static PyObject *__pyx_f_8pyregion_14_region_filter_6Circle__get_v(struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("_get_v", 0);

  /* "pyregion/_region_filter.pyx":468
 * 
 *     cdef _get_v(self):
 *         return (self.xc, self.yc, self.radius)             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, double xc, double yc, double radius,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->xc); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->yc); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->radius); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":467
 *         self.radius2 = radius * radius
 * 
 *     cdef _get_v(self):             # <<<<<<<<<<<<<<
 *         return (self.xc, self.yc, self.radius)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyregion._region_filter.Circle._get_v", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":470
 *         return (self.xc, self.yc, self.radius)
 * 
 *     def __init__(self, double xc, double yc, double radius,             # <<<<<<<<<<<<<<
 *                  RegionContext c=None):
 *         self.metric_set_origin(xc, yc, c)
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_6Circle_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_6Circle_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_xc;
  double __pyx_v_yc;
  double __pyx_v_radius;
  struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_xc,&__pyx_n_s_yc,&__pyx_n_s_radius,&__pyx_n_s_c,0};
    PyObject* values[4] = {0,0,0,0};

    /* "pyregion/_region_filter.pyx":471
 * 
 *     def __init__(self, double xc, double yc, double radius,
 *                  RegionContext c=None):             # <<<<<<<<<<<<<<
 *         self.metric_set_origin(xc, yc, c)
 *         self._set_v(xc, yc, radius)
 */
    values[3] = (PyObject *)((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_xc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_yc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, 1); __PYX_ERR(1, 470, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_radius)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, 2); __PYX_ERR(1, 470, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_c);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 470, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_xc = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_xc == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 470, __pyx_L3_error)
    __pyx_v_yc = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_yc == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 470, __pyx_L3_error)
    __pyx_v_radius = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_radius == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 470, __pyx_L3_error)
    __pyx_v_c = ((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 470, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.Circle.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_c), __pyx_ptype_8pyregion_14_region_filter_RegionContext, 1, "c", 0))) __PYX_ERR(1, 471, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_6Circle___init__(((struct __pyx_obj_8pyregion_14_region_filter_Circle *)__pyx_v_self), __pyx_v_xc, __pyx_v_yc, __pyx_v_radius, __pyx_v_c);

  /* "pyregion/_region_filter.pyx":470
 *         return (self.xc, self.yc, self.radius)
 * 
 *     def __init__(self, double xc, double yc, double radius,             # <<<<<<<<<<<<<<
 *                  RegionContext c=None):
 *         self.metric_set_origin(xc, yc, c)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_6Circle___init__(struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self, double __pyx_v_xc, double __pyx_v_yc, double __pyx_v_radius, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyregion/_region_filter.pyx":472
 *     def __init__(self, double xc, double yc, double radius,
 *                  RegionContext c=None):
 *         self.metric_set_origin(xc, yc, c)             # <<<<<<<<<<<<<<
 *         self._set_v(xc, yc, radius)
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_Circle *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.metric_set_origin(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self), __pyx_v_xc, __pyx_v_yc, __pyx_v_c); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 472, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":473
 *                  RegionContext c=None):
 *         self.metric_set_origin(xc, yc, c)
 *         self._set_v(xc, yc, radius)             # <<<<<<<<<<<<<<
 * 
 *     cdef npy_bool _inside(self, double x, double y):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_Circle *)__pyx_v_self->__pyx_base.__pyx_vtab)->_set_v(__pyx_v_self, __pyx_v_xc, __pyx_v_yc, __pyx_v_radius); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":470
 *         return (self.xc, self.yc, self.radius)
 * 
 *     def __init__(self, double xc, double yc, double radius,             # <<<<<<<<<<<<<<
 *                  RegionContext c=None):
 *         self.metric_set_origin(xc, yc, c)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Circle.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":475
 *         self._set_v(xc, yc, radius)
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef double dist2
 * 
 */

static npy_bool __pyx_f_8pyregion_14_region_filter_6Circle__inside(struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self, double __pyx_v_x, double __pyx_v_y) {
  double __pyx_v_dist2;
  npy_bool __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_inside", 0);

  /* "pyregion/_region_filter.pyx":478
 *         cdef double dist2
 * 
 *         dist2 = ((x - self.xc) * self.m.g_x) ** 2 + ((y - self.yc) * self.m.g_y) ** 2             # <<<<<<<<<<<<<<
 *         return (dist2 <= self.radius2)
 * 
 */
  __pyx_v_dist2 = (pow(((__pyx_v_x - __pyx_v_self->xc) * __pyx_v_self->__pyx_base.m.g_x), 2.0) + pow(((__pyx_v_y - __pyx_v_self->yc) * __pyx_v_self->__pyx_base.m.g_y), 2.0));

  /* "pyregion/_region_filter.pyx":479
 * 
 *         dist2 = ((x - self.xc) * self.m.g_x) ** 2 + ((y - self.yc) * self.m.g_y) ** 2
 *         return (dist2 <= self.radius2)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_r = (__pyx_v_dist2 <= __pyx_v_self->radius2);
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":475
 *         self._set_v(xc, yc, radius)
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef double dist2
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":481
 *         return (dist2 <= self.radius2)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "Circle(%f, %f, %f)" % (self.xc, self.yc, self.radius)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_6Circle_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_6Circle_3__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_6Circle_2__repr__(((struct __pyx_obj_8pyregion_14_region_filter_Circle *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_6Circle_2__repr__(struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "pyregion/_region_filter.pyx":482
 * 
 *     def __repr__(self):
 *         return "Circle(%f, %f, %f)" % (self.xc, self.yc, self.radius)             # <<<<<<<<<<<<<<
 * 
 * cdef class Ellipse(RegionBase):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->xc); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->yc); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->radius); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Circle_f_f_f, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":481
 *         return (dist2 <= self.radius2)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "Circle(%f, %f, %f)" % (self.xc, self.yc, self.radius)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pyregion._region_filter.Circle.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_6Circle_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_6Circle_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_6Circle_4__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Circle *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_6Circle_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Circle.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_6Circle_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_6Circle_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_6Circle_6__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Circle *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_6Circle_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Circle *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Circle.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":499
 *     cdef double radius_major_2_radius_minor_2
 * 
 *     def __init__(self, double xc, double yc,             # <<<<<<<<<<<<<<
 *                  double radius_major, double radius_minor,
 *                  RegionContext c=None):
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_7Ellipse_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_7Ellipse_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_xc;
  double __pyx_v_yc;
  double __pyx_v_radius_major;
  double __pyx_v_radius_minor;
  struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_xc,&__pyx_n_s_yc,&__pyx_n_s_radius_major,&__pyx_n_s_radius_minor,&__pyx_n_s_c,0};
    PyObject* values[5] = {0,0,0,0,0};

    /* "pyregion/_region_filter.pyx":501
 *     def __init__(self, double xc, double yc,
 *                  double radius_major, double radius_minor,
 *                  RegionContext c=None):             # <<<<<<<<<<<<<<
 *         # check inside
 *         # (x-xc)**2/radius_major**2 + (y-yc)**2/radius_minor**2 < 1
 */
    values[4] = (PyObject *)((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_xc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_yc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, 1); __PYX_ERR(1, 499, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_radius_major)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, 2); __PYX_ERR(1, 499, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_radius_minor)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, 3); __PYX_ERR(1, 499, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_c);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 499, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_xc = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_xc == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 499, __pyx_L3_error)
    __pyx_v_yc = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_yc == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 499, __pyx_L3_error)
    __pyx_v_radius_major = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_radius_major == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 500, __pyx_L3_error)
    __pyx_v_radius_minor = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_radius_minor == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 500, __pyx_L3_error)
    __pyx_v_c = ((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)values[4]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 499, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.Ellipse.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_c), __pyx_ptype_8pyregion_14_region_filter_RegionContext, 1, "c", 0))) __PYX_ERR(1, 501, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_7Ellipse___init__(((struct __pyx_obj_8pyregion_14_region_filter_Ellipse *)__pyx_v_self), __pyx_v_xc, __pyx_v_yc, __pyx_v_radius_major, __pyx_v_radius_minor, __pyx_v_c);

  /* "pyregion/_region_filter.pyx":499
 *     cdef double radius_major_2_radius_minor_2
 * 
 *     def __init__(self, double xc, double yc,             # <<<<<<<<<<<<<<
 *                  double radius_major, double radius_minor,
 *                  RegionContext c=None):
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_7Ellipse___init__(struct __pyx_obj_8pyregion_14_region_filter_Ellipse *__pyx_v_self, double __pyx_v_xc, double __pyx_v_yc, double __pyx_v_radius_major, double __pyx_v_radius_minor, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyregion/_region_filter.pyx":506
 *         # radius_minor**2*(x-xc)**2 + radius_major**2*(y-yc)**2 < (radius_major*radius_minor)**2
 * 
 *         self.xc = xc             # <<<<<<<<<<<<<<
 *         self.yc = yc
 *         self.radius_major = radius_major
 */
  __pyx_v_self->xc = __pyx_v_xc;

  /* "pyregion/_region_filter.pyx":507
 * 
 *         self.xc = xc
 *         self.yc = yc             # <<<<<<<<<<<<<<
 *         self.radius_major = radius_major
 *         self.radius_minor = radius_minor
 */
  __pyx_v_self->yc = __pyx_v_yc;

  /* "pyregion/_region_filter.pyx":508
 *         self.xc = xc
 *         self.yc = yc
 *         self.radius_major = radius_major             # <<<<<<<<<<<<<<
 *         self.radius_minor = radius_minor
 *         self.radius_major_2 = radius_major ** 2
 */
  __pyx_v_self->radius_major = __pyx_v_radius_major;

  /* "pyregion/_region_filter.pyx":509
 *         self.yc = yc
 *         self.radius_major = radius_major
 *         self.radius_minor = radius_minor             # <<<<<<<<<<<<<<
 *         self.radius_major_2 = radius_major ** 2
 *         self.radius_minor_2 = radius_minor ** 2
 */
  __pyx_v_self->radius_minor = __pyx_v_radius_minor;

  /* "pyregion/_region_filter.pyx":510
 *         self.radius_major = radius_major
 *         self.radius_minor = radius_minor
 *         self.radius_major_2 = radius_major ** 2             # <<<<<<<<<<<<<<
 *         self.radius_minor_2 = radius_minor ** 2
 *         self.radius_major_2_radius_minor_2 = self.radius_major_2 * self.radius_minor_2
 */
  __pyx_v_self->radius_major_2 = pow(__pyx_v_radius_major, 2.0);

  /* "pyregion/_region_filter.pyx":511
 *         self.radius_minor = radius_minor
 *         self.radius_major_2 = radius_major ** 2
 *         self.radius_minor_2 = radius_minor ** 2             # <<<<<<<<<<<<<<
 *         self.radius_major_2_radius_minor_2 = self.radius_major_2 * self.radius_minor_2
 * 
 */
  __pyx_v_self->radius_minor_2 = pow(__pyx_v_radius_minor, 2.0);

  /* "pyregion/_region_filter.pyx":512
 *         self.radius_major_2 = radius_major ** 2
 *         self.radius_minor_2 = radius_minor ** 2
 *         self.radius_major_2_radius_minor_2 = self.radius_major_2 * self.radius_minor_2             # <<<<<<<<<<<<<<
 * 
 *         self.metric_set_origin(xc, yc, c)
 */
  __pyx_v_self->radius_major_2_radius_minor_2 = (__pyx_v_self->radius_major_2 * __pyx_v_self->radius_minor_2);

  /* "pyregion/_region_filter.pyx":514
 *         self.radius_major_2_radius_minor_2 = self.radius_major_2 * self.radius_minor_2
 * 
 *         self.metric_set_origin(xc, yc, c)             # <<<<<<<<<<<<<<
 * 
 *     cdef npy_bool _inside(self, double x, double y):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_Ellipse *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.metric_set_origin(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self), __pyx_v_xc, __pyx_v_yc, __pyx_v_c); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 514, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":499
 *     cdef double radius_major_2_radius_minor_2
 * 
 *     def __init__(self, double xc, double yc,             # <<<<<<<<<<<<<<
 *                  double radius_major, double radius_minor,
 *                  RegionContext c=None):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Ellipse.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":516
 *         self.metric_set_origin(xc, yc, c)
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef double dist2
 * 
 */

static npy_bool __pyx_f_8pyregion_14_region_filter_7Ellipse__inside(struct __pyx_obj_8pyregion_14_region_filter_Ellipse *__pyx_v_self, double __pyx_v_x, double __pyx_v_y) {
  double __pyx_v_dist2;
  npy_bool __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_inside", 0);

  /* "pyregion/_region_filter.pyx":519
 *         cdef double dist2
 * 
 *         dist2 = self.radius_minor_2 * (x - self.xc) ** 2 + self.radius_major_2 * (y - self.yc) ** 2             # <<<<<<<<<<<<<<
 *         return (dist2 <= self.radius_major_2_radius_minor_2)
 * 
 */
  __pyx_v_dist2 = ((__pyx_v_self->radius_minor_2 * pow((__pyx_v_x - __pyx_v_self->xc), 2.0)) + (__pyx_v_self->radius_major_2 * pow((__pyx_v_y - __pyx_v_self->yc), 2.0)));

  /* "pyregion/_region_filter.pyx":520
 * 
 *         dist2 = self.radius_minor_2 * (x - self.xc) ** 2 + self.radius_major_2 * (y - self.yc) ** 2
 *         return (dist2 <= self.radius_major_2_radius_minor_2)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_r = (__pyx_v_dist2 <= __pyx_v_self->radius_major_2_radius_minor_2);
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":516
 *         self.metric_set_origin(xc, yc, c)
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef double dist2
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":522
 *         return (dist2 <= self.radius_major_2_radius_minor_2)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "Ellipse(%f, %f, %f, %f)" % (self.xc, self.yc, self.radius_major, self.radius_minor)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Ellipse_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Ellipse_3__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_7Ellipse_2__repr__(((struct __pyx_obj_8pyregion_14_region_filter_Ellipse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_7Ellipse_2__repr__(struct __pyx_obj_8pyregion_14_region_filter_Ellipse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "pyregion/_region_filter.pyx":523
 * 
 *     def __repr__(self):
 *         return "Ellipse(%f, %f, %f, %f)" % (self.xc, self.yc, self.radius_major, self.radius_minor)             # <<<<<<<<<<<<<<
 * 
 * cdef class Box(RegionBase):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->xc); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->yc); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->radius_major); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->radius_minor); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Ellipse_f_f_f_f, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":522
 *         return (dist2 <= self.radius_major_2_radius_minor_2)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "Ellipse(%f, %f, %f, %f)" % (self.xc, self.yc, self.radius_major, self.radius_minor)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyregion._region_filter.Ellipse.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Ellipse_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Ellipse_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_7Ellipse_4__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Ellipse *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_7Ellipse_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Ellipse *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__23, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Ellipse.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Ellipse_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Ellipse_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_7Ellipse_6__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Ellipse *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_7Ellipse_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Ellipse *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Ellipse.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":537
 *     cdef double y2
 * 
 *     def __init__(self, double xc, double yc, double width, double height,             # <<<<<<<<<<<<<<
 *                  RegionContext c=None):
 *         cdef double halfwidth
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_3Box_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_3Box_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_xc;
  double __pyx_v_yc;
  double __pyx_v_width;
  double __pyx_v_height;
  struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_xc,&__pyx_n_s_yc,&__pyx_n_s_width,&__pyx_n_s_height,&__pyx_n_s_c,0};
    PyObject* values[5] = {0,0,0,0,0};

    /* "pyregion/_region_filter.pyx":538
 * 
 *     def __init__(self, double xc, double yc, double width, double height,
 *                  RegionContext c=None):             # <<<<<<<<<<<<<<
 *         cdef double halfwidth
 *         cdef double halfheight
 */
    values[4] = (PyObject *)((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_xc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_yc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, 1); __PYX_ERR(1, 537, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, 2); __PYX_ERR(1, 537, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, 3); __PYX_ERR(1, 537, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_c);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 537, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_xc = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_xc == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 537, __pyx_L3_error)
    __pyx_v_yc = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_yc == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 537, __pyx_L3_error)
    __pyx_v_width = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_width == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 537, __pyx_L3_error)
    __pyx_v_height = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_height == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 537, __pyx_L3_error)
    __pyx_v_c = ((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)values[4]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 537, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.Box.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_c), __pyx_ptype_8pyregion_14_region_filter_RegionContext, 1, "c", 0))) __PYX_ERR(1, 538, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_3Box___init__(((struct __pyx_obj_8pyregion_14_region_filter_Box *)__pyx_v_self), __pyx_v_xc, __pyx_v_yc, __pyx_v_width, __pyx_v_height, __pyx_v_c);

  /* "pyregion/_region_filter.pyx":537
 *     cdef double y2
 * 
 *     def __init__(self, double xc, double yc, double width, double height,             # <<<<<<<<<<<<<<
 *                  RegionContext c=None):
 *         cdef double halfwidth
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_3Box___init__(struct __pyx_obj_8pyregion_14_region_filter_Box *__pyx_v_self, double __pyx_v_xc, double __pyx_v_yc, double __pyx_v_width, double __pyx_v_height, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c) {
  double __pyx_v_halfwidth;
  double __pyx_v_halfheight;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyregion/_region_filter.pyx":542
 *         cdef double halfheight
 * 
 *         halfwidth = width * .5             # <<<<<<<<<<<<<<
 *         halfheight = height * .5
 * 
 */
  __pyx_v_halfwidth = (__pyx_v_width * .5);

  /* "pyregion/_region_filter.pyx":543
 * 
 *         halfwidth = width * .5
 *         halfheight = height * .5             # <<<<<<<<<<<<<<
 * 
 *         self.x1 = xc - halfwidth
 */
  __pyx_v_halfheight = (__pyx_v_height * .5);

  /* "pyregion/_region_filter.pyx":545
 *         halfheight = height * .5
 * 
 *         self.x1 = xc - halfwidth             # <<<<<<<<<<<<<<
 *         self.x2 = xc + halfwidth
 *         self.y1 = yc - halfheight
 */
  __pyx_v_self->x1 = (__pyx_v_xc - __pyx_v_halfwidth);

  /* "pyregion/_region_filter.pyx":546
 * 
 *         self.x1 = xc - halfwidth
 *         self.x2 = xc + halfwidth             # <<<<<<<<<<<<<<
 *         self.y1 = yc - halfheight
 *         self.y2 = yc + halfheight
 */
  __pyx_v_self->x2 = (__pyx_v_xc + __pyx_v_halfwidth);

  /* "pyregion/_region_filter.pyx":547
 *         self.x1 = xc - halfwidth
 *         self.x2 = xc + halfwidth
 *         self.y1 = yc - halfheight             # <<<<<<<<<<<<<<
 *         self.y2 = yc + halfheight
 * 
 */
  __pyx_v_self->y1 = (__pyx_v_yc - __pyx_v_halfheight);

  /* "pyregion/_region_filter.pyx":548
 *         self.x2 = xc + halfwidth
 *         self.y1 = yc - halfheight
 *         self.y2 = yc + halfheight             # <<<<<<<<<<<<<<
 * 
 *         self.metric_set_origin(xc, yc, c)
 */
  __pyx_v_self->y2 = (__pyx_v_yc + __pyx_v_halfheight);

  /* "pyregion/_region_filter.pyx":550
 *         self.y2 = yc + halfheight
 * 
 *         self.metric_set_origin(xc, yc, c)             # <<<<<<<<<<<<<<
 * 
 *     cdef npy_bool _inside(self, double x, double y):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_Box *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.metric_set_origin(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self), __pyx_v_xc, __pyx_v_yc, __pyx_v_c); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":537
 *     cdef double y2
 * 
 *     def __init__(self, double xc, double yc, double width, double height,             # <<<<<<<<<<<<<<
 *                  RegionContext c=None):
 *         cdef double halfwidth
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Box.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":552
 *         self.metric_set_origin(xc, yc, c)
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         return (self.x1 <= x) & (x <= self.x2) & (self.y1 <= y) & (y <= self.y2)
 * 
 */

static npy_bool __pyx_f_8pyregion_14_region_filter_3Box__inside(struct __pyx_obj_8pyregion_14_region_filter_Box *__pyx_v_self, double __pyx_v_x, double __pyx_v_y) {
  npy_bool __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_inside", 0);

  /* "pyregion/_region_filter.pyx":553
 * 
 *     cdef npy_bool _inside(self, double x, double y):
 *         return (self.x1 <= x) & (x <= self.x2) & (self.y1 <= y) & (y <= self.y2)             # <<<<<<<<<<<<<<
 * 
 * cdef class Polygon(RegionBase):
 */
  __pyx_r = ((((__pyx_v_self->x1 <= __pyx_v_x) & (__pyx_v_x <= __pyx_v_self->x2)) & (__pyx_v_self->y1 <= __pyx_v_y)) & (__pyx_v_y <= __pyx_v_self->y2));
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":552
 *         self.metric_set_origin(xc, yc, c)
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         return (self.x1 <= x) & (x <= self.x2) & (self.y1 <= y) & (y <= self.y2)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_3Box_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_3Box_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_3Box_2__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Box *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_3Box_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Box *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Box.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_3Box_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_3Box_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_3Box_4__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Box *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_3Box_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Box *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Box.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":571
 *     cdef int n
 * 
 *     def __init__(self, x, y,             # <<<<<<<<<<<<<<
 *                  RegionContext c=None):
 *         self.xa = c_numpy.PyArray_CopyFromObject(x, c_numpy.NPY_DOUBLE, 1, 1)
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_7Polygon_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_7Polygon_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_c,0};
    PyObject* values[3] = {0,0,0};

    /* "pyregion/_region_filter.pyx":572
 * 
 *     def __init__(self, x, y,
 *                  RegionContext c=None):             # <<<<<<<<<<<<<<
 *         self.xa = c_numpy.PyArray_CopyFromObject(x, c_numpy.NPY_DOUBLE, 1, 1)
 *         self.ya = c_numpy.PyArray_CopyFromObject(y, c_numpy.NPY_DOUBLE, 1, 1)
 */
    values[2] = (PyObject *)((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, 1); __PYX_ERR(1, 571, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_c);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 571, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_x = values[0];
    __pyx_v_y = values[1];
    __pyx_v_c = ((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 571, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.Polygon.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_c), __pyx_ptype_8pyregion_14_region_filter_RegionContext, 1, "c", 0))) __PYX_ERR(1, 572, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_7Polygon___init__(((struct __pyx_obj_8pyregion_14_region_filter_Polygon *)__pyx_v_self), __pyx_v_x, __pyx_v_y, __pyx_v_c);

  /* "pyregion/_region_filter.pyx":571
 *     cdef int n
 * 
 *     def __init__(self, x, y,             # <<<<<<<<<<<<<<
 *                  RegionContext c=None):
 *         self.xa = c_numpy.PyArray_CopyFromObject(x, c_numpy.NPY_DOUBLE, 1, 1)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_7Polygon___init__(struct __pyx_obj_8pyregion_14_region_filter_Polygon *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyregion/_region_filter.pyx":573
 *     def __init__(self, x, y,
 *                  RegionContext c=None):
 *         self.xa = c_numpy.PyArray_CopyFromObject(x, c_numpy.NPY_DOUBLE, 1, 1)             # <<<<<<<<<<<<<<
 *         self.ya = c_numpy.PyArray_CopyFromObject(y, c_numpy.NPY_DOUBLE, 1, 1)
 * 
 */
  __pyx_t_1 = PyArray_CopyFromObject(__pyx_v_x, NPY_DOUBLE, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_8pyregion_7c_numpy_ndarray))))) __PYX_ERR(1, 573, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->xa);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->xa));
  __pyx_v_self->xa = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":574
 *                  RegionContext c=None):
 *         self.xa = c_numpy.PyArray_CopyFromObject(x, c_numpy.NPY_DOUBLE, 1, 1)
 *         self.ya = c_numpy.PyArray_CopyFromObject(y, c_numpy.NPY_DOUBLE, 1, 1)             # <<<<<<<<<<<<<<
 * 
 *         self.n = c_numpy.PyArray_SIZE(self.xa)
 */
  __pyx_t_1 = PyArray_CopyFromObject(__pyx_v_y, NPY_DOUBLE, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_8pyregion_7c_numpy_ndarray))))) __PYX_ERR(1, 574, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->ya);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->ya));
  __pyx_v_self->ya = ((PyArrayObject *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":576
 *         self.ya = c_numpy.PyArray_CopyFromObject(y, c_numpy.NPY_DOUBLE, 1, 1)
 * 
 *         self.n = c_numpy.PyArray_SIZE(self.xa)             # <<<<<<<<<<<<<<
 * 
 *         self.x = <double *> c_numpy.PyArray_DATA(self.xa)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->xa);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_self->n = PyArray_SIZE(((PyArrayObject *)__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":578
 *         self.n = c_numpy.PyArray_SIZE(self.xa)
 * 
 *         self.x = <double *> c_numpy.PyArray_DATA(self.xa)             # <<<<<<<<<<<<<<
 *         self.y = <double *> c_numpy.PyArray_DATA(self.ya)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->xa);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_self->x = ((double *)PyArray_DATA(((PyArrayObject *)__pyx_t_1)));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":579
 * 
 *         self.x = <double *> c_numpy.PyArray_DATA(self.xa)
 *         self.y = <double *> c_numpy.PyArray_DATA(self.ya)             # <<<<<<<<<<<<<<
 * 
 *         self.metric_set_origin(self.x[0], self.y[0], c)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_self->ya);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_self->y = ((double *)PyArray_DATA(((PyArrayObject *)__pyx_t_1)));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":581
 *         self.y = <double *> c_numpy.PyArray_DATA(self.ya)
 * 
 *         self.metric_set_origin(self.x[0], self.y[0], c)             # <<<<<<<<<<<<<<
 * 
 *     cdef npy_bool _inside(self, double x, double y):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_Polygon *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.metric_set_origin(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self), (__pyx_v_self->x[0]), (__pyx_v_self->y[0]), __pyx_v_c); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":571
 *     cdef int n
 * 
 *     def __init__(self, x, y,             # <<<<<<<<<<<<<<
 *                  RegionContext c=None):
 *         self.xa = c_numpy.PyArray_CopyFromObject(x, c_numpy.NPY_DOUBLE, 1, 1)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Polygon.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":583
 *         self.metric_set_origin(self.x[0], self.y[0], c)
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef int i, j
 *         cdef npy_bool r
 */

static npy_bool __pyx_f_8pyregion_14_region_filter_7Polygon__inside(struct __pyx_obj_8pyregion_14_region_filter_Polygon *__pyx_v_self, double __pyx_v_x, double __pyx_v_y) {
  int __pyx_v_i;
  int __pyx_v_j;
  npy_bool __pyx_v_r;
  double *__pyx_v_xp;
  double *__pyx_v_yp;
  double __pyx_v__t;
  double __pyx_v_y_yp_i;
  double __pyx_v_y_yp_j;
  npy_bool __pyx_r;
  __Pyx_RefNannyDeclarations
  double *__pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  double __pyx_t_4;
  __Pyx_RefNannySetupContext("_inside", 0);

  /* "pyregion/_region_filter.pyx":591
 *         cdef double y_yp_i, y_yp_j
 * 
 *         j = self.n - 1             # <<<<<<<<<<<<<<
 *         r = 0
 *         xp = self.x
 */
  __pyx_v_j = (__pyx_v_self->n - 1);

  /* "pyregion/_region_filter.pyx":592
 * 
 *         j = self.n - 1
 *         r = 0             # <<<<<<<<<<<<<<
 *         xp = self.x
 *         yp = self.y
 */
  __pyx_v_r = 0;

  /* "pyregion/_region_filter.pyx":593
 *         j = self.n - 1
 *         r = 0
 *         xp = self.x             # <<<<<<<<<<<<<<
 *         yp = self.y
 * 
 */
  __pyx_t_1 = __pyx_v_self->x;
  __pyx_v_xp = __pyx_t_1;

  /* "pyregion/_region_filter.pyx":594
 *         r = 0
 *         xp = self.x
 *         yp = self.y             # <<<<<<<<<<<<<<
 * 
 *         #stable version, but would require more time
 */
  __pyx_t_1 = __pyx_v_self->y;
  __pyx_v_yp = __pyx_t_1;

  /* "pyregion/_region_filter.pyx":597
 * 
 *         #stable version, but would require more time
 *         for i from 0 <= i < self.n:             # <<<<<<<<<<<<<<
 *             y_yp_i = y - yp[i]
 *             y_yp_j = y - yp[j]
 */
  __pyx_t_2 = __pyx_v_self->n;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_2; __pyx_v_i++) {

    /* "pyregion/_region_filter.pyx":598
 *         #stable version, but would require more time
 *         for i from 0 <= i < self.n:
 *             y_yp_i = y - yp[i]             # <<<<<<<<<<<<<<
 *             y_yp_j = y - yp[j]
 * 
 */
    __pyx_v_y_yp_i = (__pyx_v_y - (__pyx_v_yp[__pyx_v_i]));

    /* "pyregion/_region_filter.pyx":599
 *         for i from 0 <= i < self.n:
 *             y_yp_i = y - yp[i]
 *             y_yp_j = y - yp[j]             # <<<<<<<<<<<<<<
 * 
 *             if (y_yp_i == 0.) & (y_yp_j == 0.):  # special case for horizontal line
 */
    __pyx_v_y_yp_j = (__pyx_v_y - (__pyx_v_yp[__pyx_v_j]));

    /* "pyregion/_region_filter.pyx":601
 *             y_yp_j = y - yp[j]
 * 
 *             if (y_yp_i == 0.) & (y_yp_j == 0.):  # special case for horizontal line             # <<<<<<<<<<<<<<
 *                 if (xp[i] - x) * (xp[j] - x) <= 0.:
 *                     return 1
 */
    __pyx_t_3 = (((__pyx_v_y_yp_i == 0.) & (__pyx_v_y_yp_j == 0.)) != 0);
    if (__pyx_t_3) {

      /* "pyregion/_region_filter.pyx":602
 * 
 *             if (y_yp_i == 0.) & (y_yp_j == 0.):  # special case for horizontal line
 *                 if (xp[i] - x) * (xp[j] - x) <= 0.:             # <<<<<<<<<<<<<<
 *                     return 1
 * 
 */
      __pyx_t_3 = (((((__pyx_v_xp[__pyx_v_i]) - __pyx_v_x) * ((__pyx_v_xp[__pyx_v_j]) - __pyx_v_x)) <= 0.) != 0);
      if (__pyx_t_3) {

        /* "pyregion/_region_filter.pyx":603
 *             if (y_yp_i == 0.) & (y_yp_j == 0.):  # special case for horizontal line
 *                 if (xp[i] - x) * (xp[j] - x) <= 0.:
 *                     return 1             # <<<<<<<<<<<<<<
 * 
 *             if ((0 <= y_yp_i) & (0 > y_yp_j) | (0 <= y_yp_j) & (0 > y_yp_i)):
 */
        __pyx_r = 1;
        goto __pyx_L0;

        /* "pyregion/_region_filter.pyx":602
 * 
 *             if (y_yp_i == 0.) & (y_yp_j == 0.):  # special case for horizontal line
 *                 if (xp[i] - x) * (xp[j] - x) <= 0.:             # <<<<<<<<<<<<<<
 *                     return 1
 * 
 */
      }

      /* "pyregion/_region_filter.pyx":601
 *             y_yp_j = y - yp[j]
 * 
 *             if (y_yp_i == 0.) & (y_yp_j == 0.):  # special case for horizontal line             # <<<<<<<<<<<<<<
 *                 if (xp[i] - x) * (xp[j] - x) <= 0.:
 *                     return 1
 */
    }

    /* "pyregion/_region_filter.pyx":605
 *                     return 1
 * 
 *             if ((0 <= y_yp_i) & (0 > y_yp_j) | (0 <= y_yp_j) & (0 > y_yp_i)):             # <<<<<<<<<<<<<<
 *                 _t = xp[i] + y_yp_i / (yp[j] - yp[i]) * (xp[j] - xp[i])
 *                 if _t == x:  # return true immediately if point over the poly-edge
 */
    __pyx_t_3 = ((((0.0 <= __pyx_v_y_yp_i) & (0.0 > __pyx_v_y_yp_j)) | ((0.0 <= __pyx_v_y_yp_j) & (0.0 > __pyx_v_y_yp_i))) != 0);
    if (__pyx_t_3) {

      /* "pyregion/_region_filter.pyx":606
 * 
 *             if ((0 <= y_yp_i) & (0 > y_yp_j) | (0 <= y_yp_j) & (0 > y_yp_i)):
 *                 _t = xp[i] + y_yp_i / (yp[j] - yp[i]) * (xp[j] - xp[i])             # <<<<<<<<<<<<<<
 *                 if _t == x:  # return true immediately if point over the poly-edge
 *                     return 1
 */
      __pyx_t_4 = ((__pyx_v_yp[__pyx_v_j]) - (__pyx_v_yp[__pyx_v_i]));
      if (unlikely(__pyx_t_4 == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "float division");
        __PYX_ERR(1, 606, __pyx_L1_error)
      }
      __pyx_v__t = ((__pyx_v_xp[__pyx_v_i]) + ((__pyx_v_y_yp_i / __pyx_t_4) * ((__pyx_v_xp[__pyx_v_j]) - (__pyx_v_xp[__pyx_v_i]))));

      /* "pyregion/_region_filter.pyx":607
 *             if ((0 <= y_yp_i) & (0 > y_yp_j) | (0 <= y_yp_j) & (0 > y_yp_i)):
 *                 _t = xp[i] + y_yp_i / (yp[j] - yp[i]) * (xp[j] - xp[i])
 *                 if _t == x:  # return true immediately if point over the poly-edge             # <<<<<<<<<<<<<<
 *                     return 1
 *                 # but above does not catch horizontal line
 */
      __pyx_t_3 = ((__pyx_v__t == __pyx_v_x) != 0);
      if (__pyx_t_3) {

        /* "pyregion/_region_filter.pyx":608
 *                 _t = xp[i] + y_yp_i / (yp[j] - yp[i]) * (xp[j] - xp[i])
 *                 if _t == x:  # return true immediately if point over the poly-edge
 *                     return 1             # <<<<<<<<<<<<<<
 *                 # but above does not catch horizontal line
 *                 if (_t < x):
 */
        __pyx_r = 1;
        goto __pyx_L0;

        /* "pyregion/_region_filter.pyx":607
 *             if ((0 <= y_yp_i) & (0 > y_yp_j) | (0 <= y_yp_j) & (0 > y_yp_i)):
 *                 _t = xp[i] + y_yp_i / (yp[j] - yp[i]) * (xp[j] - xp[i])
 *                 if _t == x:  # return true immediately if point over the poly-edge             # <<<<<<<<<<<<<<
 *                     return 1
 *                 # but above does not catch horizontal line
 */
      }

      /* "pyregion/_region_filter.pyx":610
 *                     return 1
 *                 # but above does not catch horizontal line
 *                 if (_t < x):             # <<<<<<<<<<<<<<
 *                     r = not r
 *             j = i
 */
      __pyx_t_3 = ((__pyx_v__t < __pyx_v_x) != 0);
      if (__pyx_t_3) {

        /* "pyregion/_region_filter.pyx":611
 *                 # but above does not catch horizontal line
 *                 if (_t < x):
 *                     r = not r             # <<<<<<<<<<<<<<
 *             j = i
 * 
 */
        __pyx_v_r = (!(__pyx_v_r != 0));

        /* "pyregion/_region_filter.pyx":610
 *                     return 1
 *                 # but above does not catch horizontal line
 *                 if (_t < x):             # <<<<<<<<<<<<<<
 *                     r = not r
 *             j = i
 */
      }

      /* "pyregion/_region_filter.pyx":605
 *                     return 1
 * 
 *             if ((0 <= y_yp_i) & (0 > y_yp_j) | (0 <= y_yp_j) & (0 > y_yp_i)):             # <<<<<<<<<<<<<<
 *                 _t = xp[i] + y_yp_i / (yp[j] - yp[i]) * (xp[j] - xp[i])
 *                 if _t == x:  # return true immediately if point over the poly-edge
 */
    }

    /* "pyregion/_region_filter.pyx":612
 *                 if (_t < x):
 *                     r = not r
 *             j = i             # <<<<<<<<<<<<<<
 * 
 *         return r
 */
    __pyx_v_j = __pyx_v_i;
  }

  /* "pyregion/_region_filter.pyx":614
 *             j = i
 * 
 *         return r             # <<<<<<<<<<<<<<
 * 
 * cdef class AngleRange(RegionBase):
 */
  __pyx_r = __pyx_v_r;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":583
 *         self.metric_set_origin(self.x[0], self.y[0], c)
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef int i, j
 *         cdef npy_bool r
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("pyregion._region_filter.Polygon._inside", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Polygon_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Polygon_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_7Polygon_2__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Polygon *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_7Polygon_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Polygon *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Polygon.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Polygon_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_7Polygon_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_7Polygon_4__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_Polygon *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_7Polygon_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_Polygon *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__28, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.Polygon.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":630
 *     cdef double radian2
 * 
 *     def __init__(self, double xc, double yc,             # <<<<<<<<<<<<<<
 *                  double degree1, double degree2,
 *                  RegionContext c=None):
 */

/* Python wrapper */
static int __pyx_pw_8pyregion_14_region_filter_10AngleRange_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_8pyregion_14_region_filter_10AngleRange_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_xc;
  double __pyx_v_yc;
  double __pyx_v_degree1;
  double __pyx_v_degree2;
  struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_xc,&__pyx_n_s_yc,&__pyx_n_s_degree1,&__pyx_n_s_degree2,&__pyx_n_s_c,0};
    PyObject* values[5] = {0,0,0,0,0};

    /* "pyregion/_region_filter.pyx":632
 *     def __init__(self, double xc, double yc,
 *                  double degree1, double degree2,
 *                  RegionContext c=None):             # <<<<<<<<<<<<<<
 * 
 *         self.xc = xc
 */
    values[4] = (PyObject *)((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_xc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_yc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, 1); __PYX_ERR(1, 630, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_degree1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, 2); __PYX_ERR(1, 630, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_degree2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, 3); __PYX_ERR(1, 630, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_c);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 630, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_xc = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_xc == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 630, __pyx_L3_error)
    __pyx_v_yc = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_yc == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 630, __pyx_L3_error)
    __pyx_v_degree1 = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_degree1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 631, __pyx_L3_error)
    __pyx_v_degree2 = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_degree2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 631, __pyx_L3_error)
    __pyx_v_c = ((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)values[4]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 630, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pyregion._region_filter.AngleRange.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_c), __pyx_ptype_8pyregion_14_region_filter_RegionContext, 1, "c", 0))) __PYX_ERR(1, 632, __pyx_L1_error)
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10AngleRange___init__(((struct __pyx_obj_8pyregion_14_region_filter_AngleRange *)__pyx_v_self), __pyx_v_xc, __pyx_v_yc, __pyx_v_degree1, __pyx_v_degree2, __pyx_v_c);

  /* "pyregion/_region_filter.pyx":630
 *     cdef double radian2
 * 
 *     def __init__(self, double xc, double yc,             # <<<<<<<<<<<<<<
 *                  double degree1, double degree2,
 *                  RegionContext c=None):
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_8pyregion_14_region_filter_10AngleRange___init__(struct __pyx_obj_8pyregion_14_region_filter_AngleRange *__pyx_v_self, double __pyx_v_xc, double __pyx_v_yc, double __pyx_v_degree1, double __pyx_v_degree2, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *__pyx_v_c) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pyregion/_region_filter.pyx":634
 *                  RegionContext c=None):
 * 
 *         self.xc = xc             # <<<<<<<<<<<<<<
 *         self.yc = yc
 *         self.degree1 = degree1
 */
  __pyx_v_self->xc = __pyx_v_xc;

  /* "pyregion/_region_filter.pyx":635
 * 
 *         self.xc = xc
 *         self.yc = yc             # <<<<<<<<<<<<<<
 *         self.degree1 = degree1
 *         self.degree2 = degree2
 */
  __pyx_v_self->yc = __pyx_v_yc;

  /* "pyregion/_region_filter.pyx":636
 *         self.xc = xc
 *         self.yc = yc
 *         self.degree1 = degree1             # <<<<<<<<<<<<<<
 *         self.degree2 = degree2
 * 
 */
  __pyx_v_self->degree1 = __pyx_v_degree1;

  /* "pyregion/_region_filter.pyx":637
 *         self.yc = yc
 *         self.degree1 = degree1
 *         self.degree2 = degree2             # <<<<<<<<<<<<<<
 * 
 *         # theta in radian
 */
  __pyx_v_self->degree2 = __pyx_v_degree2;

  /* "pyregion/_region_filter.pyx":640
 * 
 *         # theta in radian
 *         self.radian1 = degree1 / 180. * M_PI  #3.1415926             # <<<<<<<<<<<<<<
 *         self.radian2 = self._fix_angle(degree2 / 180. * M_PI)
 * 
 */
  __pyx_v_self->radian1 = ((__pyx_v_degree1 / 180.) * M_PI);

  /* "pyregion/_region_filter.pyx":641
 *         # theta in radian
 *         self.radian1 = degree1 / 180. * M_PI  #3.1415926
 *         self.radian2 = self._fix_angle(degree2 / 180. * M_PI)             # <<<<<<<<<<<<<<
 * 
 *         self.metric_set_origin(xc, yc, c)
 */
  __pyx_v_self->radian2 = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_AngleRange *)__pyx_v_self->__pyx_base.__pyx_vtab)->_fix_angle(__pyx_v_self, ((__pyx_v_degree2 / 180.) * M_PI));

  /* "pyregion/_region_filter.pyx":643
 *         self.radian2 = self._fix_angle(degree2 / 180. * M_PI)
 * 
 *         self.metric_set_origin(xc, yc, c)             # <<<<<<<<<<<<<<
 * 
 *     cdef double _fix_angle(self, double a):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_AngleRange *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.metric_set_origin(((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)__pyx_v_self), __pyx_v_xc, __pyx_v_yc, __pyx_v_c); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 643, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":630
 *     cdef double radian2
 * 
 *     def __init__(self, double xc, double yc,             # <<<<<<<<<<<<<<
 *                  double degree1, double degree2,
 *                  RegionContext c=None):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.AngleRange.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":645
 *         self.metric_set_origin(xc, yc, c)
 * 
 *     cdef double _fix_angle(self, double a):             # <<<<<<<<<<<<<<
 *         if a > self.radian1:
 *             return self.radian1 + fmod((a - self.radian1), 2 * M_PI)
 */

static double __pyx_f_8pyregion_14_region_filter_10AngleRange__fix_angle(struct __pyx_obj_8pyregion_14_region_filter_AngleRange *__pyx_v_self, double __pyx_v_a) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_fix_angle", 0);

  /* "pyregion/_region_filter.pyx":646
 * 
 *     cdef double _fix_angle(self, double a):
 *         if a > self.radian1:             # <<<<<<<<<<<<<<
 *             return self.radian1 + fmod((a - self.radian1), 2 * M_PI)
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_a > __pyx_v_self->radian1) != 0);
  if (__pyx_t_1) {

    /* "pyregion/_region_filter.pyx":647
 *     cdef double _fix_angle(self, double a):
 *         if a > self.radian1:
 *             return self.radian1 + fmod((a - self.radian1), 2 * M_PI)             # <<<<<<<<<<<<<<
 *         else:
 *             return self.radian1 + 2. * M_PI - fmod((self.radian1 - a), 2 * M_PI)
 */
    __pyx_r = (__pyx_v_self->radian1 + fmod((__pyx_v_a - __pyx_v_self->radian1), (2.0 * M_PI)));
    goto __pyx_L0;

    /* "pyregion/_region_filter.pyx":646
 * 
 *     cdef double _fix_angle(self, double a):
 *         if a > self.radian1:             # <<<<<<<<<<<<<<
 *             return self.radian1 + fmod((a - self.radian1), 2 * M_PI)
 *         else:
 */
  }

  /* "pyregion/_region_filter.pyx":649
 *             return self.radian1 + fmod((a - self.radian1), 2 * M_PI)
 *         else:
 *             return self.radian1 + 2. * M_PI - fmod((self.radian1 - a), 2 * M_PI)             # <<<<<<<<<<<<<<
 * 
 *     cdef npy_bool _inside(self, double x, double y):
 */
  /*else*/ {
    __pyx_r = ((__pyx_v_self->radian1 + (2. * M_PI)) - fmod((__pyx_v_self->radian1 - __pyx_v_a), (2.0 * M_PI)));
    goto __pyx_L0;
  }

  /* "pyregion/_region_filter.pyx":645
 *         self.metric_set_origin(xc, yc, c)
 * 
 *     cdef double _fix_angle(self, double a):             # <<<<<<<<<<<<<<
 *         if a > self.radian1:
 *             return self.radian1 + fmod((a - self.radian1), 2 * M_PI)
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":651
 *             return self.radian1 + 2. * M_PI - fmod((self.radian1 - a), 2 * M_PI)
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef double dx, dy, theta
 * 
 */

static npy_bool __pyx_f_8pyregion_14_region_filter_10AngleRange__inside(struct __pyx_obj_8pyregion_14_region_filter_AngleRange *__pyx_v_self, double __pyx_v_x, double __pyx_v_y) {
  double __pyx_v_dx;
  double __pyx_v_dy;
  double __pyx_v_theta;
  npy_bool __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_inside", 0);

  /* "pyregion/_region_filter.pyx":654
 *         cdef double dx, dy, theta
 * 
 *         dx = x - self.xc             # <<<<<<<<<<<<<<
 *         dy = y - self.yc
 * 
 */
  __pyx_v_dx = (__pyx_v_x - __pyx_v_self->xc);

  /* "pyregion/_region_filter.pyx":655
 * 
 *         dx = x - self.xc
 *         dy = y - self.yc             # <<<<<<<<<<<<<<
 * 
 *         theta = atan2(dy, dx)
 */
  __pyx_v_dy = (__pyx_v_y - __pyx_v_self->yc);

  /* "pyregion/_region_filter.pyx":657
 *         dy = y - self.yc
 * 
 *         theta = atan2(dy, dx)             # <<<<<<<<<<<<<<
 * 
 *         theta = self._fix_angle(theta)
 */
  __pyx_v_theta = atan2(__pyx_v_dy, __pyx_v_dx);

  /* "pyregion/_region_filter.pyx":659
 *         theta = atan2(dy, dx)
 * 
 *         theta = self._fix_angle(theta)             # <<<<<<<<<<<<<<
 *         return (theta < self.radian2)
 * 
 */
  __pyx_v_theta = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_AngleRange *)__pyx_v_self->__pyx_base.__pyx_vtab)->_fix_angle(__pyx_v_self, __pyx_v_theta);

  /* "pyregion/_region_filter.pyx":660
 * 
 *         theta = self._fix_angle(theta)
 *         return (theta < self.radian2)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_r = (__pyx_v_theta < __pyx_v_self->radian2);
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":651
 *             return self.radian1 + 2. * M_PI - fmod((self.radian1 - a), 2 * M_PI)
 * 
 *     cdef npy_bool _inside(self, double x, double y):             # <<<<<<<<<<<<<<
 *         cdef double dx, dy, theta
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pyregion/_region_filter.pyx":662
 *         return (theta < self.radian2)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "AngleRange(%f, %f, %f, %f)" % (self.xc, self.yc, self.degree1, self.degree2)
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10AngleRange_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10AngleRange_3__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10AngleRange_2__repr__(((struct __pyx_obj_8pyregion_14_region_filter_AngleRange *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10AngleRange_2__repr__(struct __pyx_obj_8pyregion_14_region_filter_AngleRange *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "pyregion/_region_filter.pyx":663
 * 
 *     def __repr__(self):
 *         return "AngleRange(%f, %f, %f, %f)" % (self.xc, self.yc, self.degree1, self.degree2)             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->xc); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->yc); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->degree1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->degree2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_AngleRange_f_f_f_f, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "pyregion/_region_filter.pyx":662
 *         return (theta < self.radian2)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "AngleRange(%f, %f, %f, %f)" % (self.xc, self.yc, self.degree1, self.degree2)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pyregion._region_filter.AngleRange.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10AngleRange_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10AngleRange_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10AngleRange_4__reduce_cython__(((struct __pyx_obj_8pyregion_14_region_filter_AngleRange *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10AngleRange_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_AngleRange *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.AngleRange.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8pyregion_14_region_filter_10AngleRange_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_8pyregion_14_region_filter_10AngleRange_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_8pyregion_14_region_filter_10AngleRange_6__setstate_cython__(((struct __pyx_obj_8pyregion_14_region_filter_AngleRange *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8pyregion_14_region_filter_10AngleRange_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_8pyregion_14_region_filter_AngleRange *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__30, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pyregion._region_filter.AngleRange.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionContext __pyx_vtable_8pyregion_14_region_filter_RegionContext;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_RegionContext(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionContext *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)o);
  p->__pyx_vtab = __pyx_vtabptr_8pyregion_14_region_filter_RegionContext;
  return o;
}

static void __pyx_tp_dealloc_8pyregion_14_region_filter_RegionContext(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_RegionContext[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_13RegionContext_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_13RegionContext_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_RegionContext = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.RegionContext", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_RegionContext), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionContext, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_RegionContext, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8pyregion_14_region_filter_13RegionContext_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_RegionContext, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase __pyx_vtable_8pyregion_14_region_filter_RegionBase;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_RegionBase(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)o);
  p->__pyx_vtab = __pyx_vtabptr_8pyregion_14_region_filter_RegionBase;
  p->c = ((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_8pyregion_14_region_filter_RegionBase(PyObject *o) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *p = (struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->c);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_8pyregion_14_region_filter_RegionBase(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *p = (struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)o;
  if (p->c) {
    e = (*v)(((PyObject *)p->c), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_8pyregion_14_region_filter_RegionBase(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_8pyregion_14_region_filter_RegionBase *p = (struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)o;
  tmp = ((PyObject*)p->c);
  p->c = ((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_RegionBase[] = {
  {"set_context", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10RegionBase_3set_context, METH_O, 0},
  {"mask", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10RegionBase_11mask, METH_O, __pyx_doc_8pyregion_14_region_filter_10RegionBase_10mask},
  {"inside1", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10RegionBase_13inside1, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8pyregion_14_region_filter_10RegionBase_12inside1},
  {"inside", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10RegionBase_15inside, METH_VARARGS|METH_KEYWORDS, 0},
  {"inside_xy", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10RegionBase_17inside_xy, METH_O, __pyx_doc_8pyregion_14_region_filter_10RegionBase_16inside_xy},
  {"inside_x_y", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10RegionBase_19inside_x_y, METH_VARARGS|METH_KEYWORDS, __pyx_doc_8pyregion_14_region_filter_10RegionBase_18inside_x_y},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10RegionBase_21__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10RegionBase_23__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_RegionBase = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  0, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  __pyx_pw_8pyregion_14_region_filter_10RegionBase_5__invert__, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  __pyx_pw_8pyregion_14_region_filter_10RegionBase_7__and__, /*nb_and*/
  0, /*nb_xor*/
  __pyx_pw_8pyregion_14_region_filter_10RegionBase_9__or__, /*nb_or*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_matrix_multiply*/
  #endif
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_inplace_matrix_multiply*/
  #endif
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_RegionBase = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.RegionBase", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_RegionBase), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionBase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  &__pyx_tp_as_number_RegionBase, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_RegionBase, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_RegionBase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_RegionBase, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8pyregion_14_region_filter_10RegionBase_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_RegionBase, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionNot __pyx_vtable_8pyregion_14_region_filter_RegionNot;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_RegionNot(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionNot *p;
  PyObject *o = __pyx_tp_new_8pyregion_14_region_filter_RegionBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_RegionNot *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase*)__pyx_vtabptr_8pyregion_14_region_filter_RegionNot;
  p->child_region = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_8pyregion_14_region_filter_RegionNot(PyObject *o) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionNot *p = (struct __pyx_obj_8pyregion_14_region_filter_RegionNot *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->child_region);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionBase(o);
}

static int __pyx_tp_traverse_8pyregion_14_region_filter_RegionNot(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8pyregion_14_region_filter_RegionNot *p = (struct __pyx_obj_8pyregion_14_region_filter_RegionNot *)o;
  e = __pyx_tp_traverse_8pyregion_14_region_filter_RegionBase(o, v, a); if (e) return e;
  if (p->child_region) {
    e = (*v)(((PyObject *)p->child_region), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_8pyregion_14_region_filter_RegionNot(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_8pyregion_14_region_filter_RegionNot *p = (struct __pyx_obj_8pyregion_14_region_filter_RegionNot *)o;
  __pyx_tp_clear_8pyregion_14_region_filter_RegionBase(o);
  tmp = ((PyObject*)p->child_region);
  p->child_region = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_RegionNot[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_9RegionNot_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_9RegionNot_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_RegionNot = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.RegionNot", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_RegionNot), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionNot, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    >>> r = RegionNot(r2)\n    ", /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_RegionNot, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_RegionNot, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_RegionNot, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8pyregion_14_region_filter_9RegionNot_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_RegionNot, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionList __pyx_vtable_8pyregion_14_region_filter_RegionList;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_RegionList(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionList *p;
  PyObject *o = __pyx_tp_new_8pyregion_14_region_filter_RegionBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_RegionList *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase*)__pyx_vtabptr_8pyregion_14_region_filter_RegionList;
  p->child_regions = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_8pyregion_14_region_filter_RegionList(PyObject *o) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionList *p = (struct __pyx_obj_8pyregion_14_region_filter_RegionList *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->child_regions);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionBase(o);
}

static int __pyx_tp_traverse_8pyregion_14_region_filter_RegionList(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8pyregion_14_region_filter_RegionList *p = (struct __pyx_obj_8pyregion_14_region_filter_RegionList *)o;
  e = __pyx_tp_traverse_8pyregion_14_region_filter_RegionBase(o, v, a); if (e) return e;
  if (p->child_regions) {
    e = (*v)(p->child_regions, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_8pyregion_14_region_filter_RegionList(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_8pyregion_14_region_filter_RegionList *p = (struct __pyx_obj_8pyregion_14_region_filter_RegionList *)o;
  __pyx_tp_clear_8pyregion_14_region_filter_RegionBase(o);
  tmp = ((PyObject*)p->child_regions);
  p->child_regions = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_8pyregion_14_region_filter_RegionList(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_8pyregion_14_region_filter_RegionList(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_8pyregion_14_region_filter_10RegionList_9__setitem__(o, i, v);
  }
  else {
    return __pyx_pw_8pyregion_14_region_filter_10RegionList_11__delitem__(o, i);
  }
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_RegionList[] = {
  {"_check_type_of_list", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10RegionList_1_check_type_of_list, METH_O, 0},
  {"asList", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10RegionList_17asList, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10RegionList_19__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10RegionList_21__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_RegionList = {
  __pyx_pw_8pyregion_14_region_filter_10RegionList_5__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_8pyregion_14_region_filter_RegionList, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  __pyx_pw_8pyregion_14_region_filter_10RegionList_13__contains__, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_RegionList = {
  __pyx_pw_8pyregion_14_region_filter_10RegionList_5__len__, /*mp_length*/
  __pyx_pw_8pyregion_14_region_filter_10RegionList_7__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_8pyregion_14_region_filter_RegionList, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_RegionList = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.RegionList", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_RegionList), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionList, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_8pyregion_14_region_filter_10RegionList_15__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_RegionList, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_RegionList, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_RegionList, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_RegionList, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_RegionList, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8pyregion_14_region_filter_10RegionList_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_RegionList, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionOrList __pyx_vtable_8pyregion_14_region_filter_RegionOrList;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_RegionOrList(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *p;
  PyObject *o = __pyx_tp_new_8pyregion_14_region_filter_RegionList(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_RegionOrList *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase*)__pyx_vtabptr_8pyregion_14_region_filter_RegionOrList;
  return o;
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_RegionOrList[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_12RegionOrList_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_12RegionOrList_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_RegionOrList = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.RegionOrList", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_RegionOrList), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionList, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_8pyregion_14_region_filter_12RegionOrList_1__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    >>> r = RegionOrList(r1, r2, r3, r4, ...)\n    ", /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_RegionList, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_RegionList, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_RegionOrList, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_8pyregion_14_region_filter_10RegionList_3__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_RegionOrList, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionAndList __pyx_vtable_8pyregion_14_region_filter_RegionAndList;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_RegionAndList(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *p;
  PyObject *o = __pyx_tp_new_8pyregion_14_region_filter_RegionList(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_RegionAndList *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase*)__pyx_vtabptr_8pyregion_14_region_filter_RegionAndList;
  return o;
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_RegionAndList[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_13RegionAndList_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_13RegionAndList_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_RegionAndList = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.RegionAndList", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_RegionAndList), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionList, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_8pyregion_14_region_filter_13RegionAndList_1__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    >>> r = RegionAndList(r1, r2, r3, r4, ...)\n    ", /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_RegionList, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_RegionList, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_RegionAndList, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_pw_8pyregion_14_region_filter_10RegionList_3__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_RegionAndList, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Transform __pyx_vtable_8pyregion_14_region_filter_Transform;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Transform(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_Transform *p;
  PyObject *o = __pyx_tp_new_8pyregion_14_region_filter_RegionBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_Transform *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase*)__pyx_vtabptr_8pyregion_14_region_filter_Transform;
  p->child_region = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_8pyregion_14_region_filter_Transform(PyObject *o) {
  struct __pyx_obj_8pyregion_14_region_filter_Transform *p = (struct __pyx_obj_8pyregion_14_region_filter_Transform *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->child_region);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionBase(o);
}

static int __pyx_tp_traverse_8pyregion_14_region_filter_Transform(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8pyregion_14_region_filter_Transform *p = (struct __pyx_obj_8pyregion_14_region_filter_Transform *)o;
  e = __pyx_tp_traverse_8pyregion_14_region_filter_RegionBase(o, v, a); if (e) return e;
  if (p->child_region) {
    e = (*v)(((PyObject *)p->child_region), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_8pyregion_14_region_filter_Transform(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_8pyregion_14_region_filter_Transform *p = (struct __pyx_obj_8pyregion_14_region_filter_Transform *)o;
  __pyx_tp_clear_8pyregion_14_region_filter_RegionBase(o);
  tmp = ((PyObject*)p->child_region);
  p->child_region = ((struct __pyx_obj_8pyregion_14_region_filter_RegionBase *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_8pyregion_14_region_filter_9Transform_child(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_8pyregion_14_region_filter_9Transform_5child_1__get__(o);
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_Transform[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_9Transform_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_9Transform_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_8pyregion_14_region_filter_Transform[] = {
  {(char *)"child", __pyx_getprop_8pyregion_14_region_filter_9Transform_child, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_Transform = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.Transform", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_Transform), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_Transform, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_Transform, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_Transform, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_Transform, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_8pyregion_14_region_filter_Transform, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8pyregion_14_region_filter_9Transform_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_Transform, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Rotated __pyx_vtable_8pyregion_14_region_filter_Rotated;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Rotated(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_Rotated *p;
  PyObject *o = __pyx_tp_new_8pyregion_14_region_filter_Transform(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_Rotated *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase*)__pyx_vtabptr_8pyregion_14_region_filter_Rotated;
  return o;
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_Rotated[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_7Rotated_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_7Rotated_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_Rotated = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.Rotated", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_Rotated), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_Transform, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Rotate the region by degree in anti-colockwise direction.\n\n     >>> reg = Rotated(child_region, degree, origin_x, origin_y)\n\n    ", /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_Transform, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_Transform, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_Rotated, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8pyregion_14_region_filter_7Rotated_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_Rotated, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Translated __pyx_vtable_8pyregion_14_region_filter_Translated;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Translated(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_Translated *p;
  PyObject *o = __pyx_tp_new_8pyregion_14_region_filter_Transform(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_Translated *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase*)__pyx_vtabptr_8pyregion_14_region_filter_Translated;
  return o;
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_Translated[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10Translated_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10Translated_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_Translated = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.Translated", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_Translated), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_Transform, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Translated region.\n\n     >>> Translate(child_region, dx, dy)\n    ", /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_Transform, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_Transform, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_Translated, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8pyregion_14_region_filter_10Translated_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_Translated, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Circle __pyx_vtable_8pyregion_14_region_filter_Circle;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Circle(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_Circle *p;
  PyObject *o = __pyx_tp_new_8pyregion_14_region_filter_RegionBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_Circle *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase*)__pyx_vtabptr_8pyregion_14_region_filter_Circle;
  return o;
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_Circle[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_6Circle_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_6Circle_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_Circle = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.Circle", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_Circle), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionBase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_8pyregion_14_region_filter_6Circle_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Circle.\n\n    >>> cir = Circle(xc, yc, radius, RegionContext c=None)\n\n    ", /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_RegionBase, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_RegionBase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_Circle, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8pyregion_14_region_filter_6Circle_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_Circle, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Ellipse __pyx_vtable_8pyregion_14_region_filter_Ellipse;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Ellipse(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_Ellipse *p;
  PyObject *o = __pyx_tp_new_8pyregion_14_region_filter_RegionBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_Ellipse *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase*)__pyx_vtabptr_8pyregion_14_region_filter_Ellipse;
  return o;
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_Ellipse[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_7Ellipse_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_7Ellipse_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_Ellipse = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.Ellipse", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_Ellipse), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionBase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_8pyregion_14_region_filter_7Ellipse_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Ellipse.\n\n    >>> shape = Ellipse(xc, yc, radius_major, radius_minor)\n    ", /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_RegionBase, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_RegionBase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_Ellipse, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8pyregion_14_region_filter_7Ellipse_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_Ellipse, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Box __pyx_vtable_8pyregion_14_region_filter_Box;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Box(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_Box *p;
  PyObject *o = __pyx_tp_new_8pyregion_14_region_filter_RegionBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_Box *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase*)__pyx_vtabptr_8pyregion_14_region_filter_Box;
  return o;
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_Box[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_3Box_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_3Box_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_Box = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.Box", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_Box), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionBase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Box.\n\n    >>> shape = Box(xc, yc, width, height)\n    ", /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_RegionBase, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_RegionBase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_Box, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8pyregion_14_region_filter_3Box_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_Box, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_Polygon __pyx_vtable_8pyregion_14_region_filter_Polygon;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_Polygon(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_Polygon *p;
  PyObject *o = __pyx_tp_new_8pyregion_14_region_filter_RegionBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_Polygon *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase*)__pyx_vtabptr_8pyregion_14_region_filter_Polygon;
  p->xa = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  p->ya = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_8pyregion_14_region_filter_Polygon(PyObject *o) {
  struct __pyx_obj_8pyregion_14_region_filter_Polygon *p = (struct __pyx_obj_8pyregion_14_region_filter_Polygon *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->xa);
  Py_CLEAR(p->ya);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionBase(o);
}

static int __pyx_tp_traverse_8pyregion_14_region_filter_Polygon(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_8pyregion_14_region_filter_Polygon *p = (struct __pyx_obj_8pyregion_14_region_filter_Polygon *)o;
  e = __pyx_tp_traverse_8pyregion_14_region_filter_RegionBase(o, v, a); if (e) return e;
  if (p->xa) {
    e = (*v)(((PyObject *)p->xa), a); if (e) return e;
  }
  if (p->ya) {
    e = (*v)(((PyObject *)p->ya), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_8pyregion_14_region_filter_Polygon(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_8pyregion_14_region_filter_Polygon *p = (struct __pyx_obj_8pyregion_14_region_filter_Polygon *)o;
  __pyx_tp_clear_8pyregion_14_region_filter_RegionBase(o);
  tmp = ((PyObject*)p->xa);
  p->xa = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->ya);
  p->ya = ((PyArrayObject *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_Polygon[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_7Polygon_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_7Polygon_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_Polygon = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.Polygon", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_Polygon), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_Polygon, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Polygon.\n\n     >>> shape = Polygon(x, y)\n\n     Parameters:\n     x, y : list of floats\n    ", /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_Polygon, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_Polygon, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_Polygon, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8pyregion_14_region_filter_7Polygon_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_Polygon, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_8pyregion_14_region_filter_AngleRange __pyx_vtable_8pyregion_14_region_filter_AngleRange;

static PyObject *__pyx_tp_new_8pyregion_14_region_filter_AngleRange(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_8pyregion_14_region_filter_AngleRange *p;
  PyObject *o = __pyx_tp_new_8pyregion_14_region_filter_RegionBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_8pyregion_14_region_filter_AngleRange *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionBase*)__pyx_vtabptr_8pyregion_14_region_filter_AngleRange;
  return o;
}

static PyMethodDef __pyx_methods_8pyregion_14_region_filter_AngleRange[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10AngleRange_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_8pyregion_14_region_filter_10AngleRange_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_8pyregion_14_region_filter_AngleRange = {
  PyVarObject_HEAD_INIT(0, 0)
  "pyregion._region_filter.AngleRange", /*tp_name*/
  sizeof(struct __pyx_obj_8pyregion_14_region_filter_AngleRange), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_8pyregion_14_region_filter_RegionBase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_8pyregion_14_region_filter_10AngleRange_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    AngleRange.\n\n    >>> shape = Ellipse(xc, yc, degree1, degree2)\n    ", /*tp_doc*/
  __pyx_tp_traverse_8pyregion_14_region_filter_RegionBase, /*tp_traverse*/
  __pyx_tp_clear_8pyregion_14_region_filter_RegionBase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_8pyregion_14_region_filter_AngleRange, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_8pyregion_14_region_filter_10AngleRange_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_8pyregion_14_region_filter_AngleRange, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec__region_filter(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec__region_filter},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "_region_filter",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_All_elements_should_be_subclass, __pyx_k_All_elements_should_be_subclass, sizeof(__pyx_k_All_elements_should_be_subclass), 0, 0, 1, 0},
  {&__pyx_n_s_And, __pyx_k_And, sizeof(__pyx_k_And), 0, 0, 1, 1},
  {&__pyx_kp_s_AngleRange_f_f_f_f, __pyx_k_AngleRange_f_f_f_f, sizeof(__pyx_k_AngleRange_f_f_f_f), 0, 0, 1, 0},
  {&__pyx_n_s_BaseClassInitException, __pyx_k_BaseClassInitException, sizeof(__pyx_k_BaseClassInitException), 0, 0, 1, 1},
  {&__pyx_kp_s_Circle_f_f_f, __pyx_k_Circle_f_f_f, sizeof(__pyx_k_Circle_f_f_f), 0, 0, 1, 0},
  {&__pyx_kp_s_Ellipse_f_f_f_f, __pyx_k_Ellipse_f_f_f_f, sizeof(__pyx_k_Ellipse_f_f_f_f), 0, 0, 1, 0},
  {&__pyx_n_s_NotYetImplemented, __pyx_k_NotYetImplemented, sizeof(__pyx_k_NotYetImplemented), 0, 0, 1, 1},
  {&__pyx_n_s_Or, __pyx_k_Or, sizeof(__pyx_k_Or), 0, 0, 1, 1},
  {&__pyx_kp_s_Pickling_of_struct_members_such, __pyx_k_Pickling_of_struct_members_such, sizeof(__pyx_k_Pickling_of_struct_members_such), 0, 0, 1, 0},
  {&__pyx_n_s_RegionAnd, __pyx_k_RegionAnd, sizeof(__pyx_k_RegionAnd), 0, 0, 1, 1},
  {&__pyx_kp_u_RegionAnd_line_319, __pyx_k_RegionAnd_line_319, sizeof(__pyx_k_RegionAnd_line_319), 0, 1, 0, 0},
  {&__pyx_n_s_RegionFilterException, __pyx_k_RegionFilterException, sizeof(__pyx_k_RegionFilterException), 0, 0, 1, 1},
  {&__pyx_n_s_RegionOr, __pyx_k_RegionOr, sizeof(__pyx_k_RegionOr), 0, 0, 1, 1},
  {&__pyx_kp_u_RegionOr_line_335, __pyx_k_RegionOr_line_335, sizeof(__pyx_k_RegionOr_line_335), 0, 1, 0, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_asList, __pyx_k_asList, sizeof(__pyx_k_asList), 0, 0, 1, 1},
  {&__pyx_n_s_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 0, 1, 1},
  {&__pyx_n_s_check_type_of_list, __pyx_k_check_type_of_list, sizeof(__pyx_k_check_type_of_list), 0, 0, 1, 1},
  {&__pyx_n_s_child_region, __pyx_k_child_region, sizeof(__pyx_k_child_region), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_degree, __pyx_k_degree, sizeof(__pyx_k_degree), 0, 0, 1, 1},
  {&__pyx_n_s_degree1, __pyx_k_degree1, sizeof(__pyx_k_degree1), 0, 0, 1, 1},
  {&__pyx_n_s_degree2, __pyx_k_degree2, sizeof(__pyx_k_degree2), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_dx, __pyx_k_dx, sizeof(__pyx_k_dx), 0, 0, 1, 1},
  {&__pyx_n_s_dy, __pyx_k_dy, sizeof(__pyx_k_dy), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_height, __pyx_k_height, sizeof(__pyx_k_height), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_kp_s_input_array_has_a_wrong_shape, __pyx_k_input_array_has_a_wrong_shape, sizeof(__pyx_k_input_array_has_a_wrong_shape), 0, 0, 1, 0},
  {&__pyx_n_s_inside_x_y, __pyx_k_inside_x_y, sizeof(__pyx_k_inside_x_y), 0, 0, 1, 1},
  {&__pyx_n_s_inside_xy, __pyx_k_inside_xy, sizeof(__pyx_k_inside_xy), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_origin_x, __pyx_k_origin_x, sizeof(__pyx_k_origin_x), 0, 0, 1, 1},
  {&__pyx_n_s_origin_y, __pyx_k_origin_y, sizeof(__pyx_k_origin_y), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_pyregion__region_filter, __pyx_k_pyregion__region_filter, sizeof(__pyx_k_pyregion__region_filter), 0, 0, 1, 1},
  {&__pyx_kp_s_pyregion__region_filter_pyx, __pyx_k_pyregion__region_filter_pyx, sizeof(__pyx_k_pyregion__region_filter_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_kp_u_r_RegionAnd_reg1_reg2, __pyx_k_r_RegionAnd_reg1_reg2, sizeof(__pyx_k_r_RegionAnd_reg1_reg2), 0, 1, 0, 0},
  {&__pyx_kp_u_r_RegionOr_reg1_reg2, __pyx_k_r_RegionOr_reg1_reg2, sizeof(__pyx_k_r_RegionOr_reg1_reg2), 0, 1, 0, 0},
  {&__pyx_n_s_radius, __pyx_k_radius, sizeof(__pyx_k_radius), 0, 0, 1, 1},
  {&__pyx_n_s_radius_major, __pyx_k_radius_major, sizeof(__pyx_k_radius_major), 0, 0, 1, 1},
  {&__pyx_n_s_radius_minor, __pyx_k_radius_minor, sizeof(__pyx_k_radius_minor), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_region1, __pyx_k_region1, sizeof(__pyx_k_region1), 0, 0, 1, 1},
  {&__pyx_n_s_region1_list, __pyx_k_region1_list, sizeof(__pyx_k_region1_list), 0, 0, 1, 1},
  {&__pyx_n_s_region2, __pyx_k_region2, sizeof(__pyx_k_region2), 0, 0, 1, 1},
  {&__pyx_n_s_region2_list, __pyx_k_region2_list, sizeof(__pyx_k_region2_list), 0, 0, 1, 1},
  {&__pyx_kp_s_self_c_cannot_be_converted_to_a, __pyx_k_self_c_cannot_be_converted_to_a, sizeof(__pyx_k_self_c_cannot_be_converted_to_a), 0, 0, 1, 0},
  {&__pyx_kp_s_self_x_self_y_cannot_be_converte, __pyx_k_self_x_self_y_cannot_be_converte, sizeof(__pyx_k_self_x_self_y_cannot_be_converte), 0, 0, 1, 0},
  {&__pyx_n_s_set_context, __pyx_k_set_context, sizeof(__pyx_k_set_context), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_kp_s_shape_of_the_input_image_must_be, __pyx_k_shape_of_the_input_image_must_be, sizeof(__pyx_k_shape_of_the_input_image_must_be), 0, 0, 1, 0},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_kp_s_the_inut_needs_to_be_a_numpy_2_d, __pyx_k_the_inut_needs_to_be_a_numpy_2_d, sizeof(__pyx_k_the_inut_needs_to_be_a_numpy_2_d), 0, 0, 1, 0},
  {&__pyx_n_s_width, __pyx_k_width, sizeof(__pyx_k_width), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_xc, __pyx_k_xc, sizeof(__pyx_k_xc), 0, 0, 1, 1},
  {&__pyx_n_s_y, __pyx_k_y, sizeof(__pyx_k_y), 0, 0, 1, 1},
  {&__pyx_n_s_yc, __pyx_k_yc, sizeof(__pyx_k_yc), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(1, 170, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_self_c_cannot_be_converted_to_a); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "(tree fragment)":4
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.c cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_self_c_cannot_be_converted_to_a); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "pyregion/_region_filter.pyx":122
 *             shape = img_or_shape
 *         else:
 *             raise RegionFilterException("the inut needs to be a numpy 2-d array"             # <<<<<<<<<<<<<<
 *                                         " or a tuple of two integers")
 * 
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_the_inut_needs_to_be_a_numpy_2_d); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "pyregion/_region_filter.pyx":170
 *                 return self.inside_xy(x)
 *             else:
 *                 raise ValueError("input array has a wrong shape")             # <<<<<<<<<<<<<<
 *         else:
 *             return self.inside_x_y(x, y)
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_input_array_has_a_wrong_shape); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(1, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s_self_x_self_y_cannot_be_converte); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "(tree fragment)":4
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.x,self.y cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_kp_s_self_x_self_y_cannot_be_converte); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.m must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__30 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "pyregion/_region_filter.pyx":319
 *         return "And" + repr(self.child_regions)
 * 
 * def RegionAnd(RegionBase region1, RegionBase region2):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionAnd(reg1, reg2)
 */
  __pyx_tuple__31 = PyTuple_Pack(4, __pyx_n_s_region1, __pyx_n_s_region2, __pyx_n_s_region1_list, __pyx_n_s_region2_list); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(1, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyregion__region_filter_pyx, __pyx_n_s_RegionAnd, 319, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) __PYX_ERR(1, 319, __pyx_L1_error)

  /* "pyregion/_region_filter.pyx":335
 *     return RegionAndList(*(region1_list + region2_list))
 * 
 * def RegionOr(RegionBase region1, RegionBase region2):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionOr(reg1, reg2)
 */
  __pyx_tuple__33 = PyTuple_Pack(4, __pyx_n_s_region1, __pyx_n_s_region2, __pyx_n_s_region1_list, __pyx_n_s_region2_list); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(1, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);
  __pyx_codeobj__34 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__33, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_pyregion__region_filter_pyx, __pyx_n_s_RegionOr, 335, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__34)) __PYX_ERR(1, 335, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(1, 1, __pyx_L1_error);
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_region_filter(void); /*proto*/
PyMODINIT_FUNC init_region_filter(void)
#else
PyMODINIT_FUNC PyInit__region_filter(void); /*proto*/
PyMODINIT_FUNC PyInit__region_filter(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec__region_filter(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit__region_filter(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(1, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_region_filter", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(1, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(1, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(1, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pyregion___region_filter) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(1, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pyregion._region_filter")) {
      if (unlikely(PyDict_SetItemString(modules, "pyregion._region_filter", __pyx_m) < 0)) __PYX_ERR(1, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  __pyx_v_8pyregion_14_region_filter_metric_wcs = ((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)Py_None); Py_INCREF(Py_None);
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_vtabptr_8pyregion_14_region_filter_RegionContext = &__pyx_vtable_8pyregion_14_region_filter_RegionContext;
  __pyx_vtable_8pyregion_14_region_filter_RegionContext.set_update_func = (PyObject *(*)(struct __pyx_obj_8pyregion_14_region_filter_RegionContext *, int (*)(struct __pyx_t_8pyregion_14_region_filter_Metric *)))__pyx_f_8pyregion_14_region_filter_13RegionContext_set_update_func;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_RegionContext) < 0) __PYX_ERR(1, 39, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_RegionContext.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_RegionContext.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_RegionContext) < 0) __PYX_ERR(1, 39, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "RegionContext", (PyObject *)&__pyx_type_8pyregion_14_region_filter_RegionContext) < 0) __PYX_ERR(1, 39, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_RegionContext) < 0) __PYX_ERR(1, 39, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_RegionContext = &__pyx_type_8pyregion_14_region_filter_RegionContext;
  __pyx_vtabptr_8pyregion_14_region_filter_RegionBase = &__pyx_vtable_8pyregion_14_region_filter_RegionBase;
  __pyx_vtable_8pyregion_14_region_filter_RegionBase.update_metric = (PyObject *(*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *))__pyx_f_8pyregion_14_region_filter_10RegionBase_update_metric;
  __pyx_vtable_8pyregion_14_region_filter_RegionBase.metric_set_origin = (PyObject *(*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double, struct __pyx_obj_8pyregion_14_region_filter_RegionContext *))__pyx_f_8pyregion_14_region_filter_10RegionBase_metric_set_origin;
  __pyx_vtable_8pyregion_14_region_filter_RegionBase._inside = (npy_bool (*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double))__pyx_f_8pyregion_14_region_filter_10RegionBase__inside;
  __pyx_vtable_8pyregion_14_region_filter_RegionBase._mask = (PyArrayObject *(*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, npy_intp, npy_intp))__pyx_f_8pyregion_14_region_filter_10RegionBase__mask;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_RegionBase) < 0) __PYX_ERR(1, 68, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_RegionBase.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_RegionBase.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_RegionBase) < 0) __PYX_ERR(1, 68, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "RegionBase", (PyObject *)&__pyx_type_8pyregion_14_region_filter_RegionBase) < 0) __PYX_ERR(1, 68, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_RegionBase) < 0) __PYX_ERR(1, 68, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_RegionBase = &__pyx_type_8pyregion_14_region_filter_RegionBase;
  __pyx_vtabptr_8pyregion_14_region_filter_RegionNot = &__pyx_vtable_8pyregion_14_region_filter_RegionNot;
  __pyx_vtable_8pyregion_14_region_filter_RegionNot.__pyx_base = *__pyx_vtabptr_8pyregion_14_region_filter_RegionBase;
  __pyx_vtable_8pyregion_14_region_filter_RegionNot.__pyx_base._inside = (npy_bool (*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double))__pyx_f_8pyregion_14_region_filter_9RegionNot__inside;
  __pyx_type_8pyregion_14_region_filter_RegionNot.tp_base = __pyx_ptype_8pyregion_14_region_filter_RegionBase;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_RegionNot) < 0) __PYX_ERR(1, 237, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_RegionNot.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_RegionNot.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_RegionNot) < 0) __PYX_ERR(1, 237, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "RegionNot", (PyObject *)&__pyx_type_8pyregion_14_region_filter_RegionNot) < 0) __PYX_ERR(1, 237, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_RegionNot) < 0) __PYX_ERR(1, 237, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_RegionNot = &__pyx_type_8pyregion_14_region_filter_RegionNot;
  __pyx_vtabptr_8pyregion_14_region_filter_RegionList = &__pyx_vtable_8pyregion_14_region_filter_RegionList;
  __pyx_vtable_8pyregion_14_region_filter_RegionList.__pyx_base = *__pyx_vtabptr_8pyregion_14_region_filter_RegionBase;
  __pyx_type_8pyregion_14_region_filter_RegionList.tp_base = __pyx_ptype_8pyregion_14_region_filter_RegionBase;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_RegionList) < 0) __PYX_ERR(1, 249, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_RegionList.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_RegionList.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_RegionList) < 0) __PYX_ERR(1, 249, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "RegionList", (PyObject *)&__pyx_type_8pyregion_14_region_filter_RegionList) < 0) __PYX_ERR(1, 249, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_RegionList) < 0) __PYX_ERR(1, 249, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_RegionList = &__pyx_type_8pyregion_14_region_filter_RegionList;
  __pyx_vtabptr_8pyregion_14_region_filter_RegionOrList = &__pyx_vtable_8pyregion_14_region_filter_RegionOrList;
  __pyx_vtable_8pyregion_14_region_filter_RegionOrList.__pyx_base = *__pyx_vtabptr_8pyregion_14_region_filter_RegionList;
  __pyx_vtable_8pyregion_14_region_filter_RegionOrList.__pyx_base.__pyx_base._inside = (npy_bool (*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double))__pyx_f_8pyregion_14_region_filter_12RegionOrList__inside;
  __pyx_type_8pyregion_14_region_filter_RegionOrList.tp_base = __pyx_ptype_8pyregion_14_region_filter_RegionList;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_RegionOrList) < 0) __PYX_ERR(1, 282, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_RegionOrList.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_RegionOrList.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_RegionOrList) < 0) __PYX_ERR(1, 282, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "RegionOrList", (PyObject *)&__pyx_type_8pyregion_14_region_filter_RegionOrList) < 0) __PYX_ERR(1, 282, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_RegionOrList) < 0) __PYX_ERR(1, 282, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_RegionOrList = &__pyx_type_8pyregion_14_region_filter_RegionOrList;
  __pyx_vtabptr_8pyregion_14_region_filter_RegionAndList = &__pyx_vtable_8pyregion_14_region_filter_RegionAndList;
  __pyx_vtable_8pyregion_14_region_filter_RegionAndList.__pyx_base = *__pyx_vtabptr_8pyregion_14_region_filter_RegionList;
  __pyx_vtable_8pyregion_14_region_filter_RegionAndList.__pyx_base.__pyx_base._inside = (npy_bool (*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double))__pyx_f_8pyregion_14_region_filter_13RegionAndList__inside;
  __pyx_type_8pyregion_14_region_filter_RegionAndList.tp_base = __pyx_ptype_8pyregion_14_region_filter_RegionList;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_RegionAndList) < 0) __PYX_ERR(1, 300, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_RegionAndList.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_RegionAndList.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_RegionAndList) < 0) __PYX_ERR(1, 300, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "RegionAndList", (PyObject *)&__pyx_type_8pyregion_14_region_filter_RegionAndList) < 0) __PYX_ERR(1, 300, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_RegionAndList) < 0) __PYX_ERR(1, 300, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_RegionAndList = &__pyx_type_8pyregion_14_region_filter_RegionAndList;
  __pyx_vtabptr_8pyregion_14_region_filter_Transform = &__pyx_vtable_8pyregion_14_region_filter_Transform;
  __pyx_vtable_8pyregion_14_region_filter_Transform.__pyx_base = *__pyx_vtabptr_8pyregion_14_region_filter_RegionBase;
  __pyx_vtable_8pyregion_14_region_filter_Transform.__pyx_base._inside = (npy_bool (*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double))__pyx_f_8pyregion_14_region_filter_9Transform__inside;
  __pyx_vtable_8pyregion_14_region_filter_Transform._transform = (int (*)(struct __pyx_obj_8pyregion_14_region_filter_Transform *, double, double, double *, double *))__pyx_f_8pyregion_14_region_filter_9Transform__transform;
  __pyx_type_8pyregion_14_region_filter_Transform.tp_base = __pyx_ptype_8pyregion_14_region_filter_RegionBase;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_Transform) < 0) __PYX_ERR(1, 351, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_Transform.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_Transform.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_Transform) < 0) __PYX_ERR(1, 351, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Transform", (PyObject *)&__pyx_type_8pyregion_14_region_filter_Transform) < 0) __PYX_ERR(1, 351, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_Transform) < 0) __PYX_ERR(1, 351, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_Transform = &__pyx_type_8pyregion_14_region_filter_Transform;
  __pyx_vtabptr_8pyregion_14_region_filter_Rotated = &__pyx_vtable_8pyregion_14_region_filter_Rotated;
  __pyx_vtable_8pyregion_14_region_filter_Rotated.__pyx_base = *__pyx_vtabptr_8pyregion_14_region_filter_Transform;
  __pyx_vtable_8pyregion_14_region_filter_Rotated.__pyx_base._transform = (int (*)(struct __pyx_obj_8pyregion_14_region_filter_Transform *, double, double, double *, double *))__pyx_f_8pyregion_14_region_filter_7Rotated__transform;
  __pyx_type_8pyregion_14_region_filter_Rotated.tp_base = __pyx_ptype_8pyregion_14_region_filter_Transform;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_Rotated) < 0) __PYX_ERR(1, 383, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_Rotated.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_Rotated.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_Rotated) < 0) __PYX_ERR(1, 383, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Rotated", (PyObject *)&__pyx_type_8pyregion_14_region_filter_Rotated) < 0) __PYX_ERR(1, 383, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_Rotated) < 0) __PYX_ERR(1, 383, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_Rotated = &__pyx_type_8pyregion_14_region_filter_Rotated;
  __pyx_vtabptr_8pyregion_14_region_filter_Translated = &__pyx_vtable_8pyregion_14_region_filter_Translated;
  __pyx_vtable_8pyregion_14_region_filter_Translated.__pyx_base = *__pyx_vtabptr_8pyregion_14_region_filter_Transform;
  __pyx_vtable_8pyregion_14_region_filter_Translated.__pyx_base._transform = (int (*)(struct __pyx_obj_8pyregion_14_region_filter_Transform *, double, double, double *, double *))__pyx_f_8pyregion_14_region_filter_10Translated__transform;
  __pyx_type_8pyregion_14_region_filter_Translated.tp_base = __pyx_ptype_8pyregion_14_region_filter_Transform;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_Translated) < 0) __PYX_ERR(1, 426, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_Translated.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_Translated.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_Translated) < 0) __PYX_ERR(1, 426, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Translated", (PyObject *)&__pyx_type_8pyregion_14_region_filter_Translated) < 0) __PYX_ERR(1, 426, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_Translated) < 0) __PYX_ERR(1, 426, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_Translated = &__pyx_type_8pyregion_14_region_filter_Translated;
  __pyx_vtabptr_8pyregion_14_region_filter_Circle = &__pyx_vtable_8pyregion_14_region_filter_Circle;
  __pyx_vtable_8pyregion_14_region_filter_Circle.__pyx_base = *__pyx_vtabptr_8pyregion_14_region_filter_RegionBase;
  __pyx_vtable_8pyregion_14_region_filter_Circle.__pyx_base._inside = (npy_bool (*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double))__pyx_f_8pyregion_14_region_filter_6Circle__inside;
  __pyx_vtable_8pyregion_14_region_filter_Circle._set_v = (PyObject *(*)(struct __pyx_obj_8pyregion_14_region_filter_Circle *, double, double, double))__pyx_f_8pyregion_14_region_filter_6Circle__set_v;
  __pyx_vtable_8pyregion_14_region_filter_Circle._get_v = (PyObject *(*)(struct __pyx_obj_8pyregion_14_region_filter_Circle *))__pyx_f_8pyregion_14_region_filter_6Circle__get_v;
  __pyx_type_8pyregion_14_region_filter_Circle.tp_base = __pyx_ptype_8pyregion_14_region_filter_RegionBase;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_Circle) < 0) __PYX_ERR(1, 449, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_Circle.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_Circle.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_Circle) < 0) __PYX_ERR(1, 449, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Circle", (PyObject *)&__pyx_type_8pyregion_14_region_filter_Circle) < 0) __PYX_ERR(1, 449, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_Circle) < 0) __PYX_ERR(1, 449, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_Circle = &__pyx_type_8pyregion_14_region_filter_Circle;
  __pyx_vtabptr_8pyregion_14_region_filter_Ellipse = &__pyx_vtable_8pyregion_14_region_filter_Ellipse;
  __pyx_vtable_8pyregion_14_region_filter_Ellipse.__pyx_base = *__pyx_vtabptr_8pyregion_14_region_filter_RegionBase;
  __pyx_vtable_8pyregion_14_region_filter_Ellipse.__pyx_base._inside = (npy_bool (*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double))__pyx_f_8pyregion_14_region_filter_7Ellipse__inside;
  __pyx_type_8pyregion_14_region_filter_Ellipse.tp_base = __pyx_ptype_8pyregion_14_region_filter_RegionBase;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_Ellipse) < 0) __PYX_ERR(1, 484, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_Ellipse.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_Ellipse.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_Ellipse) < 0) __PYX_ERR(1, 484, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Ellipse", (PyObject *)&__pyx_type_8pyregion_14_region_filter_Ellipse) < 0) __PYX_ERR(1, 484, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_Ellipse) < 0) __PYX_ERR(1, 484, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_Ellipse = &__pyx_type_8pyregion_14_region_filter_Ellipse;
  __pyx_vtabptr_8pyregion_14_region_filter_Box = &__pyx_vtable_8pyregion_14_region_filter_Box;
  __pyx_vtable_8pyregion_14_region_filter_Box.__pyx_base = *__pyx_vtabptr_8pyregion_14_region_filter_RegionBase;
  __pyx_vtable_8pyregion_14_region_filter_Box.__pyx_base._inside = (npy_bool (*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double))__pyx_f_8pyregion_14_region_filter_3Box__inside;
  __pyx_type_8pyregion_14_region_filter_Box.tp_base = __pyx_ptype_8pyregion_14_region_filter_RegionBase;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_Box) < 0) __PYX_ERR(1, 525, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_Box.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_Box.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_Box) < 0) __PYX_ERR(1, 525, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Box", (PyObject *)&__pyx_type_8pyregion_14_region_filter_Box) < 0) __PYX_ERR(1, 525, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_Box) < 0) __PYX_ERR(1, 525, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_Box = &__pyx_type_8pyregion_14_region_filter_Box;
  __pyx_vtabptr_8pyregion_14_region_filter_Polygon = &__pyx_vtable_8pyregion_14_region_filter_Polygon;
  __pyx_vtable_8pyregion_14_region_filter_Polygon.__pyx_base = *__pyx_vtabptr_8pyregion_14_region_filter_RegionBase;
  __pyx_vtable_8pyregion_14_region_filter_Polygon.__pyx_base._inside = (npy_bool (*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double))__pyx_f_8pyregion_14_region_filter_7Polygon__inside;
  __pyx_type_8pyregion_14_region_filter_Polygon.tp_base = __pyx_ptype_8pyregion_14_region_filter_RegionBase;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_Polygon) < 0) __PYX_ERR(1, 555, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_Polygon.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_Polygon.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_Polygon) < 0) __PYX_ERR(1, 555, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Polygon", (PyObject *)&__pyx_type_8pyregion_14_region_filter_Polygon) < 0) __PYX_ERR(1, 555, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_Polygon) < 0) __PYX_ERR(1, 555, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_Polygon = &__pyx_type_8pyregion_14_region_filter_Polygon;
  __pyx_vtabptr_8pyregion_14_region_filter_AngleRange = &__pyx_vtable_8pyregion_14_region_filter_AngleRange;
  __pyx_vtable_8pyregion_14_region_filter_AngleRange.__pyx_base = *__pyx_vtabptr_8pyregion_14_region_filter_RegionBase;
  __pyx_vtable_8pyregion_14_region_filter_AngleRange.__pyx_base._inside = (npy_bool (*)(struct __pyx_obj_8pyregion_14_region_filter_RegionBase *, double, double))__pyx_f_8pyregion_14_region_filter_10AngleRange__inside;
  __pyx_vtable_8pyregion_14_region_filter_AngleRange._fix_angle = (double (*)(struct __pyx_obj_8pyregion_14_region_filter_AngleRange *, double))__pyx_f_8pyregion_14_region_filter_10AngleRange__fix_angle;
  __pyx_type_8pyregion_14_region_filter_AngleRange.tp_base = __pyx_ptype_8pyregion_14_region_filter_RegionBase;
  if (PyType_Ready(&__pyx_type_8pyregion_14_region_filter_AngleRange) < 0) __PYX_ERR(1, 616, __pyx_L1_error)
  __pyx_type_8pyregion_14_region_filter_AngleRange.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_8pyregion_14_region_filter_AngleRange.tp_dict, __pyx_vtabptr_8pyregion_14_region_filter_AngleRange) < 0) __PYX_ERR(1, 616, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "AngleRange", (PyObject *)&__pyx_type_8pyregion_14_region_filter_AngleRange) < 0) __PYX_ERR(1, 616, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_8pyregion_14_region_filter_AngleRange) < 0) __PYX_ERR(1, 616, __pyx_L1_error)
  __pyx_ptype_8pyregion_14_region_filter_AngleRange = &__pyx_type_8pyregion_14_region_filter_AngleRange;
  /*--- Type import code ---*/
  __pyx_ptype_8pyregion_7c_numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_8pyregion_7c_numpy_dtype)) __PYX_ERR(2, 77, __pyx_L1_error)
  __pyx_ptype_8pyregion_7c_numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_8pyregion_7c_numpy_ndarray)) __PYX_ERR(2, 82, __pyx_L1_error)
  __pyx_ptype_8pyregion_7c_numpy_flatiter = __Pyx_ImportType("numpy", "flatiter", sizeof(PyArrayIterObject), 0); if (unlikely(!__pyx_ptype_8pyregion_7c_numpy_flatiter)) __PYX_ERR(2, 91, __pyx_L1_error)
  __pyx_ptype_8pyregion_7c_numpy_broadcast = __Pyx_ImportType("numpy", "broadcast", sizeof(PyArrayMultiIterObject), 0); if (unlikely(!__pyx_ptype_8pyregion_7c_numpy_broadcast)) __PYX_ERR(2, 97, __pyx_L1_error)
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif

  /* "pyregion/_region_filter.pyx":13
 * cimport c_python
 * 
 * c_numpy.import_array()             # <<<<<<<<<<<<<<
 * 
 * ctypedef int Py_ssize_t
 */
  import_array();

  /* "pyregion/_region_filter.pyx":18
 * 
 * 
 * class NotYetImplemented(Exception):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_NotYetImplemented, __pyx_n_s_NotYetImplemented, (PyObject *) NULL, __pyx_n_s_pyregion__region_filter, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_NotYetImplemented, __pyx_t_1, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_NotYetImplemented, __pyx_t_4) < 0) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":22
 * 
 * 
 * class RegionFilterException(Exception):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_RegionFilterException, __pyx_n_s_RegionFilterException, (PyObject *) NULL, __pyx_n_s_pyregion__region_filter, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_RegionFilterException, __pyx_t_1, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RegionFilterException, __pyx_t_4) < 0) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":60
 * 
 * cdef RegionContext metric_wcs
 * metric_wcs = RegionContext()             # <<<<<<<<<<<<<<
 * metric_wcs.set_update_func(_update_metric_wcs)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_8pyregion_14_region_filter_RegionContext), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(((PyObject *)__pyx_v_8pyregion_14_region_filter_metric_wcs));
  __Pyx_DECREF_SET(__pyx_v_8pyregion_14_region_filter_metric_wcs, ((struct __pyx_obj_8pyregion_14_region_filter_RegionContext *)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":61
 * cdef RegionContext metric_wcs
 * metric_wcs = RegionContext()
 * metric_wcs.set_update_func(_update_metric_wcs)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_8pyregion_14_region_filter_RegionContext *)__pyx_v_8pyregion_14_region_filter_metric_wcs->__pyx_vtab)->set_update_func(__pyx_v_8pyregion_14_region_filter_metric_wcs, __pyx_f_8pyregion_14_region_filter__update_metric_wcs); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":64
 * 
 * 
 * class BaseClassInitException(Exception):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_BaseClassInitException, __pyx_n_s_BaseClassInitException, (PyObject *) NULL, __pyx_n_s_pyregion__region_filter, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_BaseClassInitException, __pyx_t_1, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BaseClassInitException, __pyx_t_4) < 0) __PYX_ERR(1, 64, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":319
 *         return "And" + repr(self.child_regions)
 * 
 * def RegionAnd(RegionBase region1, RegionBase region2):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionAnd(reg1, reg2)
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_8pyregion_14_region_filter_1RegionAnd, NULL, __pyx_n_s_pyregion__region_filter); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RegionAnd, __pyx_t_1) < 0) __PYX_ERR(1, 319, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":335
 *     return RegionAndList(*(region1_list + region2_list))
 * 
 * def RegionOr(RegionBase region1, RegionBase region2):             # <<<<<<<<<<<<<<
 *     """
 *     >>> r = RegionOr(reg1, reg2)
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_8pyregion_14_region_filter_3RegionOr, NULL, __pyx_n_s_pyregion__region_filter); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RegionOr, __pyx_t_1) < 0) __PYX_ERR(1, 335, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pyregion/_region_filter.pyx":1
 * cdef extern from "stdio.h":             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_kp_u_RegionAnd_line_319, __pyx_kp_u_r_RegionAnd_reg1_reg2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_kp_u_RegionOr_line_335, __pyx_kp_u_r_RegionOr_reg1_reg2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pyregion._region_filter", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pyregion._region_filter");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* GetModuleGlobalName */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* PyCFunctionFastCall */
  #if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
  #if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* ArgTypeTest */
    static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* GetAttr */
    static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* HasAttr */
    static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* ExtTypeTest */
    static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* RaiseDoubleKeywords */
    static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
    static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyIntBinop */
    #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) {
            Py_RETURN_TRUE;
        } else {
            Py_RETURN_FALSE;
        }
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                #if PyLong_SHIFT < 30 && PyLong_SHIFT != 15
                default: return PyLong_Type.tp_richcompare(op1, op2, Py_EQ);
                #else
                default: Py_RETURN_FALSE;
                #endif
            }
        }
            if (a == b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            if ((double)a == (double)b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    return PyObject_RichCompare(op1, op2, Py_EQ);
}
#endif

/* GetItemInt */
    static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* SetItemInt */
    static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely((n >= 0) & (n < PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o))) {
#else
    if (is_list || PySequence_Check(o)) {
#endif
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* DelItemInt */
      static int __Pyx_DelItem_Generic(PyObject *o, PyObject *j) {
    int r;
    if (!j) return -1;
    r = PyObject_DelItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_DelItemInt_Fast(PyObject *o, Py_ssize_t i,
                                               CYTHON_UNUSED int is_list, CYTHON_NCP_UNUSED int wraparound) {
#if !CYTHON_USE_TYPE_SLOTS
    if (is_list || PySequence_Check(o)) {
        return PySequence_DelItem(o, i);
    }
#else
    PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
    if (likely(m && m->sq_ass_item)) {
        if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
            Py_ssize_t l = m->sq_length(o);
            if (likely(l >= 0)) {
                i += l;
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    return -1;
                PyErr_Clear();
            }
        }
        return m->sq_ass_item(o, i, (PyObject *)NULL);
    }
#endif
    return __Pyx_DelItem_Generic(o, PyInt_FromSsize_t(i));
}

/* WriteUnraisableException */
      static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* SetVTable */
      static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* SetupReduce */
      static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* CalculateMetaclass */
      static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* Py3ClassCreate */
      static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
      static int __Pyx_CLineForTraceback(int c_line) {
#ifdef CYTHON_CLINE_IN_TRACEBACK
    return ((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0;
#else
    PyObject *use_cline;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *ptype, *pvalue, *ptraceback;
      PyObject *use_cline_obj;
      PyErr_Fetch(&ptype, &pvalue, &ptraceback);
      use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        use_cline = NULL;
      }
      PyErr_Restore(ptype, pvalue, ptraceback);
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    return c_line;
#endif
}

/* CodeObjectCache */
      static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
      #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        __Pyx_PyThreadState_Current, /*PyThreadState *tstate,*/
        py_code,                     /*PyCodeObject *code,*/
        __pyx_d,              /*PyObject *globals,*/
        0                            /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
      #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
      static CYTHON_INLINE PyObject* __Pyx_PyInt_From_npy_bool(npy_bool value) {
    const npy_bool neg_one = (npy_bool) -1, const_zero = (npy_bool) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(npy_bool) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(npy_bool) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(npy_bool) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(npy_bool) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(npy_bool) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(npy_bool),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
      static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
      static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
      static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
      #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
      static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
      #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
      #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* InitStrings */
      static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
